/*
	SDL - Simple DirectMedia Layer
	Copyright (C) 1997, 1998, 1999, 2000, 2001  Sam Lantinga

	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Library General Public
	License as published by the Free Software Foundation; either
	version 2 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Library General Public License for more details.

	You should have received a copy of the GNU Library General Public
	License along with this library; if not, write to the Free
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	Sam Lantinga
	slouken@devolution.com

	This file hacked^H^H^H^H^H^Hwritten by Ryan C. Gordon
	    (icculus@linuxgames.com)

***
	Copyright (C) SEIKO EPSON CORPORATION 2002-2004. All Rights Reserved.

	3-3-5, Owa, Suwa-shi Nagano-ken, 392-8502 Japan
	Phone: +81-266-52-3131
*/

/* Output raw audio data to a file. */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <unistd.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <assert.h>


#include "SDL_audio.h"
#include "SDL_error.h"
#include "SDL_audiomem.h"
#include "SDL_audio_c.h"
#include "SDL_timer.h"
#include "SDL_aclaudio.h"


/* states of UP/DOWN    */
#define CLOSED           0x00000000
#define ACL_OPEN         0x00000001
#define ACLINK_OPEN      0x00000002
#define AC97_OPEN        0x00000004
#define AM_OPEN          0x00000008
#define AP_OPEN          0x00000010
#define SI_OPEN          0x00000020
#define I2SO_OPEN        0x00000040
#define ACLINK_CONN_AC97 0x00000080
#define AM_CONN_ACLINK   0x00000100
#define AM_CONN_I2SO     0x00000200
#define AP_CONN_ACLINK   0x00000400
#define SI_CONN_AM       0x00000800
#define SI_CONN_AP       0x00001000
#define BUFF_ALLOC       0x00002000
#define CALLBACK         0x00004000
#define AC97_STARTED     0x00008000
#define ACLINK_STARTED   0x00010000
#define AM_STARTED       0x00020000
#define AP_STARTED       0x00040000
#define SI_STARTED       0x00080000
#define I2SO_STARTED     0x00100000
#define SPD_OPEN         0x00200000
#define SPD_CONN_AM      0x00400000
#define SPD_STARTED      0x00800000
#define MUTEX_OPEN	 0x01000000


#define OK   0
#define BAD -1

#define SI_NAME         2
#define ACLINK_NAME     ACL_ACLNK_0
#define AC97_NAME       ACL_AC97_CS4205_00

#define CHUNK_SAMPLES			2400
#define XILLEON_I2SO_FREQ       48000
#define XILLEON_AUDIOCHANNELS   2

//#define MAX_BUFS       			12

#define T_DEBUG

#ifdef T_DEBUG
#define DEBUGF(args...)	fprintf(stderr, args)
#else
#define DEBUGF(args...)
#endif


static int setup(struct SDL_PrivateAudioData *cp);
static int open_devices(struct SDL_PrivateAudioData *cp);
static int connect_devices(struct SDL_PrivateAudioData *cp);
static int init_devices(struct SDL_PrivateAudioData *cp);
static int setup_buffers(struct SDL_PrivateAudioData *cp);
static int start(struct SDL_PrivateAudioData *cp);
static int shutdown(struct SDL_PrivateAudioData *cp);
static int stop(struct SDL_PrivateAudioData *cp);
static int disconnect_devices(struct SDL_PrivateAudioData *cp);
static int close_devices(struct SDL_PrivateAudioData *cp);



int setup(struct SDL_PrivateAudioData *cp)
{
	int result = OK;
	ACL_RESULT rc;

	cp->state = CLOSED;

	/* open the Acl */
	rc = Acl_Open(0, &cp->chip);
	if (ACL_ERROR(rc)) {
		return BAD;
	} else {
		cp->state |= ACL_OPEN;
	}

	rc = Acl_SetDebugLevel(cp->chip, ACL_COMPONENT_MAX,
						   ACL_DEBUGLEVEL_Verbose, NULL);
	if (ACL_ERROR(rc)) {
		DEBUGF("Could not set the debug level!!\n");
		return BAD;
	}

	result = open_devices(cp);
	if (result == BAD)
		return BAD;

	result = init_devices(cp);
	if (result == BAD)
		return BAD;

	result = connect_devices(cp);
	if (result == BAD)
		return BAD;

	result = setup_buffers(cp);
	if (result == BAD)
		return BAD;

	// debug output is usually really profuse after this point
	// and pretty much useless so we want to reduce it.
	//AtiCore_DbgPortSelect ( DBG_PORT_BUF_WRAP ) ; 
	Acl_SetDebugLevel(cp->chip, ACL_COMPONENT_MAX, ACL_DEBUGLEVEL_Error,
					  NULL);
	//...but since we dont know what the fuck is going on we need to trace!
	Acl_SetDebugLevel(cp->chip, ACL_NULL, ACL_DEBUGLEVEL_Verbose, NULL);
	return OK;
}



/*****************************************************************************/
void
BufferEmptyCallback(ACL_HANDLE si, ACL_SI_BUFFER_TYPE bt, UINT32 irq_cnt,
					UINT32 cntx)
{
	struct SDL_PrivateAudioData *priv = (struct SDL_PrivateAudioData *)cntx;
	if (priv == NULL) {
		fprintf(stderr,
				"BufferEmptyCallback() was passed a NULL context pointer!! \n");
		//there is nothing we can do if this happens!!
		return;
	}
//	DEBUGF("buffCb-->\n");

	SDL_LockMutex(priv->mutex);
	priv->prodConDelta--;
	SDL_UnlockMutex(priv->mutex);

//	DEBUGF("<--buffCb\n");
}


/*****************************************************************************/

//#define BUF_SIZE        (1024*512)
//#define BUF_SIZE      (4096*16)                    !!! this one !!!!
//#define BUF_SIZE        (1024*256)

//8 X 32k

//30 X 4k 


inline static ACL_RESULT
memory_alloc(ACL_HANDLE instance, ACL_SI_BUFFER_INFO *info, size_t size)
{
	 UINT32 align, channel = MEM_CHAN_0_1, offset, length;
	 ACL_RESULT rc;

	assert(instance);
	assert(info);

	channel = MEM_CHAN_0_1;

	rc = AclSi_QueryAlignments(instance, &align, &length);
	if (ACL_ERROR(rc)) {
		AtiCore_DbgPrintf("Error AclSi_QueryAlignments(0x%X)\n", rc);
		return rc;
	}

	DEBUGF("allocate %dbytes\n", size);
	info->Address.pBuffer = (unsigned char *) AtiCore_VideoMemAlloc(0,
			&channel, size, align, &offset, ALLOC_TYPE_NORM_GUI);
	if (info->Address.pBuffer == NULL) {
		AtiCore_DbgPrintf("Error allocation memory\n");
		return rc;
	}

	memset(info->Address.pBuffer, 0, size);

	info->Address.Offset = offset;
	info->Reuse		= FALSE;
	info->Enable	= TRUE;
	info->Length	= size;
	info->Position	= size - 1;

	return ACL_OK;
}


#ifdef USE_TRANSFORM
int setup_transform(struct SDL_PrivateAudioData *priv)
{
	 ACL_SI_TRANSFORM_INFO si_info;
	 ACL_RESULT rc;
	 int i;

	memset(&priv->convbuf, 0, sizeof(priv->convbuf));

	si_info.SRCInfo.InputRate = priv->srcSpec.freq;
	si_info.SRCInfo.OutputRate = 48000;
	si_info.SRCInfo.NumberOfChannels = 2;
	si_info.SRCInfo.InputResolution = priv->srcSpec.format & 0xff;
	si_info.SRCInfo.OutputResolution = 16;

	DEBUGF("InputRate: %d OutputRate: %d InputResolution: %d\n",
		si_info.SRCInfo.InputRate, si_info.SRCInfo.OutputRate,
		si_info.SRCInfo.InputResolution);

	rc = AclSi_SetDataTransform(priv->si,
		priv->srcSpec.freq == 48000 ? ACL_SI_TRANSFORM_NONE :
										 ACL_SI_TRANSFORM_SRC);
	if (ACL_ERROR(rc)) {
		AtiCore_DbgPrintf("Error AclSi_SetDataTransform(0x%X)\n", rc);
		return BAD;
	}

	if (priv->srcSpec.freq == 48000) {
		fprintf(stderr, "here\n");
		return OK;
	}

	rc = AclSi_SetInitTransformOptions(priv->si,
			ACL_SI_SRC_FILTER_SMALL, &si_info);
	if (ACL_ERROR(rc)) {
		AtiCore_DbgPrintf("Error AclSi_SetInitTransformOptions(0x%X)\n", rc);
		return BAD;
	}

	for (i = 0; i < CONV_MAX_BUFS; i++) {
		rc = memory_alloc(priv->si, &priv->convbuf[i], CONV_BUF_SIZE);
		if (ACL_ERROR(rc)) {
			return BAD;
		}
	}

	rc = AclSi_SetTransformBuffers(priv->si, ACL_SI_MEMORY_VIDEO,
			CONV_MAX_BUFS, priv->convbuf);
	if (ACL_ERROR(rc)) {
		AtiCore_DbgPrintf("Error AclSi_SetTransformBuffer(0x%X)\n", rc);
		return BAD;
	}

	return OK;
}
#endif // USE_TRANSFORM


int setup_buffers(struct SDL_PrivateAudioData *cp)
{
	ACL_RESULT rc;

#ifdef USE_TRANSFORM
	setup_transform(cp);
#else
	rc = AclSi_SetDataTransform(cp->si, ACL_SI_TRANSFORM_NONE);
	if (ACL_ERROR(rc)) {
		AtiCore_DbgPrintf("Error AclSi_SetDataTransform(0x%X)\n", rc);
		return BAD;
	}
#endif

	rc = AclSi_SetMemoryType(cp->si, ACL_SI_MEMORY_VIDEO);
	if (ACL_ERROR(rc)) {
		AtiCore_DbgPrintf("Error AclSi_SetMemoryType(0x%X)\n", rc);
		return BAD;
	}

	cp->bufchain =
		(ACL_SI_BUFFER_INFO *)calloc(sizeof(ACL_SI_BUFFER_INFO), MAX_BUFS);
	if (!cp->bufchain) {
		AtiCore_DbgPrintf("Cant alloc memory for buffer chain!\n");
		return BAD;
	}

	/* Fill buffers */
	cp->numChainElem = 0;
	cp->curChainElem = 0;
	while (cp->numChainElem < MAX_BUFS) {
#ifdef USE_TRANSFORM
		rc = memory_alloc(cp->si, &cp->bufchain[cp->numChainElem], cp->srcSpec.size);
#else
		rc = memory_alloc(cp->si, &cp->bufchain[cp->numChainElem], cp->spec->size);
#endif
		if (ACL_ERROR(rc)) {
			return BAD;
		}
		cp->numChainElem++;
	}

	cp->state |= BUFF_ALLOC;

	rc = AclSi_AppendBuffer(cp->si, MAX_BUFS, cp->bufchain);
	if (ACL_ERROR(rc)) {
		AtiCore_DbgPrintf("Error AclSi_AppendBuffer(0x%X)\n", rc);
		return BAD;
	}

	return OK;
}

/*****************************************************************************/

int shutdown(struct SDL_PrivateAudioData *cp)
{
	int result = OK;
	ACL_RESULT rc;

	AtiCore_DbgPrintf("stop(cp);");
	stop(cp);

	AtiCore_DbgPrintf("disconnect_devices(cp)");
	disconnect_devices(cp);

	AtiCore_DbgPrintf("close_devices(cp);");
	close_devices(cp);

	AtiCore_DbgPrintf("Acl_Close(cp->chip);");
	if (cp->state & ACL_OPEN) {
		rc = Acl_Close(cp->chip);
		if (ACL_ERROR(rc)) {
			AtiCore_DbgPrintf("Error Acl_Close(0x%X)\n", rc);
			result = BAD;
		} else {
			cp->state &= ~ACL_OPEN;
		}
	}
	AtiCore_DbgPrintf("YOU HAVE cp->state == %x !!! \n", cp->state);
	return (result);
}

/*****************************************************************************/
// everything above this was path independent
// everything below this is path dependent
//(ie must be careful of USE_AM and USE_AP)
/*****************************************************************************/
int open_devices(struct SDL_PrivateAudioData *cp)
{
	BOOL bExist;
	int result = OK;
	ACL_RESULT rc;

	/* Open the stream interface */
	if (result == OK) {
		rc = AclSi_Open(&cp->si, cp->chip, SI_NAME, ACL_MODE_EXCLUSIVE,
						&bExist, ACL_SI_BUFFER_READ);
		if (ACL_ERROR(rc)) {
			AtiCore_DbgPrintf("Error AclSi_Open(0x%X)\n", rc);
			result = BAD;
		} else {
			cp->state |= SI_OPEN;
		}
	}
#ifdef USE_AP

	/* Open the AcLink */
	if (result == OK) {
		rc = AclAclnk_Open(&cp->aclink, cp->chip, ACLINK_NAME,
						   ACL_MODE_SHARED, &bExist);
		if (ACL_ERROR(rc)) {
			AtiCore_DbgPrintf("Error AclAclnk_Open(0x%X)\n", rc);
			result = BAD;
		} else {
			cp->state |= ACLINK_OPEN;
		}
	}

	/* Open the Ac97 */
	if (result == OK) {
		rc = AclAc97_Open(&cp->ac97, cp->chip, AC97_NAME,
						  ACL_MODE_SHARED, &bExist, cp->aclink);
		if (ACL_ERROR(rc)) {
			AtiCore_DbgPrintf("Error AclAc97_Open(0x%X)\n", rc);
			result = BAD;
		} else {
			cp->state |= AC97_OPEN;
		}
	}

	/* Open the Audio Parser */
	if (result == OK) {
		rc = AclAp_Open(&cp->ap, cp->chip, ACL_AP_PRIMARY,
						ACL_MODE_SHARED, &bExist);
		if (ACL_ERROR(rc)) {
			AtiCore_DbgPrintf("Error AclAp_Open(0x%X)\n", rc);
			result = BAD;
		} else {
			cp->state |= AP_OPEN;
		}
	}
#endif							//USE_AP


#ifdef USE_AM
	/* Open the i2so */
	if (result == OK) {
		rc = AclI2SO_Open(&cp->i2so, cp->chip, ACL_I2SO_PRIMARY,
						  ACL_MODE_SHARED, &bExist);

		if (ACL_ERROR(rc)) {
			AtiCore_DbgPrintf("Error AclI2SO_Open(0x%X)\n", rc);
			result = BAD;
		} else {
			cp->state |= I2SO_OPEN;
		}
	}


	/* Open the audio mixer */
	if (result == OK) {
		rc = AclAm_Open(&cp->am, cp->chip, ACL_AM_MIXER,
						ACL_MODE_SHARED, &bExist);

		if (ACL_ERROR(rc)) {
			AtiCore_DbgPrintf("Error AclAm_Open(0x%X)\n", rc);
			result = BAD;
		} else {
			cp->state |= AM_OPEN;
		}
	}
#endif							//USE_AM

#ifdef USE_SPD
	/* Open the spdif */
	if (result == OK) {
		rc = AclSPDIF_Open(&cp->spd, cp->chip, ACL_SPDIF_PRIMARY,
						   ACL_MODE_SHARED, &bExist);

		if (ACL_ERROR(rc)) {
			AtiCore_DbgPrintf("Error AclSPDIF_Open(0x%X)\n", rc);
			result = BAD;
		} else {
			cp->state |= SPD_OPEN;
		}
	}
#endif							// USE_SPD



	return (result);
}


/*****************************************************************************/

int init_devices(struct SDL_PrivateAudioData *cp)
{
	int result = OK;
	ACL_RESULT rc;
	ACL_I2SO_MODE i2so_mode;
	ACL_I2SO_DEVICE_CONFIG i2so_config;



#ifdef USE_SPD
//everything inside this ifdef is from the sample code in the docs.
#define SF_48KHZ 0x40
#define SF_44_1KHZ 0
#define SF_42KHZ 0xC0

	ACL_SPDIF_CHANNEL_STATUS PrimarySPDIFStatus[192];
	memset((void *) &PrimarySPDIFStatus, 0, sizeof(PrimarySPDIFStatus));
	PrimarySPDIFStatus[3] = SF_48KHZ;
	rc = AclSPDIF_SetChannelStatus(cp->spd,
								   ACL_SPDIF_LEFT_AND_RIGHT_CHANNEL,
								   PrimarySPDIFStatus);
	if (ACL_ERROR(rc)) {
		AtiCore_DbgPrintf("AclSPDIF_SetStatusInfo() failed\n");
		result = BAD;
	}
#endif							// USE_SPD

#ifdef USE_AM
	//ACL_I2SO_MODE i2so_mode;
	//ACL_I2SO_DEVICE_CONFIG i2so_config;

	i2so_mode.AudioStreamType = ACL_STREAM_AUDIO_PCM;
	i2so_mode.DataResolution = ACL_I2SO_DATA_RES_16;
	i2so_mode.WordSelection = ACL_I2SO_WS_BIT_32;

	i2so_config.ChannelSwapping = ACL_I2SO_NONSWAPPED;
	i2so_config.DataFormat = ACL_I2SO_NON_JUSTIFIED;
	i2so_config.OutputPinsSelection = ACL_I2SO_USE_PRIMARY_PINS;
	i2so_config.TransmitMode = ACL_I2SO_SERIAL_CONTINUOUS;
	/* initialize the mixer */
	rc = AclAm_VolumeMute(cp->am, ACL_AM_HOST_CHANNEL | ACL_AM_PRI_CHANNEL,
						  FALSE);
	if (ACL_ERROR(rc)) {
		AtiCore_DbgPrintf("AclAm_VolumeMute() failed\n");
		result = BAD;
	}

	rc = AclAm_SetVolumeConfig(cp->am, 1000, 5, 1,
							   ACL_AM_PRI_CHANNEL | ACL_AM_HOST_CHANNEL);
	if (ACL_ERROR(rc)) {
		AtiCore_DbgPrintf("AclAm_SetVolumeConfig() failed\n");
		result = BAD;
	}

	rc = AclAm_SetHostPcmRes(cp->am, ACL_DATA_RES_16);
	if (ACL_ERROR(rc)) {
		AtiCore_DbgPrintf("AclAm_SetHostPcmRes() failed\n");
		result = BAD;
	}
#if 1
	rc = AclAm_SetVolume(cp->am,
						 ACL_AM_HOST_VOLUME_LEFT | ACL_AM_PRI_VOLUME_LEFT |
						 ACL_AM_HOST_VOLUME_RIGHT |
						 ACL_AM_PRI_VOLUME_RIGHT, 950);
	if (ACL_ERROR(rc)) {
		AtiCore_DbgPrintf("AclAm_SetVolume() failed\n");
		result = BAD;
	}
#endif							// 0
	rc = AclAm_SetHostPcmRes(cp->am, ACL_DATA_RES_16);
	if (ACL_ERROR(rc)) {
		AtiCore_DbgPrintf("AclAm_SetHostPcmRes() failed\n");
		result = BAD;
	}

	/* init the I2SO */
	rc = AclI2SO_SetConfig(cp->i2so, &i2so_config);
	if (ACL_ERROR(rc)) {
		AtiCore_DbgPrintf(" AclI2SO_SetConfig() failed\n");
		result = BAD;
	}

	rc = AclI2SO_SetMode(cp->i2so, &i2so_mode);
	if (ACL_ERROR(rc)) {
		AtiCore_DbgPrintf(" AclI2SO_SetMode() failed\n");
		result = BAD;
	}

#endif							// USE_AM


#ifdef USE_AP
	/* init the AC97 */
	rc = AclAc97_MuteOutput(cp->ac97, FALSE);
	if (ACL_ERROR(rc)) {
		AtiCore_DbgPrintf("AclAc97_MuteOutput() failed\n");
		result = BAD;
	}

	rc = AclAc97_SetOutputVolume(cp->ac97, 950, 950);
	if (ACL_ERROR(rc)) {
		AtiCore_DbgPrintf("AclAc97_SetOutputVolume() failed\n");
		result = BAD;
	}

	rc = AclAc97_SetChannelConfig(cp->ac97, ACL_AC97_ACLNK, 0);
	if (ACL_ERROR(rc)) {
		AtiCore_DbgPrintf("AclAc97_SetChannelConfig() failed\n");
		result = BAD;
	}
#if 1
	rc = AclAc97_SetInputDataRate(cp->ac97, ACL_AC97_DATA_RATE_48);
	if (ACL_ERROR(rc)) {
		AtiCore_DbgPrintf("AclAc97_SetInputDataRate() failed\n");
		result = BAD;
	}

	rc = AclAc97_SetPowerState(cp->ac97, ACL_POWER_RESUME);
	if (ACL_ERROR(rc)) {
		AtiCore_DbgPrintf("AclAc97_SetPowerState() failed\n");
		result = BAD;
	}
#endif							// 0
#endif							// USE_AP
	return (OK);
}


/*****************************************************************************/

int connect_devices(struct SDL_PrivateAudioData *cp)
{

	int result = OK;
	ACL_RESULT rc;

#ifdef USE_AP
	/* connect the AP to the ACLINK */
	if (result == OK) {
		rc = AclAp_SetStreamType(cp->ap, ACL_STREAM_AUDIO_PCM,
								 ACL_STREAM_AUDIO_PCM);
		if (ACL_ERROR(rc)) {
			AtiCore_DbgPrintf("Error AclAp_SetStreamType(0x%X)\n", rc);
			result = BAD;
		}
	}
	if (result == OK) {
		rc = AclAp_Connect(cp->ap, ACL_ACLNK, cp->aclink);
		if (ACL_ERROR(rc)) {
			AtiCore_DbgPrintf("Error AclAp_Connect(0x%X)\n", rc);
			result = BAD;
		} else {
			cp->state |= AP_CONN_ACLINK;
		}
	}

	/* Connect the ACLINK to the AC97 */
	if (result == OK) {
		rc = AclAclnk_Connect(cp->aclink, ACL_AC97, cp->ac97,
							  ACL_STREAM_AUDIO_PCM);
		if (ACL_ERROR(rc)) {
			AtiCore_DbgPrintf("Error AclAclnk_Connect(0x%X)\n", rc);
			result = BAD;
		} else {
			cp->state |= ACLINK_CONN_AC97;
		}
	}

	/* Connect the SI to the AP */
	if (result == OK) {
		rc = AclSi_Connect(cp->si, ACL_AP, cp->ap, ACL_AP_PRIMARY);
		if (ACL_ERROR(rc)) {
			AtiCore_DbgPrintf("Error AclSi_Connect(0x%X)\n", rc);
			result = BAD;
		} else {
			cp->state |= SI_CONN_AP;
		}
	}
#endif							// USE_AP

#ifdef USE_AM
	/* connect the AM to the I2SO */
	if (result == OK) {
		rc = AclAm_Connect(cp->am, ACL_I2SO, cp->i2so);

		if (ACL_ERROR(rc)) {
			AtiCore_DbgPrintf("Error AclAm_Connect(0x%X)\n", rc);
			result = BAD;
		} else {
			cp->state |= AM_CONN_I2SO;
		}
	}

	/* Connect the StreamInterface to the AudioMixer */
	if (result == OK) {
		rc = AclSi_Connect(cp->si, ACL_AM, cp->am, 0);

		if (ACL_ERROR(rc)) {
			AtiCore_DbgPrintf("Error AclSi_Connect(0x%X)\n", rc);
			result = BAD;
		} else {
			cp->state |= SI_CONN_AM;
		}
	}
#endif							//USE_AM

	return (result);
}



/*****************************************************************************/
int start(struct SDL_PrivateAudioData *cp)
{
	int result = OK;
	ACL_RESULT rc;

#ifdef USE_AM

	if (result == OK) {
		rc = AclI2SO_StartStream(cp->i2so);
		if (ACL_ERROR(rc)) {
			AtiCore_DbgPrintf("Error AclI2SO_StartStream(0x%X)\n", rc);
			result = BAD;
		} else {
			cp->state |= I2SO_STARTED;
		}
	}

	if (result == OK) {
		rc = AclAm_Start(cp->am);
		if (ACL_ERROR(rc)) {
			AtiCore_DbgPrintf("Error AclAm_Start(0x%X)\n", rc);
			result = BAD;
		} else {
			cp->state |= AM_STARTED;
		}

	}
#endif							//USE_AM


#ifdef USE_AP
	if (result == OK) {
		rc = AclAc97_StartStream(cp->ac97);
		if (ACL_ERROR(rc)) {
			AtiCore_DbgPrintf("Error AclAc97_StartStream(0x%X)\n", rc);
			result = BAD;
		} else {
			cp->state |= AC97_STARTED;
		}
	}

	if (result == OK) {
		rc = AclAclnk_StartOutputStream(cp->aclink, ACL_STREAM_AUDIO_PCM);
		if (ACL_ERROR(rc)) {
			AtiCore_DbgPrintf("Error AclAclnk_StartOutputStream(0x%X)\n",
							  rc);
			result = BAD;
		} else {
			cp->state |= ACLINK_STARTED;
		}
	}

	if (result == OK) {
		rc = AclAp_Start(cp->ap);
		if (ACL_ERROR(rc)) {
			AtiCore_DbgPrintf("Error AclAp_Start(0x%X)\n", rc);
			result = BAD;
		} else {
			cp->state |= AP_STARTED;
		}
	}
#endif							//USE_AP

	if (result == OK) {
		cp->mutex = SDL_CreateMutex();
		if (cp->mutex == NULL) {
			result = BAD;
			AtiCore_DbgPrintf("Error SDL_CreateMutex gave NULL\n");
		} else {
			cp->state |= MUTEX_OPEN;
		}
	}

	if (result == OK) {
		rc = AclSi_Notify(cp->si, BufferEmptyCallback, (UINT32) cp);
		if (ACL_ERROR(rc)) {
			AtiCore_DbgPrintf("Error AclSi_Notify(0x%X)\n", rc);
			result = BAD;
		} else {
			cp->state |= CALLBACK;
			AtiCore_DbgPrintf("AclSi_Notify called\n");
		}
	}

	if (result == OK) {
		rc = AclSi_Start(cp->si);
		if (ACL_ERROR(rc)) {
			AtiCore_DbgPrintf("Error AclSi_Start(0x%X)\n", rc);
			result = BAD;
		} else {
			cp->state |= SI_STARTED;
		}
	}


	return (result);
}




/*****************************************************************************/

int stop(struct SDL_PrivateAudioData *cp)
{
	int result = OK;
	ACL_RESULT rc;

	//wait for the xilleon to send all of its data out. 
	//....or maybe not...if you do though...remember to 
	//to consider the mutex.  
	//while (cp->prodConDelta < cp->numChainElem) {
	//  usleep(10000) ; 
	//}

	if (cp->state & CALLBACK) {
		rc = AclSi_Notify(cp->si, NULL, 0);
		if (ACL_ERROR(rc)) {
			AtiCore_DbgPrintf("Error AclSi_Notify(0x%X)\n", rc);
			result = BAD;
		} else {
			cp->state &= ~CALLBACK;
		}
	}

	if (cp->state & MUTEX_OPEN) {
		SDL_DestroyMutex(cp->mutex);
		cp->mutex = NULL;
		cp->state &= ~MUTEX_OPEN;
	}

	if (cp->state & SI_STARTED) {
		rc = AclSi_Stop(cp->si);
		if (ACL_ERROR(rc)) {
			AtiCore_DbgPrintf("Error AclSi_Stop(0x%X)\n", rc);
			result = BAD;
		} else {
			AclSi_Flush(cp->si, FALSE);
			cp->state &= ~SI_STARTED;
		}

	}
#ifdef USE_AP
	if (cp->state & ACLINK_STARTED) {
		rc = AclAclnk_StopOutputStream(cp->aclink, ACL_STREAM_AUDIO_PCM);
		if (ACL_ERROR(rc)) {
			AtiCore_DbgPrintf("Error AclAclnk_StopOutputStream(0x%X)\n",
							  rc);
			result = BAD;
		} else {
			cp->state &= ~ACLINK_STARTED;
		}
		rc = AclAclnk_SetErrorNotification(cp->aclink, NULL, ACLINK_NAME,
										   ACL_STREAM_AUDIO_PCM, FALSE);
		if (ACL_ERROR(rc)) {
			AtiCore_DbgPrintf
				("Error AclAclnk_SetErrorNotification(0x%X)\n", rc);
			result = BAD;
		}
	}

	if (cp->state & AC97_STARTED) {
		rc = AclAc97_StopStream(cp->ac97);
		if (ACL_ERROR(rc)) {
			AtiCore_DbgPrintf("Error AclAc97_StopStream(0x%X)\n", rc);
			result = BAD;
		} else {
			cp->state &= ~AC97_STARTED;
		}
	}

	if (cp->state & AP_STARTED) {
		rc = AclAp_Stop(cp->ap);
		if (ACL_ERROR(rc)) {
			AtiCore_DbgPrintf("Error AclAp_Stop(0x%X)\n", rc);
			result = BAD;
		} else {
			cp->state &= ~AP_STARTED;
		}
	}
#endif							//USE_AP


#ifdef USE_AM

	if (cp->state & AM_STARTED) {
		rc = AclAm_Stop(cp->am, FALSE);
		if (ACL_ERROR(rc)) {
			AtiCore_DbgPrintf("Error AclAm_Stop(0x%X)\n", rc);
			result = BAD;
		} else {
			cp->state &= ~AM_STARTED;
		}
	}

	if (cp->state & I2SO_STARTED) {
		rc = AclI2SO_StopStream(cp->i2so);
		if (ACL_ERROR(rc)) {
			AtiCore_DbgPrintf("Error AclI2SO_StopStream(0x%X)\n", rc);
			result = BAD;
		} else {
			cp->state &= ~I2SO_STARTED;
		}
	}
#endif							//USE_AM

	return (result);
}


/*****************************************************************************/

int disconnect_devices(struct SDL_PrivateAudioData *cp)
{
	int result = OK;
	ACL_RESULT rc;

#ifdef USE_AP
	if (cp->state & ACLINK_CONN_AC97) {
		rc = AclAclnk_Disconnect(cp->aclink, ACL_AC97, cp->ac97,
								 ACL_STREAM_AUDIO_PCM);

		if (ACL_ERROR(rc)) {
			AtiCore_DbgPrintf("Error AclLnk_Disconnect(0x%X)\n", rc);
			result = BAD;
		} else {
			cp->state &= ~ACLINK_CONN_AC97;
		}
	}

	if (cp->state & AP_CONN_ACLINK) {
		rc = AclAp_Disconnect(cp->ap, ACL_ACLNK, cp->aclink);

		if (ACL_ERROR(rc)) {
			AtiCore_DbgPrintf("Error AclAp_Disconnect(0x%X)\n", rc);
			result = BAD;
		} else {
			cp->state &= ~AP_CONN_ACLINK;
		}
	}

	if (cp->state & SI_CONN_AP) {
		rc = AclSi_Disconnect(cp->si, ACL_AP, cp->ap);

		if (ACL_ERROR(rc)) {
			AtiCore_DbgPrintf("Error AclSi_Disconnect(0x%X)\n", rc);
			result = BAD;
		} else {
			cp->state &= ~SI_CONN_AP;
		}
	}
#endif							// USE_AP
#ifdef USE_AM
	/* Disconnect the StreamInterface from the audio mixer */
	if (cp->state & SI_CONN_AM) {
		rc = AclSi_Disconnect(cp->si, ACL_AM, cp->am);

		if (ACL_ERROR(rc)) {
			AtiCore_DbgPrintf("Error AclSi_Disconnect(0x%X)\n", rc);
			result = BAD;
		} else {
			cp->state &= ~SI_CONN_AM;
		}
	}


	/* Disconnect the audio mixer from the i2so */
	if (cp->state & AM_CONN_I2SO) {
		rc = AclAm_Disconnect(cp->am, ACL_I2SO, cp->i2so);

		if (ACL_ERROR(rc)) {
			AtiCore_DbgPrintf("Error AclAm_Disconnect(0x%X)\n", rc);
			result = BAD;
		} else {
			cp->state &= ~AM_CONN_I2SO;
		}
	}

#endif							//USE_AM
	return (result);

}

/*****************************************************************************/

int close_devices(struct SDL_PrivateAudioData *cp)
{
	int result = OK;
	ACL_RESULT rc;

	/* Close the stream interface */
	if (cp->state & SI_OPEN) {
		rc = AclSi_Close(cp->si);

		if (ACL_ERROR(rc)) {
			AtiCore_DbgPrintf("Error AclSi_Close(0x%X)\n", rc);
			result = BAD;
		} else {
			cp->state &= ~SI_OPEN;
		}
	}

	if (cp->state & BUFF_ALLOC) {
		void *ptr = NULL;
		int i;
		void *vptr =
			AtiCore_GetAperVirtBaseAddr(THE_CHIP_INDEX, AP_HBIU_MEM);
		if (vptr == NULL) {
			fprintf(stderr,
					"!!!!!AtiCore_GetAperVirtBaseAddr returned NULL \n");
		} else {

			for (i = 0; i < cp->numChainElem; i++) {
				if (cp->bufchain[i].Address.Offset != 0) {
					ptr = vptr + cp->bufchain[i].Address.Offset;
					AtiCore_VideoMemFree(THE_CHIP_INDEX, ptr);
					cp->bufchain[i].Address.Offset = 0;
				}				// if
			}					// for
			cp->state &= ~BUFF_ALLOC;
		}						//else  
	}							// if 

#ifdef USE_AP
	if (cp->state & ACLINK_OPEN) {
		rc = AclAclnk_Close(cp->aclink);
		if (ACL_ERROR(rc)) {
			AtiCore_DbgPrintf("Error AclAclnk_Close(0x%X)\n", rc);
			result = BAD;
		} else {
			cp->state &= ~ACLINK_OPEN;
		}
	}

	if (cp->state & AC97_OPEN) {
		rc = AclAc97_Close(cp->ac97);
		if (ACL_ERROR(rc)) {
			AtiCore_DbgPrintf("Error AclAc97_Close(0x%X)\n", rc);
			result = BAD;
		} else {
			cp->state &= ~AC97_OPEN;
		}
	}


	if (cp->state & AP_OPEN) {
		rc = AclAp_Close(cp->ap);
		if (ACL_ERROR(rc)) {
			AtiCore_DbgPrintf("Error AclAp_Close(0x%X)\n", rc);
			result = BAD;
		} else {
			cp->state &= ~AP_OPEN;
		}
	}
#endif							// USE_AP



#ifdef USE_AM
	/* Close the audio mixer */
	if (cp->state & AM_OPEN) {
		rc = AclAm_Close(cp->am);

		if (ACL_ERROR(rc)) {
			AtiCore_DbgPrintf("Error AclAm_Close(0x%X)\n", rc);
			result = BAD;
		} else {
			cp->state &= ~AM_OPEN;
		}
	}


	/* Close the i2so interface */
	if (cp->state & I2SO_OPEN) {
		rc = AclI2SO_Close(cp->i2so);

		if (ACL_ERROR(rc)) {
			AtiCore_DbgPrintf("Error AclI2SO_Close(0x%X)\n", rc);
			result = BAD;
		} else {
			cp->state &= ~I2SO_OPEN;
		}
	}
#endif							// USE_AM

#ifdef USE_SPD
	if (cp->state & SPD_OPEN) {
		rc = AclSPDIF_Close(cp->spd);

		if (ACL_ERROR(rc)) {
			AtiCore_DbgPrintf("Error AclSPDIF_Close(0x%X)\n", rc);
			result = BAD;
		} else {
			cp->state &= ~SPD_OPEN;
		}
	}
#endif							// USE_SPD

	return (result);
}

//maybe I should check for locks that might still be locked. 
void XILLAUD_CloseAudio(_THIS)
{
	shutdown(this->hidden);
}


/*********************************************************
 * num channels = 2
 * sample rate  = 44100
 * Bytes/second = 176400
 * Bytes/sample = 4
 * Bits/sample  = 16
 ********************************************************/
int XILLAUD_OpenAudio(_THIS, SDL_AudioSpec * spec)
{
	struct SDL_PrivateAudioData *priv = this->hidden;
	
	int res;

#if 0
	if (spec->freq != XILLEON_I2SO_FREQ)	//sample rate too low.
		return -1;

	//dont like unsigned data.
	if ((spec->format == AUDIO_U8) || (spec->format == AUDIO_U16))
		return -1;

	if (spec->channels != 2)	//only want to support stereo.
		return -1;
#endif

#ifdef USE_TRANSFORM
	memcpy(&priv->srcSpec, spec, sizeof(priv->srcSpec));
    priv->srcSpec.samples = CHUNK_SAMPLES;
	SDL_CalculateAudioSpec(&priv->srcSpec);

	priv->xilSpec.freq		= XILLEON_I2SO_FREQ;
	priv->xilSpec.format	= AUDIO_S16;
	priv->xilSpec.channels	= XILLEON_AUDIOCHANNELS;;	// stereo
	priv->xilSpec.samples	= CHUNK_SAMPLES;
	SDL_CalculateAudioSpec(&priv->xilSpec);
#endif
    spec->freq = XILLEON_I2SO_FREQ;
    spec->format = AUDIO_S16;
    spec->channels = XILLEON_AUDIOCHANNELS;;    // stereo
    spec->samples = CHUNK_SAMPLES;  //8192 ;  //this is smallest we could make it before it sounded like crap...tony                                           
    SDL_CalculateAudioSpec(spec);
#ifndef USE_TRANSFORM
	priv->spec = spec;
#endif // USE_TRANSFORM

	res = setup(priv);
	if (res != OK)
		return -1;

	res = start(priv);
	if (res != OK)
		return -1;

	return 0;
}
