/*
    Copyright (C) SEIKO EPSON CORPORATION 2002-2004. All Rights Reserved.

    3-3-5, Owa, Suwa-shi Nagano-ken, 392-8502 Japan
    Phone: +81-266-52-3131

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public License
    as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include <string.h>
#include "SDL_aclvideo.h"
#include "SDL_aclscalars.h"


/*XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX*/
static int FinalCheck(int rgb, int yuv) {

	if ((rgb == SDL_ACL_OVL_SCALAR_V0) && (yuv == SDL_ACL_OVL_SCALAR_V1)) 
		return SDL_ACL_OVL_RGBV0_YUVV1 ;  

	if ((rgb == SDL_ACL_OVL_SCALAR_V0) && (yuv == SDL_ACL_OVL_SCALAR_NONE)) 
		return SDL_ACL_OVL_RGBV0 ; 

	if ((rgb == SDL_ACL_OVL_SCALAR_V1) && (yuv == SDL_ACL_OVL_SCALAR_NONE)) 
		return SDL_ACL_OVL_RGBV1 ; 

	if ((rgb == SDL_ACL_OVL_SCALAR_G0) && (yuv == SDL_ACL_OVL_SCALAR_V0))
		return SDL_ACL_OVL_RGBG0_YUVV0 ;  

	if ((rgb == SDL_ACL_OVL_SCALAR_G0) && (yuv == SDL_ACL_OVL_SCALAR_V1))
		return  SDL_ACL_OVL_RGBG0_YUVV1 ;  

	return SDL_ACL_OVL_INVALID ; 
}

/*XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX*/ 
static int GetScalar(char * env_name) {
	char * val; 
	assert(env_name != NULL) ; 
	val = getenv(env_name) ;
 
	//if the environment was not set we return the default. 	
	if (val == NULL) {
		DEBUG_PRINT("tdebug: %s environment variable was not set!!\n",
                    env_name) ;
		
		if (strcmp(env_name, SDL_ACL_OVL_RGB_ENV_NAME) == 0) {
			return SDL_ACL_OVL_SCALAR_V0 ;
		} else { // must be SDL_ACL_OVL_YUV_ENV_NAME 
			return SDL_ACL_OVL_SCALAR_V1 ; 
		}
 	}

	if (strcmp(val, SDL_ACL_OVL_NONE_VALUE) == 0) 
		return SDL_ACL_OVL_SCALAR_NONE ;  

	if (strcmp(val, SDL_ACL_OVL_V0_VALUE) == 0) 
		return SDL_ACL_OVL_SCALAR_V0 ; 

	if (strcmp(val, SDL_ACL_OVL_V1_VALUE) == 0) 
		return SDL_ACL_OVL_SCALAR_V1 ; 

	if (strcmp(val, SDL_ACL_OVL_G0_VALUE) == 0) 
		return SDL_ACL_OVL_SCALAR_G0 ; 

	DEBUG_PRINT("tdebug: %s = %s is not valid!!\n", env_name, val);
		
	return SDL_ACL_OVL_INVALID ;
} 

/*XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX*/
static int ValidateRGB(int rgb) {
    // the following are all ok for rgb 
	if ((rgb == SDL_ACL_OVL_SCALAR_G0) ||
	    (rgb == SDL_ACL_OVL_SCALAR_V0) || 
	    (rgb == SDL_ACL_OVL_SCALAR_V1)
        ) return rgb ; 

	if (rgb == SDL_ACL_OVL_INVALID) {
		DEBUG_PRINT("tdebug: invalid rgb scalar\n");
		return SDL_ACL_OVL_INVALID; 
	}


	if (rgb == SDL_ACL_OVL_SCALAR_NONE) {
		DEBUG_PRINT("tdebug: MUST HAVE RGB SURFACE!!\n");
		return SDL_ACL_OVL_INVALID; 
	}
	//should never get here. 
	return SDL_ACL_OVL_INVALID; 

}

/*XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX*/
static int ValidateYUV(int yuv) {
    if ((yuv == SDL_ACL_OVL_SCALAR_V0) ||
        (yuv == SDL_ACL_OVL_SCALAR_V1) ||
	    (yuv == SDL_ACL_OVL_SCALAR_NONE)
        ) {
		return yuv; 
	}


	if (yuv == SDL_ACL_OVL_SCALAR_G0) {
		DEBUG_PRINT("tdebug: G0 does not do yuv!!\n");
		return SDL_ACL_OVL_INVALID ;
	}	

    if (yuv == SDL_ACL_OVL_INVALID) {
        DEBUG_PRINT("tdebug: invalid yuv scalar\n");
        return SDL_ACL_OVL_INVALID;
    }

	//should never get here!!
	return SDL_ACL_OVL_INVALID ; 
}


/*XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX*/
static int ACL_CheckForScalars(int* rgb, int* yuv) {

	//get the requested rgb scalar. 
	*rgb = GetScalar(SDL_ACL_OVL_RGB_ENV_NAME) ; 
	if (*rgb == SDL_ACL_OVL_INVALID) 
		return SDL_ACL_OVL_INVALID ; 

	//get the requested yuv scalar.  
	*yuv = GetScalar(SDL_ACL_OVL_YUV_ENV_NAME) ;
	if (*yuv == SDL_ACL_OVL_INVALID)
		return SDL_ACL_OVL_INVALID ; 
	
	//make sure requested scalars are supported.  
	if (ValidateYUV(*yuv) == SDL_ACL_OVL_INVALID) 
		return SDL_ACL_OVL_INVALID ; 

	if (ValidateRGB(*rgb) == SDL_ACL_OVL_INVALID)
        return SDL_ACL_OVL_INVALID ;

	//check for conflict and then determine the correct return code!! 
	return FinalCheck(*rgb, *yuv) ; 
	

}

int ACL_CheckEnvForScalars(int* rgb, int* yuv) {
	int rc ; 
	assert(rgb != NULL) ; 
	assert(yuv != NULL) ; 
	rc = ACL_CheckForScalars(rgb, yuv) ; 
	if (rc == SDL_ACL_OVL_INVALID) {
		*rgb = SDL_ACL_OVL_INVALID ; 
		*yuv = SDL_ACL_OVL_INVALID ; 
	}
	return rc ; 
} 

#if 0
/*XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX*/
int main() {
	int rgb ; 
	int yuv ; 
	int rc ; 
	rc = ACL_CheckEnvForScalars(&rgb, &yuv) ; 
	if (rc == SDL_ACL_OVL_INVALID) {
		fprintf(stderr, "INVALID scalars!!\n") ; 
		return 0 ; 
	} 
	fprintf(stderr, "VALID scalars!!\n") ; 

	return 0;  
}

#endif 
