/*
    SDL - Simple DirectMedia Layer
    Copyright (C) 1997, 1998, 1999, 2000, 2001  Sam Lantinga

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Sam Lantinga
    slouken@devolution.com

***
    Copyright (C) SEIKO EPSON CORPORATION 2002-2004. All Rights Reserved.

    3-3-5, Owa, Suwa-shi Nagano-ken, 392-8502 Japan
    Phone: +81-266-52-3131

*/

#ifdef SAVE_RCSID
static char rcsid =
 "@(#) $Id: SDL_aclvideo.h,v 1.1.1.1 2003/10/14 07:51:23 acyr Exp $";
#endif

#ifndef _SDL_aclvideo_h
#define _SDL_aclvideo_h

#include <termios.h>
#include "ati_core.h"
#include "acl.h"
#include "SDL_mouse.h"
#include "SDL_sysvideo.h"
#include "SDL_mutex.h"
#include "SDL_aclscalars.h"

#ifdef IM_RP

#define ACL_OUT_WIDTH  1280
#define ACL_OUT_HEIGHT 720
#define ACL_OUT_FORMAT ACL_VIDEO_FMT_DVI_1280x720P

#else

#define ACL_OUT_WIDTH  1024
#define ACL_OUT_HEIGHT 768
#define ACL_OUT_FORMAT ACL_VIDEO_FMT_DVI_1024x768P

#endif



/* Define a debug message macro */
#ifndef NDEBUG
#define DEBUG_PRINT(fmt , args...)    fprintf(stderr , fmt , ##args)
#else
#define DEBUG_PRINT(fmt , args...)
#endif


/* Hidden "this" pointer for the video functions */
#define _THIS	SDL_VideoDevice *this


typedef struct private_hwdata {
    struct SDL_PrivateVideoData *info ;
} private_hwdata;


struct private_yuvhwdata {
    SDL_Surface *display;
    ACL_HANDLE yuvovlhnd; 
    BOOL needOvlDisconnect;
    /* These are just so we don't have to allocate them separately */
    Uint16 pitches[1];
    Uint8 *planes[1];
    Uint8 *pixels;
};


/* Private display data */
struct SDL_PrivateVideoData {
    int w, h;
    int pitch;
    void *buffer;

    ACL_HANDLE chip;
    ACL_HANDLE disphnd;
    ACL_HANDLE ovlhnd;
    UINT32 offset;


    BOOL connected;     /* TRUE if this instance called AclOvl_Connect */

#define NUM_MODELISTS   2
    SDL_Rect** SDL_modelist[NUM_MODELISTS];

    // Tony added all the members below this. 
    int current_vt;
    int saved_vt;
    int keyboard_fd;
    int saved_kbd_mode;
    struct termios saved_kbd_termios;

    int mouse_fd;

    private_hwdata hwdata; 
    int rgbScaler, yuvScaler, rgbYuvScalers; 
    struct private_yuvhwdata * overlay;
};

#define SDL_modelist		    (this->hidden->SDL_modelist)

#define console_fd              (this->hidden->console_fd)
#define current_vt              (this->hidden->current_vt)
#define saved_vt                (this->hidden->saved_vt)
#define keyboard_fd             (this->hidden->keyboard_fd)
#define saved_kbd_mode          (this->hidden->saved_kbd_mode)
#define saved_kbd_termios       (this->hidden->saved_kbd_termios)
#define mouse_fd                (this->hidden->mouse_fd)
#define cache_vinfo             (this->hidden->cache_vinfo)
#define saved_vinfo             (this->hidden->saved_vinfo)
#define saved_cmaplen           (this->hidden->saved_cmaplen)
#define saved_cmap              (this->hidden->saved_cmap)
#define mapped_mem              (this->hidden->mapped_mem)
#define mapped_memlen           (this->hidden->mapped_memlen)
#define mapped_offset           (this->hidden->mapped_offset)
#define mapped_io               (this->hidden->mapped_io)
#define mapped_iolen            (this->hidden->mapped_iolen)
#define flip_page               (this->hidden->flip_page)
#define flip_address            (this->hidden->flip_address)
#define SDL_nummodes            (this->hidden->SDL_nummodes)
#define SDL_modelist            (this->hidden->SDL_modelist)
#define surfaces                (this->hidden->surfaces)
#define surfaces_memtotal       (this->hidden->surfaces_memtotal)
#define surfaces_memleft        (this->hidden->surfaces_memleft)
#define hw_lock                 (this->hidden->hw_lock)
#define wait_vbl                (this->hidden->wait_vbl)
#define wait_idle               (this->hidden->wait_idle)

#endif /* _SDL_aclvideo_h */
