/*
  SDL - Simple DirectMedia Layer
  Copyright (C) 1997, 1998, 1999, 2000, 2001  Sam Lantinga

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Library General Public
  License as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Library General Public License for more details.

  You should have received a copy of the GNU Library General Public
  License along with this library; if not, write to the Free
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

  Sam Lantinga
  slouken@devolution.com

***
  Copyright (C) SEIKO EPSON CORPORATION 2002-2004. All Rights Reserved.

  3-3-5, Owa, Suwa-shi Nagano-ken, 392-8502 Japan
  Phone: +81-266-52-3131

*/

#ifdef SAVE_RCSID
static char rcsid =
"@(#) $Id: SDL_aclyuv.c,v 1.4 2004/06/15 09:20:11 acyr Exp $";
#endif

/* This is the software implementation of the YUV video overlay support */

/* This code was derived from code carrying the following copyright notices:

* Copyright (c) 1995 The Regents of the University of California.
* All rights reserved.
* 
* Permission to use, copy, modify, and distribute this software and its
* documentation for any purpose, without fee, and without written agreement is
* hereby granted, provided that the above copyright notice and the following
* two paragraphs appear in all copies of this software.
* 
* IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY FOR
* DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT
* OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF THE UNIVERSITY OF
* CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
* 
* THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
* INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
* AND FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS
* ON AN "AS IS" BASIS, AND THE UNIVERSITY OF CALIFORNIA HAS NO OBLIGATION TO
* PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.

* Copyright (c) 1995 Erik Corry
* All rights reserved.
* 
* Permission to use, copy, modify, and distribute this software and its
* documentation for any purpose, without fee, and without written agreement is
* hereby granted, provided that the above copyright notice and the following
* two paragraphs appear in all copies of this software.
* 
* IN NO EVENT SHALL ERIK CORRY BE LIABLE TO ANY PARTY FOR DIRECT, INDIRECT,
* SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OF
* THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF ERIK CORRY HAS BEEN ADVISED
* OF THE POSSIBILITY OF SUCH DAMAGE.
* 
* ERIK CORRY SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT
* LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
* PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS"
* BASIS, AND ERIK CORRY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT,
* UPDATES, ENHANCEMENTS, OR MODIFICATIONS.

* Portions of this software Copyright (c) 1995 Brown University.
* All rights reserved.
* 
* Permission to use, copy, modify, and distribute this software and its
* documentation for any purpose, without fee, and without written agreement
* is hereby granted, provided that the above copyright notice and the
* following two paragraphs appear in all copies of this software.
* 
* IN NO EVENT SHALL BROWN UNIVERSITY BE LIABLE TO ANY PARTY FOR
* DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT
* OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF BROWN
* UNIVERSITY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
* 
* BROWN UNIVERSITY SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT
* LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
* PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS"
* BASIS, AND BROWN UNIVERSITY HAS NO OBLIGATION TO PROVIDE MAINTENANCE,
* SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
*/

#include <stdlib.h>
#include <string.h>

#include "SDL_error.h"
#include "SDL_video.h"
#include "SDL_stretch_c.h"
#include "SDL_yuvfuncs.h"

#include "SDL_aclvideo.h"
#include "SDL_aclyuv_c.h"

typedef Uint32 int32;

#define ACL_OUT_WIDTH   1024
#define ACL_OUT_HEIGHT  768 
#define ACL_IN_YUV_BYTEPP   2 
#define ACL_IN_ARGB_BYTEPP   4 

#define ACL_CHIP_INDEX  0
/* The functions used to manipulate software video overlays */
static struct private_yuvhwfuncs ACL_yuvfuncs = {
	ACL_LockYUV,
	ACL_UnlockYUV,
	ACL_DisplayYUV,
	ACL_ZoomYUV,
	ACL_HideYUV,
	ACL_FreeYUV
};

/* I am going to assume that the acl is initialized and the 
 * diplay and main overlay (G0) are already connected since
 * in order for the application to be calling this, the 
 * video device must have already been created.  
 */  

SDL_Overlay *ACL_CreateYUV(_THIS, int width, int height, 
                           Uint32 format, SDL_Surface *display)
{
	SDL_Overlay *overlay = NULL;
	struct private_yuvhwdata *acldata = NULL ; 
    ACL_RESULT rc ; 
	ACL_HANDLE ovlhnd; 
    ACL_DATA_FORMAT datafmt; 
    BOOL exist ; 
	UINT32 alignment; 
	UINT32 channel = MEM_CHAN_0_1;
	UINT32 size ; 
    UINT32 pitch ; 
    ACL_RECT pWin ; 
	ACL_NAME name ; 

	//ensure a valid scalar has been requested. 
	if (this->hidden->yuvScaler == SDL_ACL_OVL_SCALAR_V0) {
		name = ACL_OVL_V0SCALE; 		
	} else if (this->hidden->yuvScaler == SDL_ACL_OVL_SCALAR_V1) {
		name = ACL_OVL_V1SCALE;
	} else { 
		return NULL ; 
	}	

	/* Verify that we support the format 
     * tony: check if its ok to only support SDL_YVYU_OVERLAY
     * and SDL_UYVY_OVERLAY because these are the only two packed
	 * YUV formats that are supported by acl. 
     */ 
	switch (format) {
    case SDL_YVYU_OVERLAY:
		datafmt = ACL_DATA_FORMAT_VYUY422 ; 
		break;
    case SDL_UYVY_OVERLAY:
        datafmt =   ACL_DATA_FORMAT_YVYU422  ;
        break ;
	case SDL_ARGB_OVERLAY:
		datafmt =   ACL_DATA_FORMAT_ARGB8888_32BPP ;
		break;
	case SDL_RGB565_OVERLAY:
		datafmt =   ACL_DATA_FORMAT_RGB565 ;
		break;
    case SDL_YUY2_OVERLAY:
    case SDL_IYUV_OVERLAY:
    case SDL_YV12_OVERLAY:
    default:
		DEBUG_PRINT("Unsupported YUV format");
		return(NULL);
	}

	/* Create the overlay structure */
	overlay = (SDL_Overlay *)malloc(sizeof *overlay);
	if ( overlay == NULL ) {
		SDL_OutOfMemory();
		return(NULL);
	}
	memset(overlay, 0, (sizeof *overlay));

	/* Fill in the basic members */
    overlay->hw_overlay = 1; 
	overlay->format = format;
	overlay->w = width;
	overlay->h = height;

	/* Set up the YUV surface function structure */
	overlay->hwfuncs = &ACL_yuvfuncs;

	/* Create the pixel data */
	acldata = (struct private_yuvhwdata *)
        malloc(sizeof(struct private_yuvhwdata));

	if ( acldata == NULL ) {
		SDL_OutOfMemory();
		SDL_FreeYUVOverlay(overlay);
		return(NULL);
	}
    memset(acldata,0, sizeof(struct private_yuvhwdata));
	acldata->display = display;
	overlay->hwdata = acldata;
	overlay->hwdata->needOvlDisconnect = FALSE;
	/* we do all the acl related stuff...Video scalar!! */ 
	
	rc = AclOvl_Open (&ovlhnd, this->hidden->chip, name, ACL_MODE_SHARED, &exist); 
    if (ACL_ERROR (rc))
    {
        AtiCore_DbgPrintf ("\nError: Unable to open graphics overlay object\n");
        return NULL ;
    }
    acldata->yuvovlhnd = ovlhnd ; 

	if (exist) {
		overlay->hwdata->needOvlDisconnect = FALSE ;
	} else {
		/* Connect to the display instance */
		rc = AclOvl_Connect (ovlhnd, ACL_DISP, this->hidden->disphnd);
		if (ACL_ERROR (rc))
		{
			AtiCore_DbgPrintf ("\nError: Unable to connect graphics overlay\n");
			return NULL;
		}
		overlay->hwdata->needOvlDisconnect = TRUE ;
	}

	/* setting the destination window and crop window 
	 * to display size turns off cropping.  
	 */ 
    pWin.Left = 0;
    pWin.Top = 0;
    pWin.Width = ACL_OUT_WIDTH;
    pWin.Height = ACL_OUT_HEIGHT;
    rc = AclOvl_SetDstWindow (ovlhnd, &pWin, FALSE);
    if (ACL_ERROR (rc))
    {
        AtiCore_DbgPrintf ("\nError(1): Unable to init overlay: rc=%x\n", rc);
        return NULL;
    }

	rc = AclOvl_SetCropWindow(ovlhnd,
	                          ACL_OVL_CROP_SRC,    
	                          &pWin);        
    if (ACL_ERROR (rc))
    {
        AtiCore_DbgPrintf ("\nError(2): Unable to init overlay: rc=%x\n", rc);
        return NULL;
    }

    /* Set a graphics overlay destination window */
    pWin.Left = 0;
    pWin.Top = 0;
    pWin.Width = width ; //ACL_OUT_WIDTH;
    pWin.Height = height ; //ACL_OUT_HEIGHT;
    rc = AclOvl_SetDstWindow (ovlhnd, &pWin, FALSE);
    if (ACL_ERROR (rc))
    {
        AtiCore_DbgPrintf ("\nError: Unable to set graphics overlay window: rc=%x\n", rc);
        return NULL;
    }

    /* Set source format 
	 * Asked aric about this and he says that the alignment that is returned
     * is actually in _pixels_!!  must be careful!!  Does this make sense?? 
     * not to me!!  
     */
    rc = AclOvl_SetSrcFormat (ovlhnd, datafmt, &alignment);
    if (ACL_ERROR (rc))
    {
        AtiCore_DbgPrintf ("\nError: Unable to set graphics overlay source format\n");
        return NULL;
    }
	if (format == SDL_ARGB_OVERLAY)
		alignment *= ACL_IN_ARGB_BYTEPP ;
	else 
    	alignment *= ACL_IN_YUV_BYTEPP ;
                	
    /* Set source window size */
    rc = AclOvl_SetSrcWindowSize (ovlhnd, width, height, &pitch);
    if (ACL_ERROR (rc))
    {
        AtiCore_DbgPrintf ("\nError: Unable to set graphics overlay source window\n");
        return NULL;
    }

    /* asked aric about this and he says that the pitch that is returned 
     * by AclOvl_SetSrcWindowSize is actually in pixels.   Does this make 
     * sense??  not to me...oh well. 
     */ 
	if (format == SDL_ARGB_OVERLAY)
		pitch *= ACL_IN_ARGB_BYTEPP ;
	else 
    	pitch *= ACL_IN_YUV_BYTEPP ;
	size = pitch * height ; 
	acldata->pixels = (Uint8 *) 
        AtiCore_VideoMemAlloc(ACL_CHIP_INDEX, 
                              &channel,
                              size, 
                              alignment, 
                              &this->hidden->offset, 
                              ALLOC_TYPE_NORM_GUI);

    if (acldata->pixels == NULL) {
        DEBUG_PRINT("Couldn't allocate buffer for requested mode");
        return NULL ;
    }
	memset(acldata->pixels, 0, size);

	rc = AclOvl_SetSrcOffset(ovlhnd, this->hidden->offset, FALSE);
    if (ACL_ERROR (rc))
    {
        AtiCore_DbgPrintf ("\nError: Unable to enable graphics overlay\n");
        return NULL;
    }
	
    /* Disable the graphics overlay */    
    rc = AclOvl_Enable (ovlhnd, FALSE);
    if (ACL_ERROR (rc))
    {
        AtiCore_DbgPrintf ("\nError: Unable to disable yuv overlay\n");
        return NULL;
    }

	if ((this->hidden->rgbScaler != SDL_ACL_OVL_SCALAR_V0) &&
        (this->hidden->rgbScaler != SDL_ACL_OVL_SCALAR_V1)
        ) {
        rc = AclOvl_SetAlpha(this->hidden->ovlhnd, ACL_OVL_ALPHA_MULTIPLY,
                             ACL_OVL_ALPHA_PIXEL, 0)  ;
        if (ACL_ERROR (rc))
        {
            AtiCore_DbgPrintf ("\nError: Unable to to set the alpha: rc = %d\n", rc);
            return NULL;
        }
    }



	// note that we are supporting packed formats only 
    overlay->pixels = acldata->planes;
	overlay->pixels[0] = acldata->pixels;
    overlay->pitches = acldata->pitches;
    overlay->pitches[0] = pitch ;         
    overlay->planes = 1;


	/* We're all done.. */
	return(overlay);
}

int ACL_LockYUV(_THIS, SDL_Overlay *overlay)
{
	return(0);
}

void ACL_UnlockYUV(_THIS, SDL_Overlay *overlay)
{
	return;
}

int ACL_DisplayYUV(_THIS, SDL_Overlay *overlay, SDL_Rect *dstrect)
{
	struct private_yuvhwdata *swdata;
    ACL_RECT pWin ;
	ACL_RESULT rc ; 
	swdata = overlay->hwdata;
	pWin.Left = dstrect->x ;
	pWin.Top = dstrect->y ; 
	pWin.Width = dstrect->w ; 
	pWin.Height = dstrect->h ; 
    rc = AclOvl_SetDstWindow (swdata->yuvovlhnd, &pWin, FALSE);
    if (ACL_ERROR (rc))
    {
        AtiCore_DbgPrintf ("\nError: Unable to set graphics overlay window\n");
        return -1;
    }

    rc = AclOvl_Enable (swdata->yuvovlhnd, TRUE);
    if (ACL_ERROR (rc))
    {
        AtiCore_DbgPrintf ("\nError: Unable to enable yuv ovl\n");
        return -1;
    }

	return(0);
}
void ACL_ZoomYUV(_THIS, SDL_Overlay *overlay, SDL_Rect *srcrect)
{
	struct private_yuvhwdata *swdata = overlay->hwdata;
    ACL_RECT   srcwin ;
	ACL_RESULT rc ;

	if ( srcrect ) {
		srcwin.Left = srcrect->x ;
		srcwin.Top = srcrect->y ; 
		srcwin.Width = srcrect->w ; 
		srcwin.Height = srcrect->h ; 
	} else {
        srcwin.Left   = 0;
        srcwin.Top    = 0;
        srcwin.Width  = ACL_OUT_WIDTH;
        srcwin.Height = ACL_OUT_HEIGHT;
	}

	rc = AclOvl_SetCropWindow(swdata->yuvovlhnd,
	                          ACL_OVL_CROP_SRC,    
	                          &srcwin);        

#ifdef DEBUG
    if (ACL_ERROR (rc)) {
		fprintf (stderr, "AclOvl_SetCropWindow error!!\n");
	}
#endif
}

void ACL_HideYUV(_THIS, SDL_Overlay *overlay)
{
	struct private_yuvhwdata *swdata = overlay->hwdata;
	ACL_RESULT rc ;

    rc = AclOvl_Enable (swdata->yuvovlhnd, FALSE);

#ifdef DEBUG
    if (ACL_ERROR (rc)) {
		fprintf (stderr, "AclOvl_AclOvl_Enable(FALSE) error!!\n");
	}
#endif
}

void ACL_FreeYUV(_THIS, SDL_Overlay *overlay)
{
	struct private_yuvhwdata *swdata;
	int sdl_rc ;
	ACL_RESULT rc ;
	SDL_Rect dstrect ;
	swdata = overlay->hwdata;
	//we set the destination window back to its default.
	dstrect.x = 0;
	dstrect.y = 0;
	dstrect.w = ACL_OUT_WIDTH;
	dstrect.h = ACL_OUT_HEIGHT;
	sdl_rc = ACL_DisplayYUV(this, overlay, &dstrect) ;
	if (sdl_rc < 0)
		fprintf(stderr, "could not reset the ovl back to the right size\n");

	if ( swdata ) {
		if (ACL_HANDLE_VALID(swdata->yuvovlhnd)) {
			rc = AclOvl_Enable (swdata->yuvovlhnd, FALSE);
			if (ACL_ERROR (rc))
				fprintf(stderr, "could not disable yuvovl\n") ;

			if ((ACL_HANDLE_VALID(this->hidden->disphnd)) && (swdata->needOvlDisconnect)) {
				rc = AclOvl_Disconnect(swdata->yuvovlhnd, ACL_DISP, this->hidden->disphnd);
				if (ACL_ERROR (rc))
					fprintf(stderr, "could not disconnect the yuvovl \n");
				swdata->needOvlDisconnect = FALSE;
			}

			rc = AclOvl_Close(swdata->yuvovlhnd) ;
			if (ACL_ERROR (rc))
				fprintf(stderr, "could not close yuvovl\n") ;
		}
		swdata->yuvovlhnd = 0 ;


		if ( swdata->pixels ) {
			AtiCore_VideoMemFree(ACL_CHIP_INDEX, swdata->pixels);
            swdata->pixels = NULL;
		}
		free(swdata);
        overlay->hwdata = swdata = NULL;
	}
	/* NOTE: even though we malloc the SDL_Overlay, SDL will 
	 *       free it right after this function call. 
	 */  
}
