/*
    Copyright (C) SEIKO EPSON CORPORATION 2002-2004. All Rights Reserved.

    3-3-5, Owa, Suwa-shi Nagano-ken, 392-8502 Japan
    Phone: +81-266-52-3131

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public License
    as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include <stdlib.h>
#include <stdio.h>
#include "png.h"
#include "SDL_png.h"

#define PNG_ROW_BYTEALIGN 4





#define  RGB_PNG_MAX_AREA 3042510
#define RGBA_PNG_MAX_AREA 2104935


SDL_Surface *private_CheckPNG(const char *filename)
{

	SDL_Surface *surface = NULL;
	int ret = 0;
	png_structp read_ptr = NULL;
	png_infop read_info_ptr = NULL;
	static png_FILE_p file = NULL;
	png_uint_32 width, height;
	int bitDepth, typeColor, typeInterlace, typeCompression, typeFilter;
	Uint32 Rmask, Gmask, Bmask, Amask;

	int channels = 0;
	Rmask = Gmask = Bmask = Amask = 0;

	if (filename == NULL)
		return NULL;

	file = fopen(filename, "rb");
	if (file == NULL) {
		return NULL;
	}
	
	read_ptr =
		png_create_read_struct(PNG_LIBPNG_VER_STRING, png_voidp_NULL,
							   png_error_ptr_NULL, png_error_ptr_NULL);
	if (read_ptr == NULL) {
		if (file != NULL)
			fclose(file);
		return NULL;
	}

	read_info_ptr = png_create_info_struct(read_ptr);
	if (read_info_ptr == NULL) {
		png_destroy_read_struct(&read_ptr,
								read_info_ptr ? &read_info_ptr : NULL,
								png_infopp_NULL);
		if (file != NULL)
			fclose(file);
		return NULL;
	}
	
	if (setjmp(png_jmpbuf(read_ptr))) {
		png_destroy_read_struct(&read_ptr,
								read_info_ptr ? &read_info_ptr : NULL,
								png_infopp_NULL);
		if (file != NULL)
			fclose(file);
		return NULL;

	}
	
	png_init_io(read_ptr, file);

	
	png_set_read_status_fn(read_ptr, png_read_status_ptr_NULL);

	
	png_read_info(read_ptr, read_info_ptr);

	
	png_get_IHDR(read_ptr, read_info_ptr, &width, &height, &bitDepth,
				 &typeColor, &typeInterlace, &typeCompression,
				 &typeFilter);

	if (typeInterlace != PNG_INTERLACE_NONE) {
		goto bail;
	}


	


	if (bitDepth < 8) {
		png_set_packing(read_ptr);
	} else if (bitDepth == 16) {
		png_set_strip_16(read_ptr);
	}




	if (typeColor == PNG_COLOR_TYPE_GRAY) {
		png_set_gray_to_rgb(read_ptr);
	}

	if (typeColor & PNG_COLOR_MASK_PALETTE) {
		png_set_palette_to_rgb(read_ptr);
	}

	if (png_get_valid(read_ptr, read_info_ptr, PNG_INFO_tRNS)) {
		png_set_tRNS_to_alpha(read_ptr);
	}

	png_read_update_info(read_ptr, read_info_ptr);

	png_get_IHDR(read_ptr, read_info_ptr, &width, &height, &bitDepth,
				 &typeColor, &typeInterlace, &typeCompression,
				 &typeFilter);





	ret = 1;
	if (read_info_ptr->channels == 4) {
		if (width * height > RGBA_PNG_MAX_AREA)
			ret = 0;
	} else {
		if (width * height > RGB_PNG_MAX_AREA)
			ret = 0;
	}

	if (typeColor != PNG_COLOR_TYPE_PALETTE &&
		typeColor != PNG_COLOR_TYPE_GRAY) {
		if (SDL_BYTEORDER == SDL_LIL_ENDIAN) {
			Rmask = 0x000000ff;
			Gmask = 0x0000ff00;
			Bmask = 0x00ff0000;
			if (read_info_ptr->channels == 4) {
				Amask = 0xff000000;
			}
		} else {
			if (read_info_ptr->channels == 4) {
				Rmask = 0xff000000;
				Gmask = 0x00ff0000;
				Bmask = 0x0000ff00;
				Amask = 0x000000ff;
			} else {
				Rmask = 0x00ff0000;
				Gmask = 0x0000ff00;
				Bmask = 0x000000ff;
			}
		}
	}

	channels = read_info_ptr->channels;

bail:
	png_destroy_read_struct(&read_ptr,
							read_info_ptr ? &read_info_ptr : NULL,
							png_infopp_NULL);

	if (file != NULL)
		fclose(file);

	if (ret == 0)
		return NULL;

	surface = SDL_CreateRGBSurface(SDL_SWSURFACE, width, height,
								   bitDepth * channels,
								   Rmask, Gmask, Bmask, Amask);




	return surface;
}



SDL_Surface *SDL_LoadPNG(const char *filename)
{
	SDL_Surface *surface = NULL;
	png_structp read_ptr = NULL;
	png_infop read_info_ptr;
	static png_FILE_p file;
	png_uint_32 width, height;
	int bitDepth, typeColor, typeInterlace, typeCompression, typeFilter;
	Uint32 Rmask, Gmask, Bmask, Amask;
	int i;
	png_bytep ptr;
	int bytes_per_row;

	surface = private_CheckPNG(filename);

	if (surface == NULL) {
		return NULL;
	}
	Rmask = Gmask = Bmask = Amask = 0;
	read_info_ptr = NULL;

	
	file = fopen(filename, "rb");
	if (file == NULL) {
		goto bail;
	}
	
	read_ptr =
		png_create_read_struct(PNG_LIBPNG_VER_STRING, png_voidp_NULL,
							   png_error_ptr_NULL, png_error_ptr_NULL);
	if (read_ptr == NULL) {
		goto bail;
	}
	
	read_info_ptr = png_create_info_struct(read_ptr);
	if (read_info_ptr == NULL) {
		goto bail;
	}
	
	if (setjmp(png_jmpbuf(read_ptr))) {
		goto bail;
	}
	
	png_init_io(read_ptr, file);

	
	png_set_read_status_fn(read_ptr, png_read_status_ptr_NULL);

	
	png_read_info(read_ptr, read_info_ptr);

	
	png_get_IHDR(read_ptr, read_info_ptr, &width, &height, &bitDepth,
				 &typeColor, &typeInterlace, &typeCompression,
				 &typeFilter);

#if LOADPNG_DEBUG
	fprintf(stderr,
			"width:%d height:%d depth:%d color:%d interlace:%d comp:%d filter:%d\n",
			width, height, bitDepth, typeColor, typeInterlace,
			typeCompression, typeFilter);
#endif							

	if (bitDepth < 8) {
		png_set_packing(read_ptr);
	} else if (bitDepth == 16) {
		png_set_strip_16(read_ptr);
	}




	
	if (typeColor == PNG_COLOR_TYPE_GRAY) {
		png_set_gray_to_rgb(read_ptr);
	}
	
	if (typeColor & PNG_COLOR_MASK_PALETTE) {
		png_set_palette_to_rgb(read_ptr);
	}

	if (png_get_valid(read_ptr, read_info_ptr, PNG_INFO_tRNS)) {
		png_set_tRNS_to_alpha(read_ptr);
	}

	png_read_update_info(read_ptr, read_info_ptr);

	png_get_IHDR(read_ptr, read_info_ptr, &width, &height, &bitDepth,
				 &typeColor, &typeInterlace, &typeCompression,
				 &typeFilter);



#ifdef LOADPNG_DEBUG
	fprintf(stderr,
			"width:%d height:%d depth:%d color:%d interlace:%d comp:%d filter:%d\n",
			width, height, bitDepth, typeColor, typeInterlace,
			typeCompression, typeFilter);

	fprintf(stderr, "channels %d\n", read_info_ptr->channels);
#endif							
	bytes_per_row = png_get_rowbytes(read_ptr, read_info_ptr);

	if (typeColor & PNG_COLOR_MASK_PALETTE) {
		int i;
		int ncolors;
		SDL_Color color;
		png_colorp palette;

		if (png_get_valid(read_ptr, read_info_ptr, PNG_INFO_PLTE)) {
			png_get_PLTE(read_ptr, read_info_ptr, &palette, &ncolors);

			for (i = 0; i < ncolors; i++) {
				color.r = palette[i].red;
				color.g = palette[i].green;
				color.b = palette[i].blue;
				SDL_SetColors(surface, &color, i, 1);
			}
			fprintf(stderr, "palette colors: %d\n", ncolors);
		} else {
			goto bail;
		}

	} else if (typeColor == PNG_COLOR_TYPE_GRAY) {
		int i;
		SDL_Color color;
		png_color palette[256];

		png_build_grayscale_palette(read_info_ptr->bit_depth,
									(png_colorp) & palette);

		for (i = 0; i < 256; i++) {
			color.r = palette[i].red;
			color.g = palette[i].green;
			color.b = palette[i].blue;
			SDL_SetColors(surface, &color, i, 1);
		}
		fprintf(stderr, "grayscale palette\n");
	}



	if (SDL_MUSTLOCK(surface)) {
		SDL_LockSurface(surface);
	}

	ptr = (png_bytep) surface->pixels;
	for (i = 0; i < height; i++) {
		png_read_row(read_ptr, ptr, NULL);
		ptr += surface->pitch;
	}

	png_read_end(read_ptr, NULL);

	if (SDL_MUSTLOCK(surface)) {
		SDL_UnlockSurface(surface);
	}

bail:
	if (read_ptr != NULL) {
		png_destroy_read_struct(&read_ptr,
								read_info_ptr ? &read_info_ptr : NULL,
								png_infopp_NULL);
	}
	if (file != NULL) {
		fclose(file);
	}

	return surface;
}

#if LOADPNG_DEBUG

int main(int argc, char **argv)
{
	SDL_Surface *surface;
	SDL_Surface *screen;
	if (argc < 2)
		return 1;

	SDL_Init(SDL_INIT_VIDEO);
	atexit(SDL_Quit);

	surface = SDL_LoadPNG(argv[1]);
	if (!surface) {
		exit(1);
	}

	screen = SDL_SetVideoMode(320, 240, 32, 0);;

	SDL_BlitSurface(surface, NULL, screen, NULL);

	SDL_FreeSurface(surface);

	SDL_UpdateRect(screen, 0, 0, 0, 0);

	for (;;) {
		SDL_Event event;
		while (SDL_PollEvent(&event)) {
			if (event.type == SDL_MOUSEBUTTONDOWN) {
				exit(1);
			}

		}
	}
	return 0;
}
#endif
