/*
    Copyright (C) SEIKO EPSON CORPORATION 2002-2004. All Rights Reserved.

    3-3-5, Owa, Suwa-shi Nagano-ken, 392-8502 Japan
    Phone: +81-266-52-3131

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public License
    as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <assert.h>
#include <setjmp.h>
#include <jpeglib.h>
#include <sys/time.h>


struct my_error_mgr {
        struct jpeg_error_mgr pub;      /* "public" fields */
        jmp_buf setjmp_buffer;          /* for return to caller */
};
typedef struct my_error_mgr *my_error_ptr;

METHODDEF(void) my_error_exit(j_common_ptr cinfo)
{
	my_error_ptr myerr = (my_error_ptr) cinfo->err;
	longjmp(myerr->setjmp_buffer, 1);
}


static void
do_it(FILE *fp, J_COLOR_SPACE colorspace)
{
	struct jpeg_decompress_struct cinfo;
	JSAMPROW rows[2];
	struct my_error_mgr jerr;
	size_t size;

        cinfo.err = jpeg_std_error(&jerr.pub);
        jerr.pub.error_exit = my_error_exit;
	if (setjmp(jerr.setjmp_buffer)) {
		jpeg_destroy_decompress(&cinfo);
		printf("Ops.\n");
		return;
	}
	
	jpeg_create_decompress(&cinfo);
	jpeg_stdio_src(&cinfo, fp);

	jpeg_read_header(&cinfo, TRUE);

	cinfo.dct_method = JDCT_FASTEST;
	cinfo.do_fancy_upsampling = FALSE;
	cinfo.do_block_smoothing = FALSE;

	jpeg_start_decompress(&cinfo);

	cinfo.out_color_space = colorspace;

	if (colorspace == JCS_RGB) {
		size = cinfo.output_width * 4 + 128; 
	} else if (colorspace == JCS_RGB16_565) {
		size = cinfo.output_width * 4 + 128;
	} else if (colorspace == JCS_YCbCr) {
		size = cinfo.output_width * 4 + 128;
	}

	rows[0] = malloc(size);
	if (!rows[0])
		exit(1);
	rows[1] = malloc(size);
	if (!rows[1])
		exit(1);
	while (cinfo.output_scanline < cinfo.output_height) {
		jpeg_read_scanlines(&cinfo, rows, 2);
	}
	free(rows[0]);
	free(rows[1]);

	jpeg_finish_decompress(&cinfo);
	jpeg_destroy_decompress(&cinfo);
}

unsigned int timediff(struct timeval *cur, struct timeval *old)
{
	unsigned int t;
        t = (cur->tv_sec - old->tv_sec) * 1000;
        if (cur->tv_usec > old->tv_usec) {
                t += (cur->tv_usec - old->tv_usec) / 1000;
        } else {
                t += (cur->tv_usec +(1000000 - old->tv_usec)) / 1000;
                t -= 1000;
        }
	return t;
}

#define TEST_TIME	16

int main(int argc, char **argv)
{
	FILE *fp;
	unsigned int t;
	struct timeval cur, old;
	int i;
	if (argc < 2) {
		printf("jpeg_bench: jpeg_bench <filename>\n");
		return 1;
	}

	if (!strstr(argv[1], "jpg") && !strstr(argv[1], "JPG")) {
		return 1;
	}
	
	fp = fopen(argv[1], "r");
	if (!fp)
		return 1;

	gettimeofday(&old, NULL);
	for (i = 0; i < TEST_TIME; i++) {
		do_it(fp, JCS_RGB);
		fseek(fp, 0, SEEK_SET);
	}
	gettimeofday(&cur, NULL);

	t = timediff(&cur, &old);
	printf("%d times jpeg loading time(24bpp): %d msec  avg: %fmsec\n", TEST_TIME, t, (double)t / (double)TEST_TIME);
	
	gettimeofday(&old, NULL);
	for (i = 0; i < TEST_TIME; i++) {
		do_it(fp, JCS_RGB16_565);
		fseek(fp, 0, SEEK_SET);
	}
	gettimeofday(&cur, NULL);

	t = timediff(&cur, &old);
	printf("%d times jpeg loading time(16bpp): %d msec  avg: %fmsec\n", TEST_TIME, t, (double)t / (double)TEST_TIME);

	gettimeofday(&old, NULL);
	for (i = 0; i < TEST_TIME; i++) {
		do_it(fp, JCS_YCbCr);
		fseek(fp, 0, SEEK_SET);
	}
	gettimeofday(&cur, NULL);

	t = timediff(&cur, &old);
	printf("%d times jpeg loading time(YCbCr): %d msec  avg: %fmsec\n", TEST_TIME, t, (double)t / (double)TEST_TIME);

	fclose(fp);
}
