/*
    Copyright (C) SEIKO EPSON CORPORATION 2002-2004. All Rights Reserved.

    3-3-5, Owa, Suwa-shi Nagano-ken, 392-8502 Japan
    Phone: +81-266-52-3131

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public License
    as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#ifndef GRAIN_H
#define GRAIN_H

#include <SDL.h>

#include "UIRsrcMgr.h"

#include "EventImpl.h"


const ::SDL_Rect	SDLRECT_ZERO = {0, 0, 0, 0};


class Grain : public EventImpl {
public:
	Grain								(SDL_Surface		*screen,
										 const SDL_Rect&	rect);

		virtual 		~Grain			();

public: 
	virtual Language	GetLang			() {return m_lang;}

	virtual int			SetLang			(const Language		lang) { return 0; }	

public: 
	virtual int			Init			();

	virtual int			Idle			();

	virtual int			Draw			();

	virtual int			Clear			(const SDL_Rect& rect);

	virtual	int			Redraw			();

	virtual void		Update			();

	virtual int			Suspend			();

	virtual void		DontRestore		() {m_restore = false;}

	virtual int			Activate		() {m_active = true; return 0;}

	virtual int			Deactivate		() {m_active = false; return 0;}

	virtual int			IsActive		() {return m_active;}

	virtual int			IsDead			() {return m_dead;}

	virtual void		SetDrawableArea	(const SDL_Rect& rect) {drawable_area = rect;}

	virtual const SDL_Rect&
						GetDrawableArea() {return drawable_area;}

public: 
	Grain*				GetNext			() {return m_next;}

	Grain*				GetPrev			() {return m_prev;}

	Grain*				GetParent		() {return m_parent;}

	void				SetParent		(Grain      *node) {m_parent = node;}

	void				SetNext			(Grain		*node);

	void				SetPrev			(Grain		*node);

	void				AddGrain		(Grain		*node);

	void				DeleteGrain		(Grain		*node);

	static void			SwapGrain		(Grain		*a,
										 Grain		*b);

	static int			MoveGrain		(Grain		*a,
										 int		distance);

	static Grain*		GetBottomGrain	(Grain		*node);

	static Grain*		GetTopGrain		(Grain *node);

	static int			GetIndex		(Grain *node);


protected: 
	virtual int			StoreOrigImage	();

	virtual int			RestoreOrigImage();

	virtual int			LoadBgRsrc		();

	virtual int			LoadFrameRsrc	();

	virtual int			CreateBgImage	();

protected:
	virtual ProcStatus	EventUser		(const SDL_Event& event);

protected: 
	virtual void		UpdateRect		(::SDL_Surface		*surface,
										 const ::SDL_Rect&	rect = SDLRECT_ZERO) {
		::SDL_UpdateRect(m_screen, rect.x, rect.y, rect.w, rect.h);
	}

protected: 
	
	SDL_Surface			*m_screen;		

	
	SDL_Rect			drawable_area;

	
	SDL_Rect			m_UpdateArea;

	
	SDL_Surface			*m_Image;

	
	SDL_Surface			*m_OrigImage;	

	
	
	bool				m_restore;

	
	SDL_Surface			*m_BgImage;		

	
	SDL_Surface			*m_BgRsrc;		
	const char			*m_BgRsrcName;	

	
	int					m_AlphaValue;	

	SDL_Surface			*m_FrameRsrc;	
	const char			*m_FrameRsrcName;	

	Language			m_lang;

	
	Grain				*m_next;

	
	Grain				*m_prev;

	
	Grain				*m_child;

	
	Grain				*m_parent;

	
	bool				m_active;

	
	bool				m_dead;
};

#endif 
