/*
    Copyright (C) SEIKO EPSON CORPORATION 2002-2004. All Rights Reserved.

    3-3-5, Owa, Suwa-shi Nagano-ken, 392-8502 Japan
    Phone: +81-266-52-3131

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public License
    as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#ifndef UIRSRCMGR_H
#define UIRSRCMGR_H

#include <SDL.h>

enum Language {
	LangEnglish,	
	LangJapanese,	
	LangFrench,		
	LangGermany,	
	LangItaly,		
	LangSpanish,	
	LangPortuguese,	
	LangChinese,	
	LangKorean		
};

struct rsrc_file_t;
class UIRsrcMgr {
public:
	enum Rsrc {
		
		iconSelector,		
		iconSelectorOn,		
		iconFolder,			
		iconParFolder,		
		iconFileJPEG,		
		iconFileMPEG,		
		iconFileSIT1,		
		iconFileSIT2,		
		iconFileDPOF,		
		iconFileNCF,

		
		btbBtnSelector,		
		btbBtnSelectorOn,	
		btbButton,			
		btbBtnBlue,			

		
		btbNextPage,		
		btbPrevPage,		
		btbThumbnail,		
		btbIcon,			
		btbOK,				
		btbCancel,			
		btbReset,			
        btbBtnAdvanced,     
        btbBasic,           
        btbTagMail,         
        btbTagSNMP,         
       	btbTagCCX,
		btbTagLEAP,
		btbTagPSK,
		btbTagWPA,				
	   	btbSecurity,		
		btbEject,			

		
		propUnselectedL,	
		propUnselectedB,	
		propUnselectedR,	
		propSelectedL,		
		propSelectedB,		
		propSelectedR,		
		propShadowL,		
		propShadowB,		
		propShadowR,		
		propOnL,			
		propOnB,			
		propOnR,			

		
		propOpenFolder,		
		propPlaySlideShow,	
		propSetOptions,		
		propPlayScenario,	
		propEditScenario,	
		propPlayPict,		
		propPlayMovie,		
		propMove,			
		propShowHide,		
		propCancel,			
		propToUp,			
		propSetting,
		
		
		optRepeat,			
		optInterval,		
		optSort,			
		optEffect,			
		optUIMode,
		optHLine,			

		
		rbtnBtnOn,
		rbtnBtnOff,
		rbtnBtnDisabled,
		rbtnBtnSelector,
		rbtnBtnSelectorOn,
		rbtnTag10sec,
		rbtnTag20sec,
		rbtnTag30sec,
		rbtnTag1min,
		rbtnTagDisolve,
		rbtnTagNone,
		rbtnTagOn,
		rbtnTagOff,
		rbtnTagSortName,
		rbtnTagSortTime,
		rbtnTagWipe1,
		rbtnTagWipe2,
		rbtnTagGuideMode,
		rbtnTagQuickMode,
		rbtnTagNetEasy,
		rbtnTagNetAP,
        rBtnTagWEP,
		rbtnTagWipe,
		rbtnTagRnd,

        
        chkEmpty,
        chkChecked,

		
		navIconEnter,
		navIconEnterCardType,	
		navIconEscape,
		navIconEscapeCardType,	
		navIconCursors,
		navIconPageUp,
		navIconPageDown,
		navIconArwVert,
		navIconArwHorz,

		
		navTagDecide,
		navTagSelect,
		navTagNextPage,	
		navTagPrevPage,
		navTagBack,	
		navTagCancel,	
		navTagOpFolder,
		navTagProperty,	
		navTagPlMovie,	
		navTagPlPict,	
		navTagPlScenario,	
		navTagShowPict,	
		navTagHidePict,	
		navTagToUp,	
		navTagMove,
		navTagToTop,
		navTagMoveCursor,
		navTagRotation,
		navTagSetting,

		
		dlgBackground,

		
		dlgExit,
		dlgContinue,
		
		
		sedSelector,
		sedSelectorOn,
		sedMarker,
		sedMarkerOn,
		sedDisable,

		
		pvtFolder,
		pvtPreview,
		pvtOption,
		pvtShow,
		pvtHide,
		pvtMoveFile,

		
		cfgFrameL,
		cfgFrameM,
		cfgFrameR,
        cfgAntennaOn,
        cfgAntennaOff,
        cfgAntennaGray,
        cfgAntenna,
		cfgTtlNetwork,
		cfgTtlWiredBasic,
		cfgTtlWlessBasic,
		cfgTtlCPOption,

		
		msgBg,
		msgQuitScenario,
		msgQuitSlideshow,
		msgQuitMovie,

		msgAcquireDHCP,
		msgAuthenticating,
		msgCertFailure,
		msgSetFailure,	
        
        erdTagDHCP,
        erdTagESSID,
        erdTagGateway,
        erdTagIPAddress,
        erdTagKeyword,
        erdTagMACAddress,
        erdTagPassword,
        erdTagProjectorName,
        erdTagSubnet,
        erdTagTrapIP1,
        erdTagTrapIP2,
        erdTagSeparator,
        erdInputL,
        erdInputR,
        erdStr128Bit,
        erdStr1,
        erdStr2,
        erdStr3,
        erdStr40Bit,
        erdStr4,
        erdStrHEX,
        erdStrText1,
        erdStrText2,
        erdStrText3,
        erdCooling,
        erdFanErr,
        erdIntErr,
        erdLampCvr,
        erdLampOut,
        erdLampReplc,
        erdLampTimer,
        erdMailNotify,
        erdNoSignal,
        erdNofEvent,
	erdSctyPassword,
	erdUserName,
	erdSMTPAdr1,
        erdSMTPAdr2,
        erdSMTPAdr3,
        erdSMTPPort,
        erdSMTPSrv,
        erdSnsrErr,
        erdTempErr,
        erdWEPEncKey,
        erdWEPEnc,
        erdWEPInput,
        erdWEPKeyID,
        erdNetwork,
        erdStrSmall1,
        erdStrSmall2,
        erdStrSmall3,

        
        btnClr,
        btnClrSelectorOff,
        btnClrSelectorOn,
        btnClrLabel,

        
        navBtn,
        navBtnGray,
        navBtnPress,
        navBtnSelect,
        navBtnLabel,
        labGuide1,
        labGuide2_1,
        labGuide2_2,
        labGuide2_3,
        labGuide2_4,
        labGuide3,
        labGuide4,
        msgBarBack1,
        msgBarBack2,

		
		cfgSetWired,
		cfgSetWireless,
		cfgSetNetwork,

	
	stuPjName,
	stuTitleCard,
	stuTitleDSC,
	stuTitleWired,
	stuTitleWless,
	stuEssid,
	stuIPAddress,
	stuColon,
	stuKeyword,	
	
	
	stuTitle,			
	stuNsTitle,			
	stuCpTitle,			
	stuNsDsc,			
	stuCpDsc,			

		
		nofFileNotFound,
		nofBadFile
	};

	enum Module {
		ModNone		= 0x0,
		ModEMP		= 0x1,	
		ModVNC		= 0x2,	
		ModCONF		= 0x4,	
		ModAll 		= ModEMP|ModVNC|ModCONF
	};

public:
	static UIRsrcMgr*	Instance(const Language& lang)
	{
		instance = Instance();
		if (instance && instance->SetLang(lang)) {
			delete instance;
			instance = NULL;
		}
		return instance;
	}	

	static UIRsrcMgr*	Instance(Uint8 lang)
	{
		return Instance(E21Lang2UILang(lang));
	}

	static UIRsrcMgr*	Instance()
	{
		if (instance == NULL) {
			instance = new UIRsrcMgr();
		}
		return instance;
	}

protected: 
	UIRsrcMgr();
		~UIRsrcMgr();

	static Language	E21Lang2UILang(Uint8 lang);

public:
	int				Init		(Module using_module = ModNone);

	SDL_Surface*	GetResource	(const Rsrc	what);

	void			Dispose		();

	int				SetLang		(const Uint8 lang);

	const Language&	GetLang		() {return m_lang;}

protected:
	void			DisposeLocalizeRsrc();

	int				Load		(rsrc_file_t& rsrc_file);

	int				SetLang		(const Language& lang);


protected:
	static UIRsrcMgr*	instance;

	Language		m_lang;

	Module			m_using_module;

	unsigned int	m_unBoardID;
};

#endif 
