/*
    Copyright (C) SEIKO EPSON CORPORATION 2002-2004. All Rights Reserved.

    3-3-5, Owa, Suwa-shi Nagano-ken, 392-8502 Japan
    Phone: +81-266-52-3131

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public License
    as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include "config.h"

#include "View.h"
#include "BtnBox.h"


BtnBox* BtnBox::instance = NULL;

#define BTN_LOCATION(x,b,n) ((x) + (b) * (n))

const int BTN_WIDTH		= 188;
const int BTN_HEIGHT	= 34;
const int BTN_CREVICE	= 7;
const int BTN_THIN = 3;

const ::SDL_Rect btnitems_rects[] = 
{
	{BTN_LOCATION(224, BTN_WIDTH+BTN_CREVICE, 0), 16, BTN_WIDTH + BTN_THIN * 2, BTN_HEIGHT + BTN_THIN * 2},
	{BTN_LOCATION(224, BTN_WIDTH+BTN_CREVICE, 1), 16, BTN_WIDTH + BTN_THIN * 2, BTN_HEIGHT + BTN_THIN * 2},
	{BTN_LOCATION(224, BTN_WIDTH+BTN_CREVICE, 2), 16, BTN_WIDTH + BTN_THIN * 2, BTN_HEIGHT + BTN_THIN * 2},
	{BTN_LOCATION(224, BTN_WIDTH+BTN_CREVICE, 3), 16, BTN_WIDTH + BTN_THIN * 2, BTN_HEIGHT + BTN_THIN * 2}
};

#undef BTN_LOCATION

BtnBox::
BtnBox			(::SDL_Surface *screen,
				 const ::SDL_Rect& rect)
				 :Grain(screen, rect)
{
	instance = this;
	m_target = NULL;
	m_UpdateArea = drawable_area;
}


BtnBox::
~BtnBox			()
{
	instance = NULL;
}



int	BtnBox::
Init			()
{
	 int result;
	result = StoreOrigImage();
	if (result != 0) {
		return result;
	}

	
	 Grain *p;
	for (int i = 0; i < (int)(sizeof(btnitems_rects)/sizeof(::SDL_Rect)); i++) {
		p = new BtnBoxItem(m_screen, btnitems_rects[i]);
		result = p->Init();
		if (result != 0) {
			return result;
		}
		AddGrain(p);
	}

	return 0;
}




int	BtnBox::
Draw			()
{
	 int result;
	result = ::SDL_BlitSurface(m_OrigImage, NULL, m_screen, &m_UpdateArea);
	if (result != 0) {
		SDL_ERROR();
		return result;
	}

	 Grain *p = m_child;
	while (p) {
		result = p->Draw();
		if (result != 0) {
			return result;
		}
		p = p->GetNext();
	}

	return 0;
}



void BtnBox::
Update			()
{
	UpdateRect(m_screen, m_UpdateArea);
}



ProcStatus BtnBox::
Event			(const ::SDL_Event& event)
{
	
	 Item *p = BtnBoxItem::GetSelectedItem(m_child);
	if (p) {
		return p->Event(event);
	}

	return ProcKeepGoing;
}



BtnBoxItem::BtnStatus BtnBox::
GetBtnStatus	(BtnID	id)
{
	 Grain *p = m_child;
	for (int i = 0; p != NULL; i++) {
		if (i == id) {
			assert(dynamic_cast<BtnBoxItem *>(p));
			return reinterpret_cast<BtnBoxItem *>(p)->GetStatus();
		}
		p = p->GetNext();
	}
	return BtnBoxItem::BtnHidden;
}




BtnBoxItem::BtnType BtnBox::
GetBtnType		(BtnID	id)
{
	 Grain *p = m_child;
	for (int i = 0; p != NULL; i++) {
		if (i == id) {
			assert(dynamic_cast<BtnBoxItem *>(p));
			return reinterpret_cast<BtnBoxItem *>(p)->GetType();
		}
		p = p->GetNext();
	}
	return BtnBoxItem::BtnNone;
}



void BtnBox::
SetTarget		(View			*target)
{
	if (!target) {
		return;
	}

	m_target = target;

	
	target->Deactivate();

	
	 BtnBoxItem *bp;
	 Grain *p = m_child;
	while (p) {
		assert(dynamic_cast<BtnBoxItem *>(p));
		bp = reinterpret_cast<BtnBoxItem *>(p);
		if (bp->GetType() != BtnBoxItem::BtnNone &&
			bp->GetStatus() == BtnBoxItem::BtnActive) {
			bp->Select();
			bp->Draw();
			bp->Update();
			break;
		}
		p = p->GetNext();
	}
}



void BtnBox::
UnsetTarget		(View *target)
{
	DebugF("View::UnsetTarget\n");
	assert(target != NULL);

	if (target != m_target) {
		return;
	}
	m_target = NULL;

	
	 Grain *p = Item::GetSelectedItem(m_child);
	if (p) {
		assert(dynamic_cast<BtnBoxItem *>(p));
		reinterpret_cast<BtnBoxItem *>(p)->Unselect();
	}

	

	target->Activate();
}



ProcStatus BtnBox::
DoCallback		(const BtnBoxItem::BtnType type)
{
	DebugF("BtnBox::DoCallback\n");
	if (m_target) {
		return m_target->Callback(type);
	}
	return ProcKeepGoing;
}



int BtnBox::
SetBtnStatus	(BtnID id,
				 BtnBoxItem::BtnStatus stat)
{
	 Grain *p = GetTopGrain(m_child);
	for (int i = 0; p != NULL; i++) {
		if (id == i) {
			assert(dynamic_cast<BtnBoxItem *>(p));
			return reinterpret_cast<BtnBoxItem *>(p)->SetBtnStatus(stat);
		}
		p = p->GetNext();
	}
	return 0;
}



int	BtnBox::
SetBtnType		(BtnID id,
				 BtnBoxItem::BtnType type)
{
	
	 Grain *p = GetTopGrain(m_child);
	for (int i = 0; p != NULL; i++) {
		if (id == i) {
			assert(dynamic_cast<BtnBoxItem *>(p));
			return reinterpret_cast<BtnBoxItem *>(p)->SetBtnType(type);
		}
		p = p->GetNext();
	}
	return 0;
}



int	BtnBox::
SetBtn			(BtnID id,
				 BtnBoxItem::BtnStatus stat,
				 BtnBoxItem::BtnType type)
{
	 int result;
	result = SetBtnStatus(id, stat);
	if (result != 0) {
		return result;
	}
	result = SetBtnType(id, type);
	if (result != 0) {
		return result;
	}
	return 0;
}

int BtnBox::
Redraw			()
{
	if (m_target) {
		return m_target->Redraw();
	}
	return 0;
}

