/*
    Copyright (C) SEIKO EPSON CORPORATION 2002-2004. All Rights Reserved.

    3-3-5, Owa, Suwa-shi Nagano-ken, 392-8502 Japan
    Phone: +81-266-52-3131

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public License
    as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include <limits.h>
#include "config.h"

#include "View.h"

#include "BtnBox.h"
#include "BtnBoxItem.h"

const int SELECTOR_THIN = 3;


BtnBoxItem::
BtnBoxItem		(::SDL_Surface		*screen,
				 const ::SDL_Rect&	rect)
				 : Item(screen, rect)
{
	m_type = BtnNone;
	m_stat = BtnHidden;

	m_UpdateArea = drawable_area;
}



BtnBoxItem::
~BtnBoxItem		()
{
}



int BtnBoxItem::
Init			()
{
	 int result;
	result = StoreOrigImage();
	if (result != 0) {
		return result;
	}

	
	 ::SDL_PixelFormat *format = m_screen->format;
	m_Image = ::SDL_CreateRGBSurface(SDL_SWSURFACE,
		drawable_area.w, drawable_area.h, format->BitsPerPixel,
			format->Rmask, format->Gmask, format->Bmask, format->Amask);
	if (!m_Image) {
		SDL_ERROR();
		return -1;
	}

	
	result = ::SDL_BlitSurface(m_OrigImage, NULL, m_Image, NULL);
	if (result != 0) {
		SDL_ERROR();
		return result;
	}
	m_UpdateArea = drawable_area;

	return 0;
}



int BtnBoxItem::
Draw			()
{
	 int result;
	result = CreateBtnImage();
	if (result != 0) {
		return result;
	}

	result = ::SDL_BlitSurface(m_Image, NULL, m_screen, &m_UpdateArea);
	if (result != 0) {
		SDL_ERROR();
		return result;
	}

	if (m_selected && reinterpret_cast<BtnBox *>(m_parent)->IsActive()) {
        SDL_Surface* rsrc;
        rsrc = UIRsrcMgr::Instance()->GetResource(UIRsrcMgr::btbBtnSelector);
		result = ::SDL_BlitSurface(rsrc, NULL, m_screen, &m_UpdateArea);
		if (result != 0) {
			SDL_ERROR();
			return result;
		}
	}

	return 0;
}



void BtnBoxItem::
Update			()
{
	UpdateRect(m_screen, m_UpdateArea);
}



int BtnBoxItem::
CreateBtnImage()
{
	 int result;
	 ::SDL_Rect rect;
	 ::SDL_Surface *btn, *descr;

	result = ::SDL_BlitSurface(m_OrigImage, NULL, m_Image, NULL);
	if (result != 0) {
		return result;
	}

	
	btn = UIRsrcMgr::Instance()->GetResource(UIRsrcMgr::btbButton);
    if ( btn == 0 ) {
        return -1;
    }
	if (m_stat != BtnHidden) {
		rect.x = SELECTOR_THIN;
		rect.y = SELECTOR_THIN;
		rect.w = btn->w;
		rect.h = btn->h;

		
		result = ::SDL_BlitSurface(btn, NULL, m_Image, &rect);
		if (result != 0) {
			SDL_ERROR();
			return result;
		}
	}

	if (m_type == BtnNone) {
		return 0;
	}

	descr = GetResource();
    if ( descr == 0 ) {
        return -1;
    }

	
	rect.w = descr->w;
	rect.h = descr->h;
	rect.x = (btn->w - descr->w) / 2 + SELECTOR_THIN;
	rect.y = (btn->h - descr->h) / 2 + SELECTOR_THIN;
	if (rect.x < 0) rect.x = SELECTOR_THIN;
	if (rect.y < 0) rect.y = SELECTOR_THIN;

	result = ::SDL_BlitSurface(descr, NULL, m_Image, &rect);
	if (result != 0) {
		SDL_ERROR();
		return result;
	}

	result = SDL_SetAlpha(m_Image, SDL_SRCALPHA,
		m_stat == BtnInactive ? ((UCHAR_MAX * 25) / 100) : UCHAR_MAX);
	if (result != 0) {
		SDL_ERROR();
		return result;
	}

	return 0;
}



int BtnBoxItem::
SetBtnType		(BtnType			type)
{
	if (m_type == type) {
		return 0;
	}

	m_type = type;
#if 0
	return CreateBtnImage();
#else
	return 0;
#endif
}



int BtnBoxItem::
SetBtnStatus	(BtnStatus			stat)
{
	if (m_stat == stat) {
		return 0;
	}

	m_stat = stat;
	if ((m_stat == BtnInactive) && m_selected) {
		 Grain *p = GetTopGrain(this);
		while (p) {
			assert(dynamic_cast<BtnBoxItem *>(p));
			if (reinterpret_cast<BtnBoxItem *>(p)->m_stat == BtnActive &&
				reinterpret_cast<BtnBoxItem *>(p)->m_type != BtnNone) {
				reinterpret_cast<BtnBoxItem *>(p)->Select();
				Unselect();
				break;
			}
			p = p->GetNext();
		}
	}

#if 0
	return CreateBtnImage();
#else
	return 0;
#endif
}



::SDL_Surface* BtnBoxItem::
GetResource		() const
{
	const static struct rsrc_conv_t {
		BtnType			type;
		UIRsrcMgr::Rsrc rsrc;
	} rsrc_converter[] = {
		{BtnNextPage,	UIRsrcMgr::btbNextPage},
		{BtnPrevPage,	UIRsrcMgr::btbPrevPage},
		{BtnViewThumb,	UIRsrcMgr::btbThumbnail},
		{BtnViewIcon,	UIRsrcMgr::btbIcon},
		{BtnOK,			UIRsrcMgr::btbOK},
		{BtnCancel,		UIRsrcMgr::btbCancel},
		{BtnReset,		UIRsrcMgr::btbReset},
		{BtnEject,		UIRsrcMgr::btbEject}        
	};

	for (int i = 0; i < (int)(sizeof(rsrc_converter)/sizeof(rsrc_conv_t)); i++) {
		if (m_type == rsrc_converter[i].type) {
			return UIRsrcMgr::Instance()->GetResource(rsrc_converter[i].rsrc);
		}
	}
	return NULL;
}



ProcStatus BtnBoxItem::
DoEnter	()
{
	Flash(UIRsrcMgr::btbBtnSelectorOn, UIRsrcMgr::btbBtnSelector);

	return BtnBox::Instance()->DoCallback(m_type);
}




ProcStatus BtnBoxItem::
DoLeft	()
{
	 Grain *p;
	p = m_prev;
	 BtnBoxItem *bp;
retry:
	while (p) {
		assert(dynamic_cast<BtnBoxItem *>(p));
		bp = reinterpret_cast<BtnBoxItem *>(p);
		if (bp->m_type == BtnNone ||
			bp->m_stat != BtnActive) {
			p = p->GetPrev();
			continue;
		}
		if (bp == this) {
			break;
		}
		bp->Select();
		Unselect();
		bp->Draw(); 
		Draw(); 
		bp->Update();
		Update();
		break;
	}
	if (!p) {
		p = this;
		while (p) {
			if (p->GetNext()) {
				p = p->GetNext();
			} else {
				break;
			}
		}
		goto retry;
	}
	return ProcKeepGoing;
}



ProcStatus BtnBoxItem::
DoRight	()
{
	 Grain *p = m_next;
	if (m_next) {
		p = m_next;
	} else {
		p = this;
		while (p && p->GetPrev()) {
			p = p->GetPrev();
		}
	}
	 BtnBoxItem *bp;
	while (p) {
		assert(dynamic_cast<BtnBoxItem *>(p));
		bp = reinterpret_cast<BtnBoxItem *>(p);
		if ((bp->m_type == BtnNone) ||
			(bp->m_stat != BtnActive)) {
			p = p->GetNext();
			continue;
		}
		if (bp == this) {
			break;
		}
		bp->Select();
		Unselect();
		bp->Draw();
		Draw();
		bp->Update();
		Update();
		break;
	}
	return ProcKeepGoing;
}


ProcStatus BtnBoxItem::
DoUp		()
{
	 View *p = BtnBox::Instance()->GetTarget();
	if (!p) {
		return ProcKeepGoing;
	}
	switch (p->GetMode()) {
	case View::FileList:
	case View::SitEditor:
		return DoPageUp();
	case View::Option:
	case View::ERDConf:
		BtnBox::Instance()->DoCallback(BtnBoxItem::BtnUp);
		if (BtnBox::Instance()->GetTarget()) {
			BtnBox::Instance()->UnsetTarget(BtnBox::Instance()->GetTarget());
		}
		return ProcYield;
		
	default:
		break;
	}
	return ProcKeepGoing;
}



ProcStatus BtnBoxItem::
DoDown		()
{
	 View *p = BtnBox::Instance()->GetTarget();
	if (!p) {
		return ProcKeepGoing;
	}
	switch (p->GetMode()) {
	case View::FileList:
	case View::SitEditor:
		BtnBox::Instance()->UnsetTarget(BtnBox::Instance()->GetTarget());
		return ProcYield;
	case View::Option:
	case View::ERDConf:
		BtnBox::Instance()->DoCallback(BtnBoxItem::BtnDown);
		break;
	default:
		break;
	}

	return ProcKeepGoing;
}



ProcStatus BtnBoxItem::
DoPageUp	()
{
	 View *p = BtnBox::Instance()->GetTarget();
	if (!p) {
		return ProcKeepGoing;
	}
	switch (p->GetMode()) {
	case View::FileList:
	case View::Option:
	case View::SitEditor:
		BtnBox::Instance()->DoCallback(BtnBoxItem::BtnPrevPage);
		break;
	case View::ERDConf:
        {
            BtnBox::Instance()->SetBtn(BtnBox::Btn1, BtnBoxItem::BtnActive, BtnBoxItem::BtnReset);
            BtnBoxItem* bp = this;
            while (bp->GetPrev()) {
                bp = static_cast<BtnBoxItem*>(bp->GetPrev());
            }

            if ( bp != this ) {
                bp->Select();
                Unselect();
                bp->Draw(); 
                Draw(); 
                bp->Update();
                Update();
            }
        }
        break;
	default:
		break;
	}

	return ProcKeepGoing;
}



ProcStatus BtnBoxItem::
DoPageDown	()
{
	 View *p = BtnBox::Instance()->GetTarget();
	if (!p) {
		return ProcKeepGoing;
	}
	switch (p->GetMode()) {
	case View::FileList:
	case View::Option:
	case View::SitEditor:
		BtnBox::Instance()->DoCallback(BtnBoxItem::BtnNextPage);
		break;
	case View::ERDConf:
		BtnBox::Instance()->UnsetTarget(p);
        p->Draw();
        p->Update();
		return ProcYield;
		break;
	default:
		break;
	}

	return ProcKeepGoing;
}


