/*
    Copyright (C) SEIKO EPSON CORPORATION 2002-2004. All Rights Reserved.

    3-3-5, Owa, Suwa-shi Nagano-ken, 392-8502 Japan
    Phone: +81-266-52-3131

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public License
    as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include "config.h"

#include "UIRsrcMgr.h"

#include "BaseDialog.h"


DlgBtn::
DlgBtn			(::SDL_Surface		*screen,
				 const ::SDL_Rect&	rect,
				 BtnType			type)
				: Item(screen, rect)
{
	m_type = type;
	m_UpdateArea = drawable_area;
}



DlgBtn::
~DlgBtn			()
{
}



int DlgBtn::
Init			()
{
	return 0;
}



int DlgBtn::
Draw			()
{
	 int result;
	 ::SDL_Surface *btn, *selector, *descr;
	 ::SDL_Rect rect;

	btn = UIRsrcMgr::Instance()->GetResource(UIRsrcMgr::btbButton);
	selector = UIRsrcMgr::Instance()->GetResource(UIRsrcMgr::btbBtnSelector);
	if (!btn || !selector) {
		return -1;
	}

	
	rect = drawable_area;
	rect.w = btn->w;
	rect.h = btn->h; 
	rect.x += (selector->w - btn->w) / 2; 
	rect.y += (selector->h - btn->h) / 2; 

	
	result = ::SDL_BlitSurface(btn, NULL, m_screen, &rect);
	if (result != 0) {
		SDL_ERROR();
		return result;
	}

	descr = UIRsrcMgr::Instance()->GetResource( m_type == BtnExit ? UIRsrcMgr::dlgExit : UIRsrcMgr::dlgContinue);
	if (!descr) {
		return -1;
	}
	rect = drawable_area;

	
	rect.w = descr->w;
	rect.h = descr->h;
	rect.x += (drawable_area.w - descr->w) / 2;
	rect.y += (drawable_area.h - descr->h) / 2;
	result = ::SDL_BlitSurface(descr, NULL, m_screen, &rect);
	if (result != 0) {
		SDL_ERROR();
		return result;
	}

	if (!m_selected) {
		return 0;
	}

	
	rect = drawable_area;
	result = ::SDL_BlitSurface(UIRsrcMgr::Instance()->GetResource(
		UIRsrcMgr::btbBtnSelector), NULL, m_screen, &rect);
	if (result != 0) {
		SDL_ERROR();
		return result;
	}

	return 0;
}



void DlgBtn::
Update			()
{
	UpdateRect(m_screen, drawable_area);
}



ProcStatus DlgBtn::
DoEnter			()
{
	
	Flash(UIRsrcMgr::btbBtnSelectorOn, UIRsrcMgr::btbBtnSelector);

	assert(dynamic_cast<BaseDialog *>(m_parent));
	reinterpret_cast<BaseDialog *>(m_parent)->DoCallback(m_type);
	return ProcEnd;
}



ProcStatus DlgBtn::
DoEscape		()
{
	assert(dynamic_cast<BaseDialog *>(m_parent));
	reinterpret_cast<BaseDialog *>(m_parent)->DoCallback(BtnContinue);
	return ProcEnd;
}



ProcStatus DlgBtn::
DoRight			()
{
	 DlgBtn *p;
	if (m_next) {
		assert(dynamic_cast<DlgBtn *>(m_next));
		p = reinterpret_cast<DlgBtn *>(m_next);
	} else {
		assert(dynamic_cast<DlgBtn *>(GetTopGrain(this)));
		p = reinterpret_cast<DlgBtn *>(GetTopGrain(this));
	}

	p->Select();
	Unselect();

	m_parent->Draw(); 
	m_parent->Update();

	return ProcKeepGoing;
}



ProcStatus DlgBtn::
DoLeft			()
{
	 DlgBtn *p;
	if (m_prev) {
		assert(dynamic_cast<DlgBtn *>(m_prev));
		p = reinterpret_cast<DlgBtn *>(m_prev);
	} else {
		assert(dynamic_cast<DlgBtn *>(GetBottomGrain(this)));
		p = reinterpret_cast<DlgBtn *>(GetBottomGrain(this));
	}

	p->Select();
	Unselect();
	
	m_parent->Draw(); 
	m_parent->Update();

	return ProcKeepGoing;
}


