/*
    Copyright (C) SEIKO EPSON CORPORATION 2002-2004. All Rights Reserved.

    3-3-5, Owa, Suwa-shi Nagano-ken, 392-8502 Japan
    Phone: +81-266-52-3131

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public License
    as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include "config.h"
#include <stdlib.h>
#include <SDL.h>

#include "SDL_e21.h"

#include "UIRsrcMgr.h"

#include "EventImpl.h"



int EventImpl::
FlashKeyEvents	()
{
	 SDL_Event event;
	SDL_PumpEvents();

	while (SDL_PeepEvents(&event, 1, SDL_GETEVENT,
		SDL_KEYDOWNMASK|SDL_KEYUPMASK) > 0) {
		DebugF("skipping\n");
	}

	return 0;
}


int EventImpl::
FlashAllEvents	()
{
	 SDL_Event event;
	SDL_PumpEvents();
	
	while (SDL_PeepEvents(&event, 1, SDL_GETEVENT, SDL_ALLEVENTS) > 0) {
		fprintf(stderr, "skipping\n");
	}

	return 0;
}



ProcStatus EventImpl::
EventActivate	(const SDL_Event& event)
{
	DebugF("EventActivate\n");
	return ProcKeepGoing;
}



ProcStatus EventImpl::
EventKeyboard	(const SDL_Event& event)
{
#ifdef KEYWORKS_ON_MAKE
	if (event.key.state != SDL_PRESSED) {
		return ProcKeepGoing;
	}
#else
	if (event.key.state != SDL_RELEASED) {
		return ProcKeepGoing;
	}
#endif
	 ProcStatus stat = ProcKeepGoing;
	switch (event.key.keysym.sym) {
	case SDLK_UP:
		stat = DoUp();
		break;
	case SDLK_DOWN:
		stat = DoDown();
		break;
	case SDLK_RIGHT:
		stat = DoRight();
		break;
	case SDLK_LEFT:
		stat = DoLeft();
		break;
	case SDLK_PAGEUP:
		stat = DoPageUp();
		break;
	case SDLK_PAGEDOWN:
		stat = DoPageDown();
		break;
	case SDLK_RETURN:
		stat = DoEnter();
		break;
	case SDLK_ESCAPE:
		stat = DoEscape();
		break;
	default:

		break;
	}

	
	
	

	return stat;
}



ProcStatus EventImpl::
EventMouse		(const SDL_Event& event)
{
	return ProcKeepGoing;
}



ProcStatus EventImpl::
EventJoystick	(const SDL_Event& event)
{
	DebugF("EventJoystick\n");
	return ProcKeepGoing;
}



ProcStatus EventImpl::
EventUser		(const SDL_Event& event)
{
	return ProcKeepGoing;
}



ProcStatus EventImpl::
EventWM			(const SDL_Event& event)
{
	DebugF("EventWM\n");
	return ProcKeepGoing;
}



ProcStatus EventImpl::
EventSystem		(const SDL_Event& event)
{
	DebugF("EventSystem\n");
	return ProcKeepGoing;
}



ProcStatus EventImpl::
EventVideo		(const SDL_Event& event)
{
	DebugF("EventVideo\n");
	return ProcKeepGoing;
}



ProcStatus EventImpl::
Event			(const SDL_Event& event)
{
	 ProcStatus result = ProcKeepGoing;

	switch (event.type) {
	case SDL_ACTIVEEVENT:
		result = EventActivate(event);
		break;
	case SDL_KEYDOWN:
	case SDL_KEYUP:
		result = EventKeyboard(event);
		break;
	case SDL_MOUSEMOTION:
	case SDL_MOUSEBUTTONDOWN:
	case SDL_MOUSEBUTTONUP:
		result = EventMouse(event);
		break;
	case SDL_JOYAXISMOTION:
	case SDL_JOYBALLMOTION:
	case SDL_JOYHATMOTION:
		result = EventJoystick(event);
		break;
	case SDL_USEREVENT:
		result = EventUser(event);
		break;
	case SDL_SYSWMEVENT:
		result = EventSystem(event);
		break;
	case SDL_VIDEORESIZE:
	case SDL_VIDEOEXPOSE:
		result = EventVideo(event);
		break;
	default:
		break;
	}

	
	
	

	return result;
}


