/*
    Copyright (C) SEIKO EPSON CORPORATION 2002-2004. All Rights Reserved.

    3-3-5, Owa, Suwa-shi Nagano-ken, 392-8502 Japan
    Phone: +81-266-52-3131

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public License
    as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/


#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <limits.h>
#include "config.h"

#include "SDL_e21.h"

#include "SDL_util.h"

#include "Grain.h"


#define DUMP_GRAIN(x) fprintf(stderr, "---- prev:%08x this:%08x next:%08x\n", x->m_prev, x, x->m_next)


#define PURGE_GRAIN(x) if ((x)->m_next) ((x)->m_next)->m_prev = (x)->m_prev; if ((x)->m_prev) ((x)->m_prev)->m_next = (x)->m_next;





Grain::
Grain		(SDL_Surface		*screen,
			 const SDL_Rect&	rect)
{
	m_screen		= screen;
	drawable_area	= rect;
	if (!drawable_area.w || !drawable_area.h) {
		drawable_area.w = m_screen->w;
		drawable_area.h = m_screen->h;
	}

	memset(&m_UpdateArea, 0, sizeof(m_UpdateArea));
	
	m_next = m_prev = m_child = m_parent = NULL;

	m_Image	= m_OrigImage = m_BgImage = m_BgRsrc = m_FrameRsrc = NULL;

	m_BgRsrcName	= NULL;

	m_FrameRsrcName	= NULL;

	m_AlphaValue	= UCHAR_MAX;

	m_restore		= true;

	m_active		= true;

	m_dead			= false;
}




Grain::
~Grain()
{
	if (m_restore) {
		RestoreOrigImage();
	}
	delete m_child;
	delete m_next;
	m_next = NULL;
	if (m_prev) {
		m_prev->m_next = NULL;
	}

	if (m_Image)
		::SDL_FreeSurface(m_Image);
	m_Image = NULL;

	if (m_OrigImage)
		::SDL_FreeSurface(m_OrigImage);
	m_OrigImage = NULL;

	if (m_BgImage)
		::SDL_FreeSurface(m_BgImage);
	m_BgImage = NULL;

	if (m_BgRsrc)
		::SDL_FreeSurface(m_BgRsrc);
	m_BgRsrc = NULL;

	if (m_FrameRsrc)
		::SDL_FreeSurface(m_FrameRsrc);
	m_FrameRsrc = NULL;
}



int Grain::  
LoadBgRsrc      ()
{        
	DebugF("Grain::LoadBgRsrc\n");
	 ::SDL_Surface		*tmp = NULL;
	 ::SDL_PixelFormat	*format;
	if (m_BgRsrcName && m_BgRsrcName[0]) { 
		 static char filename[FILENAME_MAX];
		strcpy(filename, RESOURCE_DIR);
		strcat(filename, m_BgRsrcName);
		tmp = SDL_LoadImage(filename, NULL, SDL_ROTATE_0, SDL_STRETCH_BILINEAR, SDL_FALSE);
	}
#if 1
	else { 
		format = m_screen->format;
		tmp = ::SDL_CreateRGBSurface(SDL_SWSURFACE,
		  drawable_area.w, drawable_area.h, format->BitsPerPixel,
		  format->Rmask, format->Gmask, format->Bmask, format->Amask);
		if (!tmp) {
			SDL_ERROR();
			return -1;
		}
		memset(tmp->pixels, 0, format->BytesPerPixel * tmp->w * tmp->h);
	}
#endif

	m_BgRsrc = tmp;

	if (m_BgRsrc) {
		return 0;
	}
	return -1;
}



int Grain::
LoadFrameRsrc	()
{
	return 0;
}




int Grain::  
CreateBgImage   ()
{       
	assert(m_OrigImage != NULL);
	DebugF("Grain::CreateBgImage\n");

	 int result;
	 ::SDL_PixelFormat *format = m_screen->format;
	m_BgImage = ::SDL_CreateRGBSurface(SDL_SWSURFACE,
		    drawable_area.w, drawable_area.h, format->BitsPerPixel,
		    format->Rmask, format->Gmask, format->Bmask, format->Amask);
	if (!m_BgImage) {
		SDL_ERROR();
		return -1; 
	}

	if (m_AlphaValue > 0 && m_AlphaValue < UCHAR_MAX) {
		if (!m_BgRsrc) {
			result = LoadBgRsrc();
			if (result != 0) {
				DebugF("LoadBgRsrc Error\n");
				return result;
			}     
		}
		assert(m_BgRsrc != NULL);
	}

	
	result = ::SDL_BlitSurface(m_OrigImage, NULL, m_BgImage, NULL);
	if (result != 0) {
		SDL_ERROR();
		return result;
	}
	if (m_AlphaValue > 0 && m_BgRsrc != NULL) {
		if (m_AlphaValue < UCHAR_MAX) {
			
			result = ::SDL_SetAlpha(m_BgRsrc, SDL_SRCALPHA|SDL_RLEACCEL, m_AlphaValue);
			if (result != 0) {
				SDL_ERROR();
				return result;
			}
		}
		result = ::SDL_BlitSurface(m_BgRsrc, NULL, m_BgImage, NULL);
		if (result != 0) {
			SDL_ERROR();
			return result;
		}
		
		SDL_FreeSurface(m_BgRsrc);
		m_BgRsrc = NULL;
	}

	if (!m_FrameRsrcName || !m_FrameRsrcName[0]) { 
		return 0;
	}

	
	if (!m_FrameRsrc) {
		result = LoadFrameRsrc();
		if (result != 0) {
			return result;
		}
	}
	assert(m_FrameRsrc != NULL);

	
	
	result = ::SDL_SetColorKey(m_FrameRsrc, SDL_SRCCOLORKEY|SDL_RLEACCEL,
				::SDL_MapRGB(m_FrameRsrc->format, 0xff, 0x00, 0xff));
	if (result != 0) {
		SDL_ERROR();
		return result;
	}

	result = ::SDL_BlitSurface(m_FrameRsrc, NULL, m_BgImage, NULL);
	if (result != 0) {
		SDL_ERROR();
		return result;
	}

	return 0;
}




int Grain::
StoreOrigImage	()
{
	if (m_OrigImage)
		return 0;
	assert(m_screen != NULL);
	DebugF("Grain::StoreOrigImage\n");
	 ::SDL_PixelFormat	*format = m_screen->format;
	 ::SDL_Surface		*surface = NULL;

	surface = ::SDL_CreateRGBSurface(SDL_SWSURFACE,
		    drawable_area.w, drawable_area.h, format->BitsPerPixel,
		    format->Rmask, format->Gmask, format->Bmask, format->Amask);
	if (!surface) { 
		SDL_ERROR();
		return -1;
	}

	 ::SDL_Rect rect = drawable_area;
	 int result = ::SDL_BlitSurface(m_screen, &rect, surface, NULL);
	if (result != 0) {
		SDL_ERROR();
		::SDL_FreeSurface(surface);
		return result;
	}

	m_OrigImage = surface;	

	return 0;
}




int Grain::
RestoreOrigImage	()
{
	if (!m_OrigImage) {
		return 0;
	}
	DebugF("Grain::RestoreOrigImage\n");
	assert(m_screen != NULL);
	 ::SDL_Rect rect = drawable_area;
	 int result = ::SDL_BlitSurface(m_OrigImage, NULL, m_screen, &rect);
	if (result != 0) {
		SDL_ERROR();
		return result;
	}

	return 0;
}




int Grain::
Init		()
{
	 int result;

	result = StoreOrigImage();
	if (result != 0) {
		return result;
	}

	result = CreateBgImage();
	if (result != 0) {
		return result;
	}
	
	 ::SDL_PixelFormat *format = m_BgImage->format;
	m_Image = ::SDL_CreateRGBSurface(SDL_SWSURFACE,
				drawable_area.w, drawable_area.h, format->BitsPerPixel,
				format->Rmask, format->Gmask, format->Bmask, format->Amask);
	if (!m_Image) {
		SDL_ERROR();
		return -1;
	}

	
	result = ::SDL_BlitSurface(m_BgImage, NULL, m_Image, NULL);
	if (result != 0) {
		SDL_ERROR();
		return result;
	}

	return 0;
}




int Grain::
Suspend		()
{
	 int result;
	result = RestoreOrigImage();
	if (result != 0) {
		return result;
	}

	Update();

	return 0;
}




int Grain::
Draw		()
{
	return 0;
}




void Grain::
Update		()
{
	::SDL_UpdateRect(m_screen, drawable_area.x, drawable_area.y,
		drawable_area.w, drawable_area.h);
#if 0
	DebugF(("Grain::UpdateRect x:%d y:%d w:%d h:%d p:%x\n", drawable_area.x,
		drawable_area.y, drawable_area.w, drawable_area.h, (unsigned int)m_screen));
#endif
}




int Grain::  
Clear   (const SDL_Rect& rect)
{       
	 int result = 0;
     SDL_Rect tmpRect = rect;

	if (m_BgImage) {
		result = ::SDL_BlitSurface(m_BgImage, &tmpRect, m_screen, &tmpRect);
	} else if (m_OrigImage) {
		result = ::SDL_BlitSurface(m_OrigImage, &tmpRect, m_screen, &tmpRect);
	}
	if (result != 0) {
		SDL_ERROR();
		return result;
	}
	
	return 0;
}

int Grain::
Redraw		()
{
	DebugF("Grain::Redraw\n");
	 Grain *p = m_parent;

	while (p && p->GetParent()) {
		p = p->GetParent();
	}

	return (p ? p->Redraw() : Draw());
}



int Grain::
Idle		()
{
#if 0
	 int result;
	 Grain *p = m_child;
	while (p) {
		result = p->Idle();
		if (result != 0) {
			return result;	
		}
		p = p->GetNext();
	}
#endif
	usleep(10);
	return 0;
}



ProcStatus Grain::
EventUser       (const SDL_Event& event)
{
    DebugF("EventUser\n");
    if (event.user.code != SDL_E21_MAGIC) {
        return ProcKeepGoing;
    }

    if (!event.user.data1) {
        return ProcKeepGoing;
    }

     SDL_E21Event *e21ev = static_cast<SDL_E21Event *>(event.user.data1);
    switch(e21ev->code) {
    case EV_LANG:
        UIRsrcMgr::Instance()->SetLang(e21ev->param[0]);
        Redraw();
		break;
    default:
        break;
    }

    free(event.user.data1);
    ((SDL_Event)event).user.data1 = NULL;

    return ProcKeepGoing;
}





void Grain::
SetNext(Grain *node)
{
	m_next = node;
	if (m_next) {
		m_next->m_prev = this;
	}
}




void Grain::
SetPrev(Grain *node)
{
	m_prev = node;
	if (m_prev) {
		m_prev->m_next = this;
	}
}




void Grain::
SwapGrain(Grain *a, Grain*b)
{
	assert(a && b);
	assert(a != b);

	 Grain *tmp;
	tmp = b->m_next;
	if (tmp) {
		tmp->m_prev = a;
	}
	if (a->m_next) {
		a->m_next->m_prev = b;
	}
	b->m_next = a->m_next;
	a->m_next = tmp;

	tmp = b->m_prev;
	if (tmp) {
		tmp->m_next = a;
	}
	if (a->m_prev) {
		a->m_prev->m_next = b;
	}
	b->m_prev = a->m_prev;
	a->m_prev = tmp;
}




int Grain::
MoveGrain	(Grain	*obj,
			 int	distance)
{
	if (!obj) {
		return -1;
	}
	if (distance == 0) {
		return 0;
	}

	
	 Grain *tmp;
	 Grain *needle = obj;
	if (distance > 0) {
		for (int i = 0; i < distance; i++) {
			needle = needle->m_next;
			if (!needle) {
				return -1;
			}
		}

		PURGE_GRAIN(obj);

		tmp = needle->m_next;
		needle->m_next = obj;
		obj->m_prev = needle;
		if (tmp) {
			tmp->m_prev = obj;
		}	
		obj->m_next = tmp;

	} else {
		for (int i = 0; i > distance; i--) {
			needle = needle->m_prev;
			if (!needle) {
				return -1;
			}
		}

		PURGE_GRAIN(obj);

		tmp = needle->m_prev;
		needle->m_prev = obj;
		obj->m_next = needle;
		if (tmp) {
			tmp->m_next = obj;
		}
		obj->m_prev = tmp;
	}

	return 0;
}




int Grain::
GetIndex		(Grain *node)
{
	assert(node);
	 int i = 0;
	 Grain *p = GetTopGrain(node);
	while (p) {
		if (node == p) {
			return i;
		}
		i++;
		p = p->m_next;
	}
	return -1;
}




Grain* Grain::
GetBottomGrain	(Grain *node)
{
	 Grain *p = node;
	while (p && p->m_next) {
		p = p->m_next;
	}
	return p;
}




Grain* Grain::
GetTopGrain		(Grain *node)
{
	 Grain *p = node;
	while (p && p->m_prev) {
		p = p->m_prev;
	}
	return p;
}




void Grain::
AddGrain		(Grain *node)
{
	assert(node != NULL);

	node->m_parent = this;

	 Grain *p = GetBottomGrain(m_child);
	if (p) {
		p->m_next = node;
		node->m_prev = p;
	} else {
		m_child = node;
	}
}




void Grain::
DeleteGrain	(Grain *node)
{
	assert(node != NULL);
	assert(m_child != NULL);

	 Grain *p = m_child;
	while (p) {
		if (p != node) {
			p = p->m_next;
			continue;
		}

		PURGE_GRAIN(p);

		if (m_child == p) {
			m_child = p->m_next;
		}
		break;
	}
}




