/*
    Copyright (C) SEIKO EPSON CORPORATION 2002-2004. All Rights Reserved.

    3-3-5, Owa, Suwa-shi Nagano-ken, 392-8502 Japan
    Phone: +81-266-52-3131

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public License
    as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/




#include "SDL_stretch.h"
#include "config.h"
#include "InfoBar.h"


InfoBar::InfoBar(::SDL_Surface *screen,
                 const ::SDL_Rect&	rect) : Grain(screen, rect)
{
	m_UpdateArea = drawable_area;    
    m_FgImage = 0;
}


InfoBar::~InfoBar()
{
    
    if ( m_FgImage ) {
        SDL_FreeSurface(m_FgImage);
    }
}


int
InfoBar::Init()
{
    int result;
	result = StoreOrigImage();
	if (result != 0) {
		return result;
	}

	return 0;
}



int 
InfoBar::Draw()
{
    int result;

    
    if ( m_BgImage ) {
        result = ::SDL_BlitSurface(m_BgImage, NULL, m_screen, &m_UpdateArea);
        if (result != 0) {
            SDL_ERROR();
            return result;
        }

        
        if ( m_FgImage ) {
            result = ::SDL_BlitSurface(m_FgImage, NULL, m_screen, &m_FgRect);
            if (result != 0) {
                SDL_ERROR();
                return result;
            }
        }
    }

	return 0;
}



int 
InfoBar::SetBackground(::SDL_Surface* bg)
{
    assert( bg != 0 );

    if ( m_BgImage ) {
        SDL_FreeSurface(m_BgImage);
    }

#if 0 
    m_BgImage = SDL_DisplayFormat(bg);
#else
    m_BgImage = SDL_ConvertSurface(bg, bg->format, bg->flags);
#endif
    if ( m_BgImage == 0 ) {
        SDL_ERROR();
        return -1;
    }
    
    
    if ( m_BgImage->w != drawable_area.w ||
         m_BgImage->h != drawable_area.h ) {

        
        SDL_PixelFormat* fmt = m_BgImage->format;
        SDL_Surface* tmp;
        tmp = SDL_CreateRGBSurface(SDL_SWSURFACE, 
                                   drawable_area.w, drawable_area.h,
                                   fmt->BitsPerPixel,
                                   fmt->Rmask, fmt->Gmask,
                                   fmt->Bmask, fmt->Amask);
        if ( tmp == 0 ) {
            SDL_ERROR();
            SDL_FreeSurface(m_BgImage);
            m_BgImage = 0;
            return -1;
        }

        
        if ( m_BgImage->flags & SDL_SRCCOLORKEY ) {
            SDL_SetColorKey(tmp, 
                            m_BgImage->flags & (SDL_SRCCOLORKEY|SDL_RLEACCEL),
                            fmt->colorkey);
        }

        
        if ( m_BgImage->flags & SDL_SRCALPHA ) {
            SDL_SetAlpha(tmp, 
                         m_BgImage->flags & (SDL_SRCALPHA|SDL_RLEACCEL),
                         fmt->alpha);
        }

        
        int ret = SDL_StretchBlit(m_BgImage, NULL,
                                  tmp, NULL, SDL_STRETCH_BILINEAR);
        SDL_FreeSurface(m_BgImage);
        m_BgImage = 0;
        if ( ret != 0 ) {
            SDL_ERROR();
            SDL_FreeSurface(tmp);
            return -2;            
        }
        
        m_BgImage = tmp;
    }

    return 0;
}



int 
InfoBar::SetForeground(::SDL_Surface* fg, ::SDL_Rect& rect)
{
    assert( fg != 0 );

    if ( m_FgImage ) {
        SDL_FreeSurface(m_FgImage);
    }

#if 0 
    m_FgImage = SDL_DisplayFormat(fg);
#else
    m_FgImage = SDL_ConvertSurface(fg, fg->format, fg->flags);
#endif
    if ( m_FgImage == 0 ) {
        SDL_ERROR();
        return -1;
    }

    
    m_FgRect.w = rect.w;
    m_FgRect.h = rect.h;
    m_FgRect.x = rect.x + m_UpdateArea.x;
    m_FgRect.y = rect.y + m_UpdateArea.y;

    
    if ( m_FgImage->w != rect.w ||
         m_FgImage->h != rect.h ) {

        
        SDL_PixelFormat* fmt = m_FgImage->format;
        SDL_Surface* tmp;
        tmp = SDL_CreateRGBSurface(SDL_SWSURFACE, rect.w, rect.h,
                                   fmt->BitsPerPixel,
                                   fmt->Rmask, fmt->Gmask,
                                   fmt->Bmask, fmt->Amask);
        if ( tmp == 0 ) {
            SDL_ERROR();
            SDL_FreeSurface(m_FgImage);
            m_FgImage = 0;
            return -1;
        }

        
        if ( m_FgImage->flags & SDL_SRCCOLORKEY ) {
            SDL_SetColorKey(tmp, 
                            m_FgImage->flags & (SDL_SRCCOLORKEY|SDL_RLEACCEL),
                            fmt->colorkey);
        }

        
        if ( m_FgImage->flags & SDL_SRCALPHA ) {
            SDL_SetAlpha(tmp, 
                         m_FgImage->flags & (SDL_SRCALPHA|SDL_RLEACCEL),
                         fmt->alpha);
        }

        
        int ret = SDL_StretchBlit(m_FgImage, NULL,
                                  tmp, NULL, SDL_STRETCH_BILINEAR);
        SDL_FreeSurface(m_FgImage);
        m_FgImage = 0;
        if ( ret != 0 ) {
            SDL_ERROR();
            SDL_FreeSurface(tmp);
            return -2;            
        }
        
        m_FgImage = tmp;
    }

    return 0;
}
