/*
    Copyright (C) SEIKO EPSON CORPORATION 2002-2004. All Rights Reserved.

    3-3-5, Owa, Suwa-shi Nagano-ken, 392-8502 Japan
    Phone: +81-266-52-3131

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public License
    as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include <limits.h>
#include "config.h"

#include "NavGuide.h"








NavGuide::
NavGuide	(::SDL_Surface		*screen,
			 const ::SDL_Rect&	rect)
			 : Grain(screen, rect)
{
	m_BgRsrcName = "navBackground.png";
	m_AlphaValue = (UCHAR_MAX * 8) / 10;
}

NavGuide::
~NavGuide	()
{
}

int NavGuide::
Init		()
{
	 const static ::SDL_Rect navItemRects[] = {
		{9,  9,48,25},
		{204,9,48,25},
		{398,9,48,25},
		{592,9,48,25},
		{786,9,48,25}
	 };

	if (m_Image) {
		SDL_FreeSurface(m_Image);
		m_Image = NULL;
	}
	if (m_OrigImage) {
		SDL_FreeSurface(m_OrigImage);
		m_OrigImage = NULL;
	}
	if (m_BgImage) {
		SDL_FreeSurface(m_BgImage);
		m_BgImage = NULL;
	}

	 int result;
	result = Grain::Init();
	if (result != 0) {
		return result;
	}

	 Grain *p;
	for (int i = 0; i < (int)(sizeof(navItemRects)/sizeof(::SDL_Rect)); i++) {
		p = new NavItem(m_Image, navItemRects[i]);
		result = p->Init();
		if (result != 0) {
			return result;
		}
		AddGrain(p);
	}

	m_UpdateArea = drawable_area;
	return 0;
}

int NavGuide::
Draw		()
{
	DebugF("NavGuide::Draw()\n");
	 int result;
	result = ::SDL_BlitSurface(m_BgImage, NULL, m_Image, NULL);
	if (result != 0) {
		SDL_ERROR();
		return result;
	}

	 Grain *p = m_child;
	while (p) {
		result = p->Draw();
		if (result != 0) {
			return result;
		}
		p = p->GetNext();
	}

	result = ::SDL_BlitSurface(m_Image, NULL, m_screen, &m_UpdateArea);
	if (result != 0) {
		SDL_ERROR();
		return result;
	}

	return 0;
}




int NavGuide::
SetIcon			(const IconID				id,
				 const NavItem::Icon		icon)
{
	 Grain *p = m_child;
	for (int i = 0; p != NULL; i++) {
		if ((IconID)i == id) {
			return ((NavItem *)p)->SetIcon(icon);
		}
		p = p->GetNext();
	}
	return 0;
}




int NavGuide::
SetDescription	(const IconID				id,
				 const NavItem::Description	descr)
{
	 Grain *p = m_child;
	for (int i = 0; p != NULL; i++) {
		if ((IconID)i == id) {
			return ((NavItem *)p)->SetDescription(descr);
		}
		p = p->GetNext();
	}
	return 0;
}




int NavGuide::
SetIconDescr	(const IconID				id,
				 const NavItem::Icon		icon,
				 const NavItem::Description	descr)
{
	 int result;
	result = SetIcon(id, icon);
	if (result != 0) {
		return result;
	}
	result = SetDescription(id, descr);
	if (result != 0) {
		return result;
	}
	return 0;
}


