/*
    Copyright (C) SEIKO EPSON CORPORATION 2002-2004. All Rights Reserved.

    3-3-5, Owa, Suwa-shi Nagano-ken, 392-8502 Japan
    Phone: +81-266-52-3131

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public License
    as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include "config.h"
#include <string.h>
#include <SDL.h>

#include "NavItem.h"

const int DESCR_PADDING  = 8;

const char *BtnRsrcName[] = {"btn_enter.bmp", "btn_esc.bmp", "btn_cursor.bmp", "btn_pup.bmp", "btn_pdown.bmp"};

NavItem::
NavItem		(SDL_Surface		*screen,
			 const SDL_Rect&	rect)
			: Grain(screen, rect)
{
	m_icon = IconHidden;
	m_descr = DescrNone;
}

NavItem::
~NavItem	()
{
}


int NavItem::
Init		()
{
	return 0;
}


int NavItem::
Draw		()
{
	if (m_icon == IconHidden || m_descr == DescrNone) {
		return 0;
	}

	 int result;
	 ::SDL_Rect rect = drawable_area;
	 ::SDL_Surface *icon = GetResource(m_icon);
	if (icon) {
		rect.w = icon->w;
		rect.h = icon->h;
		result = ::SDL_BlitSurface(GetResource(m_icon), NULL, m_screen, &rect);
		if (result != 0) {
			SDL_ERROR();
			return result;
		}
	}

	if (m_descr == DescrNone) {
		return 0;
	}

	 SDL_Surface *descr = GetResource(m_descr);
	if (descr && (rect.h > descr->h)) {
		rect.x += rect.w + DESCR_PADDING; 
		rect.y += (rect.h - descr->h) / 2; 
		rect.w = descr->w;
		rect.h = descr->h;
		result = ::SDL_BlitSurface(descr, NULL, m_screen, &rect);
		if (result != 0) {
			SDL_ERROR();
			return result;
		}
	}

	return 0;
}

int NavItem::
SetIcon			(const Icon	icon)
{
	if (m_icon != icon) {
		m_icon = icon;
		return Draw();
	}
	return 0;
}

int NavItem::
SetDescription	(const Description descr)
{
	if (m_descr != descr) {
		m_descr = descr;
		return Draw();
	}
	return 0;
}

SDL_Surface* NavItem::
GetResource		(const Description descr) const
{
	 const static struct conv_t {
		Description		descr;
		UIRsrcMgr::Rsrc	rsrc;
	} converter[] = {
		{DescrDecide,	UIRsrcMgr::navTagDecide},
		{DescrSelect,	UIRsrcMgr::navTagSelect},
		{DescrNextPage,	UIRsrcMgr::navTagNextPage},
		{DescrPrevPage,	UIRsrcMgr::navTagPrevPage},
		{DescrBack,		UIRsrcMgr::navTagBack},
		{DescrCancel,	UIRsrcMgr::navTagCancel},
		{DescrOpFolder,	UIRsrcMgr::navTagOpFolder},
		{DescrProperty,	UIRsrcMgr::navTagProperty},
		{DescrPlMovie,	UIRsrcMgr::navTagPlMovie},
		{DescrPlPict,	UIRsrcMgr::navTagPlPict},
		{DescrPlScenario,UIRsrcMgr::navTagPlScenario},
		{DescrToUp,		UIRsrcMgr::navTagToUp},
		{DescrMove,		UIRsrcMgr::navTagMove},
		{DescrToTop,	UIRsrcMgr::navTagToTop},
		{DescrMoveCursor,UIRsrcMgr::navTagMoveCursor},
		{DescrRotate,	UIRsrcMgr::navTagRotation},
		{DescrSetting,	UIRsrcMgr::navTagSetting},
	};

	for (int i = 0; i < (int)(sizeof(converter)/sizeof(conv_t)); i++) {
		if (converter[i].descr != descr) {
			continue;
		}
		return UIRsrcMgr::Instance()->GetResource(converter[i].rsrc);
	}
	return NULL;
}

SDL_Surface* NavItem::
GetResource		(const Icon icon) const
{
	 const static struct conv_t {
		Icon			icon;
		UIRsrcMgr::Rsrc	rsrc;
	} converter[] = {
		{IconEnter,		UIRsrcMgr::navIconEnter},
		{IconESC,		UIRsrcMgr::navIconEscape},
		{IconCursor,	UIRsrcMgr::navIconCursors},
		{IconPageUp,	UIRsrcMgr::navIconPageUp},
		{IconPageDown,	UIRsrcMgr::navIconPageDown},
		{IconVert,	    UIRsrcMgr::navIconArwVert},
		{IconHoriz,	    UIRsrcMgr::navIconArwHorz}
	};

	for (int i = 0; i < (int)(sizeof(converter)/sizeof(conv_t)); i++) {
		if (converter[i].icon != icon) {
			continue;
		}
		return UIRsrcMgr::Instance()->GetResource(converter[i].rsrc);
	}
	return NULL;
}

