/*
    Copyright (C) SEIKO EPSON CORPORATION 2002-2004. All Rights Reserved.

    3-3-5, Owa, Suwa-shi Nagano-ken, 392-8502 Japan
    Phone: +81-266-52-3131

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public License
    as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include "config.h"

#include "RadioBtnItem.h"
#include "RadioBtnGrp.h"

RadioBtnGrp::
RadioBtnGrp			(const int			items,
					 const int			default_val,
					 ::SDL_Surface		*screen,
					 const ::SDL_Rect&	rect)
					: BaseGroup(items, screen, rect)
{
	m_default	= default_val;
}

RadioBtnGrp::
~RadioBtnGrp()
{
}

int RadioBtnGrp::
Activate	()
{
	Grain::Activate();
#if 0
	 Grain *p = m_child;
	while (p) {
		assert(dynamic_cast<Item *>(p));
		
		if (reinterpret_cast<Item *>(p)->IsSelected()) {
			return p->Activate();
			result = p->Draw();
			if (result != 0) {
				return result;
			}
		}
		p = p->GetNext();
	}
#else
	 Grain *p = Item::GetSelectedItem(m_child);
	if (p) {
		return p->Activate();
	}
#endif
	return 0;
}

int RadioBtnGrp::
Deactivate	()
{
	Grain::Deactivate();
#if 0
	 Grain *p = m_child;
	while (p) {
		assert(dynamic_cast<Item *>(p));
		if (reinterpret_cast<Item *>(p)->IsSelected()) {
			return p->Deactivate(); 
			result = p->Draw();
			if (result != 0) {
				return result;
			}
		}
		p = p->GetNext();
	}
#else
	 Grain *p = Item::GetSelectedItem(m_child);
	if (p) {
		return p->Activate();
	}
#endif
	return 0;
}

int RadioBtnGrp::
Init		()
{
	 int result;

	 Grain		*p;
	 ::SDL_Rect rect = RADIOBTNITEM_DEFAULT_RECT; 
	rect.x = drawable_area.x;
	rect.y = drawable_area.y;

	for (int i = 0; i < m_items; i++) {
		p = new RadioBtnItem(m_screen, rect);
		AddGrain(p);
		result = p->Init();
		if (result != 0) {
			return result;
		}
		rect.y += 40; 
	}

	
	p = m_child;
	for (int i = 0; (i <= m_default) && p; i++) {
		if (m_default == i) {
			reinterpret_cast<RadioBtnItem *>(p)->On(false);
		}
		p = p->GetNext();
	}

	return 0;
}

int RadioBtnGrp::
Draw		()
{
	 int result;
	 Grain *p = m_child;
	while (p) {
		result = p->Draw();
		if (result != 0) {
			return result;
		}
		p = p->GetNext();
	}
	return 0;
}

void RadioBtnGrp::
Update		()
{
	UpdateRect(m_screen, drawable_area);
}

int RadioBtnGrp::
Select		(bool top) 
{
	Item::Select();
	if (!m_child) { 
		return -1; 
	} 

	 Grain *p = m_child;
	if (top) { 
		while (p && p->GetNext()) {
			p = p->GetNext();
		}
	}
	assert(dynamic_cast<Item *>(p));
	return reinterpret_cast<Item *>(p)->Select();
}

ProcStatus RadioBtnGrp::
Event		(const ::SDL_Event&	event)
{
	 Grain *p = m_child;
	while (p) {
		assert(dynamic_cast<Item *>(p));
		if (reinterpret_cast<Item *>(p)->IsSelected()) {
			return p->Event(event);
		}
		p = p->GetNext();
	}

	return ProcKeepGoing;
}


int RadioBtnGrp::
GetValue		()
{
	 int val = 0;
	 Grain *p = m_child;
	while (p) {
		assert(dynamic_cast<RadioBtnItem *>(p));
		if (reinterpret_cast<RadioBtnItem *>(p)->IsOn()) {
			return val;
		}
		val++;
		p = p->GetNext();
	}
	return -1;
}

