/*
    Copyright (C) SEIKO EPSON CORPORATION 2002-2004. All Rights Reserved.

    3-3-5, Owa, Suwa-shi Nagano-ken, 392-8502 Japan
    Phone: +81-266-52-3131

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public License
    as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include "config.h"

#include "BtnBox.h"

#include "RadioBtnGrp.h"
#include "RadioBtnItem.h"


RadioBtnItem::
RadioBtnItem		(::SDL_Surface 		*screen,
					 const ::SDL_Rect&	rect)
					: Item(screen, rect)
{
	m_on		= false;
	m_UpdateArea = drawable_area;
}

RadioBtnItem::
~RadioBtnItem()
{
}

int RadioBtnItem::
Init				()
{
#if 0
	 int result;
	result = StoreOrigImage();
	if (result != 0) {
		return result;
	}
#endif
	return 0;
}

int RadioBtnItem::
Draw				()
{
	 int result;


	assert(m_parent);
	assert(m_parent->GetParent());
	assert(m_parent->GetParent()->GetParent());
	m_parent->GetParent()->GetParent()->Clear(drawable_area);

	 ::SDL_Rect rect = m_UpdateArea;
	rect.x += 3; 
	rect.y += 4;
	result = ::SDL_BlitSurface(UIRsrcMgr::Instance()->GetResource(
		m_on ? UIRsrcMgr::rbtnBtnOn : UIRsrcMgr::rbtnBtnOff),
		NULL, m_screen, &rect);
	if (result != 0) {
		SDL_ERROR();
		return result;
	}

	assert(BtnBox::Instance());
	if (!m_selected || BtnBox::Instance()->GetTarget()) {
		return 0;
	}

	
	 ::SDL_Surface *surface = UIRsrcMgr::Instance()->GetResource(UIRsrcMgr::rbtnBtnSelector);
	if (!surface) {
		return -1;
	}
	result = ::SDL_BlitSurface(surface, NULL, m_screen, &m_UpdateArea);
	if (result != 0) {
		SDL_ERROR();
		return result;
	}

	return 0;
}


void RadioBtnItem::	
Update				()
{
	UpdateRect(m_screen, m_UpdateArea);
}

#if 0
int RadioBtnItem::
Select				()
{
	m_selected = true;
	return 0;
}

int RadioBtnItem::
Unselect			()
{
	m_selected = false;
	return 0;
}
#endif

int RadioBtnItem::
On					(bool immediate)
{
	if (m_on) {
		return 0;
	}
	 Grain *p = GetTopGrain(this);
	while (p) {
		if (reinterpret_cast<RadioBtnItem *>(p)->m_on == true) {
			reinterpret_cast<RadioBtnItem *>(p)->Off();
			break;	
		}
		p = p->GetNext();
	}
	m_on = true;
	Draw();
	if (immediate) {
		Update();
	}
	return 0;
}

int RadioBtnItem::
Off					(bool immediate)
{
	m_on = false;
	Draw();
	if (immediate) {
		Update();
	}
	return 0;
}

ProcStatus RadioBtnItem::
DoUp				()
{
	 Item *p;
	if (m_prev) {
		
		assert(dynamic_cast<Item *>(m_prev));
		p = dynamic_cast<Item *>(m_prev);
		p->Select();
	} else if (m_parent && m_parent->GetPrev()) {
		
		assert(dynamic_cast<Item *>(m_parent->GetPrev()));
		p = reinterpret_cast<Item *>(m_parent->GetPrev());

		assert(dynamic_cast<RadioBtnGrp *>(m_parent));
		reinterpret_cast<RadioBtnGrp *>(m_parent)->Unselect();
		reinterpret_cast<RadioBtnGrp *>(p)->Select(true);
	} else {
		DoPageUp();
		return ProcKeepGoing;
	}
	Unselect();
	p->Draw();
	Draw();
	p->Update();
	Update();

	return ProcKeepGoing;
}

ProcStatus RadioBtnItem::
DoDown				()
{
	 Item *p;
	if (m_next) {
		
		assert(dynamic_cast<Item *>(m_next));
		p = dynamic_cast<Item *>(m_next);
		p->Select();
	} else if (m_parent && m_parent->GetNext()){
		
		assert(dynamic_cast<Item *>(m_parent->GetNext()));
		p = reinterpret_cast<Item *>(m_parent->GetNext());

		assert(dynamic_cast<RadioBtnGrp *>(m_parent));
		reinterpret_cast<RadioBtnGrp *>(m_parent)->Unselect();
		reinterpret_cast<RadioBtnGrp *>(p)->Select(false);
	} else {
		
		DoPageUp();
		return ProcKeepGoing;
	}
	Unselect();

	p->Draw();
	Draw();

	p->Update();
	Update();
	return ProcKeepGoing;
}

ProcStatus RadioBtnItem::
DoEnter				()
{
	On();
	Flash(UIRsrcMgr::rbtnBtnSelectorOn, UIRsrcMgr::rbtnBtnSelector);
	Update();
	return ProcKeepGoing;
}


ProcStatus RadioBtnItem::
DoPageUp			()
{
	
	BtnBox::Instance()->SetTarget((View *)(m_parent->GetParent()));

	return ProcKeepGoing;
}

ProcStatus RadioBtnItem::
DoPageDown			()
{
	return ProcKeepGoing;
}

void RadioBtnItem::
SetDrawableArea(const ::SDL_Rect& rect)
{
	Grain::SetDrawableArea(rect);
	m_UpdateArea = rect;
}
