/*
    Copyright (C) SEIKO EPSON CORPORATION 2002-2004. All Rights Reserved.

    3-3-5, Owa, Suwa-shi Nagano-ken, 392-8502 Japan
    Phone: +81-266-52-3131

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public License
    as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/
#include <SDL.h>

#include "config.h"
#include "UIRsrcMgr.h"

#include "SDL_text.h"

#include "UIUtils.h"




int UIUtils::
DrawSurface(SDL_Surface *screen, SDL_Surface *surface, int x, int y, Xalign xalign, Yalign yalign)
{
	 SDL_Rect rect;
	rect.x = x;
	rect.y = y;
	rect.w = surface->w;
	rect.h = surface->h;

	switch (xalign) {
	case XALIGN_CENTER: rect.x -= rect.w / 2; break;
	case XALIGN_RIGHT: rect.x -= rect.w; break;
	case XALIGN_LEFT:
	default: break;
	}

	switch (yalign) {
	case YALIGN_CENTER: rect.y -= rect.h / 2; break;
	case YALIGN_BOTTOM: rect.y -= rect.h; break;
	case YALIGN_TOP:
	default: break;
	}

	int result;
	result = ::SDL_BlitSurface(surface, NULL, screen, &rect);
#ifdef SHOW_BASE_LINE
	{
	 SDL_Rect rect;
	rect.x = 50; rect.y = y; rect.w = screen->w - 100; rect.h = 1;
	SDL_FillRect(screen, &rect, SDL_MapRGB(screen->format, 0xff, 0, 0));
	rect.y = 50; rect.h = screen->h - 100; rect.x = x; rect.w = 1;
	SDL_FillRect(screen, &rect, SDL_MapRGB(screen->format, 0xff, 0, 0xff));
	}
#endif

#ifdef SHOW_TOMBO
	{
	SDL_Rect tmp;
	tmp.w = 1; tmp.h = 10; tmp.x = rect.x; tmp.y = rect.y - 10; 
	SDL_FillRect(screen, &tmp, SDL_MapRGB(screen->format, 0xff, 0xff, 0xff));
	tmp.y = rect.y + rect.h;
	SDL_FillRect(screen, &tmp, SDL_MapRGB(screen->format, 0xff, 0xff, 0xff));
	tmp.x = rect.x + rect.w;
	SDL_FillRect(screen, &tmp, SDL_MapRGB(screen->format, 0xff, 0xff, 0xff));
	tmp.y = rect.y - 10;
	SDL_FillRect(screen, &tmp, SDL_MapRGB(screen->format, 0xff, 0xff, 0xff));

	tmp.h = 1; tmp.x = rect.x - 10; tmp.y = rect.y; tmp.w = 10;
	SDL_FillRect(screen, &tmp, SDL_MapRGB(screen->format, 0xff, 0xff, 0xff));
	tmp.y = rect.y + rect.h;
	SDL_FillRect(screen, &tmp, SDL_MapRGB(screen->format, 0xff, 0xff, 0xff));
	tmp.x = rect.x + rect.w;
	SDL_FillRect(screen, &tmp, SDL_MapRGB(screen->format, 0xff, 0xff, 0xff));
	tmp.y = rect.y;
	SDL_FillRect(screen, &tmp, SDL_MapRGB(screen->format, 0xff, 0xff, 0xff));
	
	}
#endif
	return result;
}

int UIUtils::
DrawResource(SDL_Surface *screen, UIRsrcMgr::Rsrc rsrc, int x, int y, Xalign xalign, Yalign yalign)
{
	 SDL_Surface *surface;
	surface = UIRsrcMgr::Instance()->GetResource(rsrc);
	if (!surface) {
		fprintf(stderr, "Could not load resource\n");
		return -1;
	}

	return DrawSurface(screen, surface, x, y, xalign, yalign);
}

int UIUtils::
DrawResource(SDL_Surface *screen, const struct RsrcParam* rsrcs, size_t num)
{
	 int result;
	for (size_t i = 0; i < num; i++) {
		result = UIUtils::DrawResource(screen, rsrcs[i].rsrc,
				rsrcs[i].x, rsrcs[i].y, rsrcs[i].xalign, rsrcs[i].yalign);
		if (result != 0) {
			return -1;
		}
	}
	return 0;
}

int UIUtils::
DrawText(SDL_Surface *screen, const char *text, int x, int y, Xalign xalign, Yalign yalign)
{
	 SDL_Surface *str = SDL_CreateText(text, RESOURCE_DIR "fonts/epson0820b.pcf");
	if (!str) {
		return -1;
	}

	DrawSurface(screen, str, x, y, xalign, yalign);
	
	SDL_FreeSurface(str);
	return 0;
}
