/*
    Copyright (C) SEIKO EPSON CORPORATION 2002-2004. All Rights Reserved.

    3-3-5, Owa, Suwa-shi Nagano-ken, 392-8502 Japan
    Phone: +81-266-52-3131

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public License
    as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/


#include "config.h"
#include "SDL_util.h"
#include "VarNavGuide.h"

VarNavGuide::VarNavGuide(SDL_Surface *screen,
                         const SDL_Rect& rect)
    : NavGuide(screen,rect)
{
    m_BgRsrcName = 0;
}

VarNavGuide::~VarNavGuide()
{    
}

int
VarNavGuide::LoadBgRsrc()
{
    DebugF("%s: Entering\n", __PRETTY_FUNCTION__);
    
    SDL_PixelFormat* format = m_screen->format;
    m_BgRsrc = SDL_CreateRGBSurface(SDL_SWSURFACE,
                                    drawable_area.w, drawable_area.h,
                                    format->BitsPerPixel,
                                    format->Rmask,
                                    format->Gmask,
                                    format->Bmask,
                                    format->Amask);
    if ( m_BgRsrc == 0 ) {
        SDL_ERROR();
        return -1;
    }

    
    SDL_Surface* leftImage;
    leftImage = SDL_LoadImage(RESOURCE_DIR "navBarEndLeft.png", 
                              NULL, SDL_ROTATE_0, 
                              SDL_STRETCH_BILINEAR, SDL_FALSE);
    if ( leftImage == 0 ) {
        DebugF("%s: Unable to load %s",     
                   __PRETTY_FUNCTION__, "navBarEndLeft.png\n");
        SDL_FreeSurface(m_BgRsrc);
        m_BgRsrc = 0;
        return -1;
    }
    SDL_Rect leftRect = { 0, 0, leftImage->w, leftImage->h };
    SDL_BlitSurface(leftImage, NULL, m_BgRsrc, &leftRect);
    SDL_FreeSurface(leftImage);
    leftImage = 0;

    
    SDL_Surface* rightImage;
    rightImage = SDL_LoadImage(RESOURCE_DIR "navBarEndRight.png", 
                               NULL, SDL_ROTATE_0, 
                               SDL_STRETCH_BILINEAR, SDL_FALSE);
    if ( rightImage == 0 ) {
        DebugF("%s: Unable to load %s",     
                   __PRETTY_FUNCTION__, "navBarEndRight.png\n");
        SDL_FreeSurface(m_BgRsrc);
        m_BgRsrc = 0;
        return -1;
    }
    SDL_Rect rightRect = { drawable_area.w - rightImage->w, 0, 
                           rightImage->w, 
                           rightImage->h };
    SDL_BlitSurface(rightImage, NULL, m_BgRsrc, &rightRect);
    SDL_FreeSurface(rightImage);
    rightImage = 0;

    
    SDL_Surface* midImage;
    midImage = SDL_LoadImage(RESOURCE_DIR "navBarMiddle.png", 
                             NULL, SDL_ROTATE_0, 
                             SDL_STRETCH_BILINEAR, SDL_FALSE);
    if ( midImage == 0 ) {
        DebugF("%s: Unable to load %s",     
                   __PRETTY_FUNCTION__, "navBarMiddle.png\n");
        SDL_FreeSurface(m_BgRsrc);
        m_BgRsrc = 0;
        return -1;
    }
    SDL_Rect midRect = { leftRect.w, 0, 
                         (drawable_area.w - rightRect.w - leftRect.w), 
                         midImage->h };
    SDL_StretchBlit(midImage, NULL, m_BgRsrc, &midRect, SDL_STRETCH_BILINEAR);
    SDL_FreeSurface(midImage);
    midImage = 0;

    DebugF("%s: Leaving\n", __PRETTY_FUNCTION__);
    return 0;
}
