/*
    Copyright (C) SEIKO EPSON CORPORATION 2002-2004. All Rights Reserved.

    3-3-5, Owa, Suwa-shi Nagano-ken, 392-8502 Japan
    Phone: +81-266-52-3131

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public License
    as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include "config.h"

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <stdarg.h>
#include <limits.h>

#include "SDL_util.h"

#include "libkeyinput.h"

#include "helper.h"



#if 0
#if _DEBUG
int
debugf(const char *format, ...)
{
	va_list args;
	va_start(args, format);
	vfprintf(stderr, format, args);
	va_end(args);
	return 0;
}
#endif
#endif

long
difftimeval(const struct timeval *a, const struct timeval *b)
{
	 long tmp;
	 const struct timeval *x, *y;
	if (a->tv_sec > b->tv_sec) {
		x = a;
		y = b;
	} else if (a->tv_sec < b->tv_sec) {
		x = b;
		y = a;
	} else {
		if (a->tv_usec > b->tv_usec) {
			x = a;
			y = b;
		} else {
			y = a;
			x = b;
		}
	}
	tmp = (x->tv_sec - y->tv_sec) * 1000;
	if (x->tv_usec > y->tv_usec) {
		tmp += (x->tv_usec - y->tv_usec) / 1000;
	} else {
		tmp -= (y->tv_usec - x->tv_usec) / 1000;
	}
	return tmp;	
}


int
draw_raw_rsrc(SDL_Surface *screen, const raw_rsrc_t *rsrc, int nums)
{
	 int i;
     int result;
     SDL_Surface *surface;
     SDL_Rect rect;
	 char tmp[FILENAME_MAX + 1];
    assert(screen);
    assert(rsrc);

    for (i = 0; i < nums; i++) {
        assert(rsrc[i].name);

        strcpy(tmp, RESOURCE_DIR);
        strcat(tmp, rsrc[i].name);

        surface = SDL_LoadImage(tmp, NULL,
				SDL_ROTATE_0, SDL_STRETCH_BILINEAR, SDL_TRUE);
		if (surface == NULL) {
			printf("cannot resoruce %s\n", tmp);
			return -1;
        }

        rect.x = rsrc[i].x;
        rect.y = rsrc[i].y;
        rect.w = surface->w;
        rect.h = surface->h;

		result = SDL_BlitSurface(surface, NULL, screen, &rect);

		SDL_FreeSurface(surface);
		if (result != 0) {
			SDL_ERROR();
			return -1;
        }
    }

    return 0;
}


Uint8
get_lang()
{
	 int i, len;
	 unsigned long tmp;
	 char *buf = NULL;
     int retry;

     for ( retry = 0; retry < 10; retry++ ) {
         len = LKI_ComSequence("_LANG?\r", &buf);
         if (len < 3) {
             DebugF(("not enough packet %d\n", len));
             goto bail;
         }
         buf[len - 2] = '\0';
         len -= 2;

         
         if (strncmp(buf, "_LANG", 5) == 0) {
             break;
         }

         
         if (buf) {
             free(buf);
             buf = 0;
         }

         
         usleep(100000);
     }

     if ( retry < 10 ) {
         for (i = 0; i < len; i++) {
             if (buf[i] == '=') {
                 tmp = strtoul(&buf[i+1], NULL, 16);
                 if (tmp > UCHAR_MAX)
                     break;
                 free(buf);
                 return tmp;
             }
         }
     }

 bail:
     if (buf)
         free(buf);
     return 1;
}


