/*
    Copyright (C) SEIKO EPSON CORPORATION 2002-2004. All Rights Reserved.

    3-3-5, Owa, Suwa-shi Nagano-ken, 392-8502 Japan
    Phone: +81-266-52-3131

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public License
    as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/







#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <signal.h>
#include <errno.h>

#ifdef WIN32
#include <winsock2.h>
#include <sys/Timeb.h>
#else
#include <unistd.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <sys/time.h>
#include <termios.h>
#include <netinet/in.h>
#include <err.h>
#include <netdb.h>
#include <arpa/inet.h>
#endif


#include "lki_private.h"
#include "libkeyinput.h"
#include "lki_extern.h"
#include "lki_define.h"

#define CMD_APPINFO		"_APPINFO"


int LKI_SetAcceptableEvent(const int iEvent)
{
	int iRv;
	char sCmd[ESCVP21_MAXLEN+1];
	char* pReceive;

	DEBUGF("----- LKI_SetAcceptableEvent start -----\n");

	
	pReceive = NULL;

	
	sprintf(sCmd, "%s %s %x" ESCVP21_DELIMITER_S, CMD_APPINFO, DEFALUT_APPNAME, iEvent);

	
	iRv = LKI_ComSequenceINT(giEvFD, sCmd, &pReceive);
	if(iRv < 1){
		DEBUGF("sequence error\n");
		DEBUGF("----- LKI_SetAcceptableEvent end -----\n");
		return iRv;
	}

	
	DEBUGF("receive length=%d\n", iRv);
	DEBUGF("pReceive=%x\n", pReceive);

	pReceive[iRv] = '\0';
	if(strcmp(ESCVP21_STATUS_SUCCESS_S, pReceive) != 0){
		DEBUGF("status error\n");
		DEBUGF("----- LKI_SetAcceptableEvent end -----\n");

		return LKI_ERR_CMD;
	}

	DEBUGF("----- LKI_SetAcceptableEvent end -----\n");

	return LKI_SUCCESS;

}


int LKI_GetAcceptableEvent(unsigned int* pEvent)
{
	int iRv, ii;
	unsigned int uiTmp;
	char sTmp[ESCVP21_MAXLEN+1];
	char *pRet, *pTmp;

	
	sprintf(sTmp, "%s? DUMMY" ESCVP21_DELIMITER_S, CMD_APPINFO);

	
	iRv = LKI_ComSequenceINT(giEvFD, sTmp, &pRet);
	if(iRv < 1){
		return iRv;
	}
	pTmp = pRet;

	
	pRet[iRv] = '\0';
	if(ESCVP21_STATUS_SUCCESS != pRet[iRv-1]){
		return LKI_ERR_CMD;
	}

	
	
	pTmp += iRv-1;
	*pEvent = 0;
	for(ii=0; ii<8; ii++){

		uiTmp = *pTmp - '0';
		*pEvent |= uiTmp << (4*ii);
		pTmp--;

		
		if(*pTmp == ' '){
			break;
		}
	}

	return LKI_SUCCESS;
}


