/*
    Copyright (C) SEIKO EPSON CORPORATION 2002-2004. All Rights Reserved.

    3-3-5, Owa, Suwa-shi Nagano-ken, 392-8502 Japan
    Phone: +81-266-52-3131

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public License
    as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/







#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <signal.h>
#include <errno.h>

#ifdef WIN32
#include <winsock2.h>
#include <sys/Timeb.h>
#else
#include <pthread.h>
#include <unistd.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <sys/time.h>
#include <termios.h>
#include <netinet/in.h>
#include <err.h>
#include <netdb.h>
#include <arpa/inet.h>
#endif


#include "lki_private.h"
#include "libkeyinput.h"
#include "lki_extern.h"
#include "lki_define.h"


static pthread_mutex_t g_lki_lock = PTHREAD_MUTEX_INITIALIZER;


static int LKI_SendSocket(const int iFd, const char* pBuff, const size_t iLen);
static int LKI_ReceiveExtCmd21(int iFd, char** ppReceive, const int iSec, const int iMSec);
static int LKI_IsBinCmd(const char* pCmd);



int LKI_ComSequenceINT(int iFd, const char* pSend, char** ppReceive)
{
	int iRv, ii, iFlg;
	unsigned char* pBin;

	unsigned char* pKisyu = NULL;

	int	ptbl_flg = 0;
	int 	fd;
	struct	stat	stat_buf;

#ifdef ___DEBUG___
	char sTmp[ESCVP21_MAXLEN+1];
	fprintf(stderr, "----- LKI_ComSequence -----\n");
#endif

	
	if(pSend == NULL){
		iRv = LKI_ERR_FATAL;
		goto ERR_RET_SEQ1;
	}
	if(ppReceive == NULL){
		iRv = LKI_ERR_FATAL;
		goto ERR_RET_SEQ1;
	}

	
	iRv = LKI_StrCmdLen(FALSE, pSend);
	if(iRv < 0){

#ifdef ___DEBUG___
		fprintf(stderr, "command length ?\n");
#endif
		goto ERR_RET_SEQ1;

	}else if(iRv > ESCVP21_MAXLEN){

		iRv = LKI_ERR_CMD_OVERLENGTH;
		goto ERR_RET_SEQ1;

	}
#ifdef ___DEBUG___
	fprintf(stderr, "command length = %d\n", iRv);
	memcpy(sTmp, pSend, iRv);
	sTmp[iRv] = '\0';
	fprintf(stderr, "%s\n", sTmp);
	for(ii=0; ii<iRv; ii++){
		fprintf(stderr, "data[%d]=0x%x(%c)\n", ii, pSend[ii], pSend[ii]);
	}
#endif

	if (memcmp(pSend, "PTBL? ", 6) == 0) {
		for (ii = 6; pSend[ii] != 0x00; ii++) {
			if (pSend[ii] != 0x20) {
				if (strcmp(&pSend[ii], "ALL\r") == 0) {
					ptbl_flg = 1;
				} else {
					ptbl_flg = 0;
				}
				break;
			}
		} 

		if (ptbl_flg) {
			
			if ((fd = open(KISYU_TBL_FILE, O_RDONLY)) < 0) {
				iRv = LKI_ERR_FATAL;
				goto ERR_RET_SEQ1;
			}

			fstat(fd, &stat_buf);
			if (!stat_buf.st_size) {
				close(fd);
				iRv = LKI_ERR_FATAL;
				goto ERR_RET_SEQ1;
			}

			pKisyu = ((unsigned char*)malloc(stat_buf.st_size));
			if (pKisyu == NULL) {
				close(fd);
				iRv = LKI_ERR_FATAL;
				goto ERR_RET_SEQ1;
			}

			read(fd, pKisyu, stat_buf.st_size);
			*ppReceive = pKisyu;
			close(fd);

			return stat_buf.st_size;
		} 
	} 

	
	iRv = LKI_SendSocket(iFd, pSend, iRv);
	if(iRv < 0){
#ifdef ___DEBUG___
		fprintf(stderr, "send...error\n");
#endif
		goto ERR_RET_SEQ1;

	}
#ifdef ___DEBUG___
	fprintf(stderr, "send...ok\n");
#endif

	
	iRv = LKI_ReceiveExtCmd21(iFd, ppReceive, ESCVP21_RECEIVE_TIMEOUT_SEC, 0);
	if(iRv < 0){
		if(iRv == LKI_TIMEOUT){
			giRecErrCntPW++;
		}
		goto ERR_RET_SEQ1;
	}

#ifdef ___DEBUG___
	fprintf(stderr, "receive length=%d\n", iRv);
	fprintf(stderr, "*ppReceive=%x\n", *ppReceive);
#endif

	
	if(*ppReceive[0] == ESCVP21_STATUS_SUCCESS){
		goto SUCCESS_RET_SEQ1;
	}

	
	if(strncmp(*ppReceive, ESCVP21_STATUS_ERR, 3) == 0){
		goto SUCCESS_RET_SEQ1;
	}

	
	iFlg = LKI_IsBinCmd(*ppReceive);
	if(iFlg < 0){
		iRv = iFlg;
		goto ERR_RET_SEQ1;

	}else if(iFlg == TRUE){

		for(ii=0; ii<iRv; ii++){
#ifdef ___DEBUG___
			fprintf(stderr, "[%d]=%x(%c)\n", ii, (*ppReceive)[ii], (*ppReceive)[ii]);
#endif
			if((*ppReceive)[ii] == '='){
				iRv = LKI_Ascii2Bin(&(*ppReceive)[ii+1], &pBin);
				if(iRv < 0){
					goto ERR_RET_SEQ1;
				}
				free(*ppReceive);
				*ppReceive = (char*)pBin;
				goto SUCCESS_RET_SEQ1;
			}
		}

	}else{

		goto SUCCESS_RET_SEQ1;

	}

SUCCESS_RET_SEQ1:
#ifdef ___DEBUG___
	fprintf(stderr, "return value=%d\n", iRv);
	fprintf(stderr, "command sequence...ok\n");
	fprintf(stderr, "----- LKI_ComSequence end ------\n");
#endif
	return iRv;


ERR_RET_SEQ1:
#ifdef ___DEBUG___
	fprintf(stderr, "return value=%d\n", iRv);
	fprintf(stderr, "----- LKI_ComSequence err end ------\n");
#endif
	if(*ppReceive != NULL){
		free(*ppReceive);
	}
	*ppReceive = NULL;
	return iRv;

}






int LKI_ComSequence(const char* pSend, char** ppReceive)
{
	int iRv;
	pthread_mutex_lock(&g_lki_lock);
	iRv = LKI_ComSequenceINT(giCmdFD, pSend, ppReceive);
	pthread_mutex_unlock(&g_lki_lock);
	return iRv;
}




int LKI_ComSequence2(const COMID eID, const char* pParam, char** ppRet)
{
	char sSend[ESCVP21_MAXLEN+1];
	unsigned char* pBin;
	char* pTmp;
	int iLen, iRv = -1, ii;

	
	pBin = NULL;

	
	if(ppRet == NULL){
		goto ERR_RET_SEQ2;
	}

	
	iRv = LKI_Encode21(sSend, eID, pParam);
	if(iRv < 0){
		goto ERR_RET_SEQ2;
	}

	
	iRv = LKI_ComSequence(sSend, ppRet);
	if(iRv < 0){
		goto ERR_RET_SEQ2;
	}

	
	if (strcmp(sSend, "PTBL? ALL\r") == 0) {
		return iRv;
	}

	
	if(!(gstCmdTbl[(int)eID].iClass & CMD_CLASS_BIN)){

		
		if(*ppRet[0] == ESCVP21_STATUS_SUCCESS){

#ifdef ___DEBUG___
			fprintf(stderr, "status(:) only\n");
#endif
			pTmp = (char*)calloc(2, 1);
			if(pTmp == NULL){
				goto ERR_RET_SEQ2;
			}
			pTmp[0] = (*ppRet)[0];
			free(*ppRet);
			*ppRet = pTmp;
			iRv = 1;

		}
		
		else if(strncmp(*ppRet, ESCVP21_STATUS_ERR, 3) == 0){

#ifdef ___DEBUG___
			fprintf(stderr, "status(ERR)\n");
#endif
			(*ppRet)[iRv - 2] = '\0';
			iRv -= 2;

		}
		
		else{

#ifdef ___DEBUG___
			fprintf(stderr, "return value\n");
#endif
			
			(*ppRet)[iRv - 2] = '\0';

			
			for(ii=0; ii<iRv; ii++){
				if((*ppRet)[ii] == '='){
					break;
				}
			}
#ifdef ___DEBUG___
			fprintf(stderr, "*ppRet[%d] = %02x\n");
#endif
			iLen = strlen((*ppRet)+ii+1);
			pTmp = (char*)calloc(iLen+1, 1);
			if(pTmp == NULL){
				goto ERR_RET_SEQ2;
			}
			memcpy(pTmp, (*ppRet)+ii+1, iLen);
			free(*ppRet);
			*ppRet = pTmp;
			iRv = iLen;

		}

	}

	return iRv;


ERR_RET_SEQ2:
	if(*ppRet != NULL){
		free(*ppRet);
	}
	*ppRet = NULL;
	return iRv;

}


int LKI_ReceiveEventWait(char** ppReceive, const int iSec, const int iMSec)
{
	int iRv;
	fd_set stFdSet;
	struct timeval stTimeVal;
	char* pRBuff;

#ifdef ___DEBUG___
	fprintf(stderr, "----- LKI_ReceiveEventWait start -----\n");
#endif

	
	pRBuff = NULL;

	
	if(giEvFD < 0){
		iRv = LKI_ERR_FATAL;
		goto ERR_RET_EVENTWAIT;
	}
	if(ppReceive == NULL){
		iRv = LKI_ERR_FATAL;
		goto ERR_RET_EVENTWAIT;
	}

	
	FD_ZERO(&stFdSet);
	FD_SET(giEvFD, &stFdSet);
	stTimeVal.tv_sec  = iSec;
	stTimeVal.tv_usec = iMSec;
	if((iSec != -1) && (iMSec != -1)){
		iRv = select(giEvFD+1, &stFdSet, NULL, NULL, &stTimeVal);
	}else{
		iRv = select(giEvFD+1, &stFdSet, NULL, NULL, NULL);
	}
	if(iRv < 0){
#ifdef ___DEBUG___
		fprintf(stderr, "event select...error\n");
#endif
		iRv = LKI_ERR_FATAL;
		goto ERR_RET_EVENTWAIT;

	}else if(iRv == 0){
#ifdef ___DEBUG___
		fprintf(stderr, "event select...timeout\n");
#endif
		iRv = LKI_TIMEOUT;
		goto ERR_RET_EVENTWAIT;

	}
#ifdef ___DEBUG___
	fprintf(stderr, "event select...wake up\n");
	fprintf(stderr, "giEvFD=%d\n", giEvFD);
	fprintf(stderr, "wake up fd=%d\n", iRv);
#endif

	
	pRBuff = calloc(ESCVP21_MAXLEN, 1);
	if(pRBuff == NULL){
		iRv = LKI_ERR_FATAL;
		goto ERR_RET_EVENTWAIT;
	}
#ifdef ___DEBUG___
	fprintf(stderr, "memory allocation...ok\n");
#endif

	
	


	iRv = recv(giEvFD, pRBuff, ESCVP21_MAXLEN, 0);

	if(iRv < 0){
#ifdef ___DEBUG___
		fprintf(stderr, "read...error\n");
#endif
		iRv = LKI_ERR_FATAL;
		goto ERR_RET_EVENTWAIT;
	}else if(iRv == 0){
#ifdef ___DEBUG___
		fprintf(stderr, "read...disconnect\n");
#endif
		LKI_CloseSocket();
		iRv = LKI_DISCONNECT;
		goto ERR_RET_EVENTWAIT;
	}

#ifdef ___DEBUG___
	fprintf(stderr, "read...ok\n");
	fprintf(stderr, "----- LKI_ReceiveEventWait end -----\n");
#endif
	*ppReceive = pRBuff;
	return iRv;

ERR_RET_EVENTWAIT:
#ifdef ___DEBUG___
	fprintf(stderr, "----- LKI_ReceiveEventWait err end -----\n");
#endif
	if(pRBuff != NULL){
		free(pRBuff);
	}
	ppReceive = NULL;
	return iRv;
}






static int LKI_ReceiveExtCmd21(int iFd, char** ppReceive, const int iSec, const int iMSec)
{
	int iRv;
	int iCnt;
	int iRetry;
	int iDelimiter;
	fd_set stFdSet;
	char sReceiveBuff[ESCVP21_MAXLEN+1];
	char *pRBuff, *pNewBuff;
#ifdef ___DEBUG___
	char sTmp[ESCVP21_MAXLEN+2];
#endif

	FD_ZERO(&stFdSet);
	FD_SET(iFd, &stFdSet);
	iCnt = 0;
	iRetry = 0;
	pRBuff = pNewBuff = NULL;
	iDelimiter = FALSE;


#ifdef ___DEBUG___
	fprintf(stderr, "----- LKI_ReceiveExtCmd21 start ------\n");
#endif

	
	if(iFd < 0){
#ifdef ___DEBUG___
		fprintf(stderr, "fd error\n");
#endif
		iRv = LKI_ERR_FATAL;
		goto ERR_RET_RECCMD;
	}
	if(ppReceive == NULL){
#ifdef ___DEBUG___
		fprintf(stderr, "ppReceive NULL\n");
#endif
		iRv = LKI_ERR_FATAL;
		goto ERR_RET_RECCMD;
	}
	
	while(iRetry < LKI_RECEIVE_COMMAND_RETRY_CNT){

		int ii;
#ifdef ___DEBUG___
		int jj;
#endif

		iRv = select(iFd+1, &stFdSet, NULL, NULL, NULL);
		if(iRv == -1){
#ifdef ___DEBUG___
			fprintf(stderr, "select...error\n");
#endif
			if (errno == EINTR) {
				continue;
			}
			iRv = LKI_ERR_FATAL;
			goto ERR_RET_RECCMD;

		}else if(iRv == 0){
#ifdef ___DEBUG___
			fprintf(stderr, "select...timeout\n");
#endif
			if (errno == EINTR) {
				continue;
			}
			iRetry++;
			iRv = LKI_TIMEOUT;
			continue;
		}
#ifdef ___DEBUG___
		fprintf(stderr, "select...ok\n");
#endif

		


		for (;;) {
			iRv = recv(iFd, sReceiveBuff, ESCVP21_MAXLEN, 0);

			if(iRv < 0){
#ifdef ___DEBUG___
				fprintf(stderr, "read...error\n");
#endif
				if (errno == EINTR) {
					continue;
				}
				iRv = LKI_ERR_FATAL;
				goto ERR_RET_RECCMD;

			}else if(iRv == 0){
#ifdef ___DEBUG___
				fprintf(stderr, "read...error...KID disconnect\n");
#endif
				iRv = LKI_DISCONNECT;
				goto ERR_RET_RECCMD;
			} else {
				break;
			}
		}
#ifdef ___DEBUG___
		fprintf(stderr, "read...ok\n");
#endif

		
		iRetry = 0;
		pRBuff = (char*)realloc(pRBuff, iCnt + iRv);
		if(pRBuff == NULL){
			iRv = LKI_ERR_FATAL;
			goto ERR_RET_RECCMD;
		}
		memcpy(pRBuff + iCnt, sReceiveBuff, iRv);
		iCnt += iRv;

#ifdef ___DEBUG___
		for(jj=0; jj<iCnt; jj++){
			fprintf(stderr, "[%d]=%x(%c)\n", jj, pRBuff[jj],pRBuff[jj]);
		}
		fprintf(stderr, "\n");
		fprintf(stderr, "read data length = %d\n", iCnt);
#endif

		
		if(sReceiveBuff[0] == ESCVP21_STATUS_SUCCESS){
#ifdef ___DEBUG___
			fprintf(stderr, "status ':' receive...ok\n");
			fprintf(stderr, "----- LKI_ReceiveExtCmd21 end ------\n");
#endif
			*ppReceive = pRBuff;
			return iCnt;
		}

		
		for(ii=iCnt-1; ii>=0; ii--){

			if(pRBuff[ii] == ESCVP21_DELIMITER){
#ifdef ___DEBUG___
				fprintf(stderr, "delimiter...found\n");
#endif
				iDelimiter = TRUE;
				break;
			}
		}
		if(iDelimiter != TRUE){
#ifdef ___DEBUG___
			fprintf(stderr, "delimiter...not found\n");
#endif
			continue;
		}

		
		for(ii=ii+1; ii<iCnt; ii++){

			if(pRBuff[ii] == ESCVP21_STATUS_SUCCESS){

#ifdef ___DEBUG___
				fprintf(stderr, "status(:)...ok\n");
				fprintf(stderr, "----- LKI_ReceiveExtCmd21 end ------\n");
#endif
				*ppReceive = pRBuff;
				return iCnt;
			}
		}
#ifdef ___DEBUG___
		fprintf(stderr, "status(:)...not found\n");
#endif

	}


ERR_RET_RECCMD:
#ifdef ___DEBUG___
	fprintf(stderr, "----- LKI_ReceiveExtCmd21 err end ------\n");
#endif
	if(pRBuff != NULL){
		free(pRBuff);
	}
	*ppReceive = NULL;
	return iRv;

}


static int LKI_SendSocket(const int iFd, const char* pBuff, const size_t iLen)
{
	int iRv;

	if(iFd < 0){
		return LKI_ERR_FATAL;
	}
	if(pBuff == NULL){
		return LKI_ERR_FATAL;
	}

	


	for (;;)  {
		iRv = send(iFd, pBuff, iLen, 0);

		if (iRv < 0) {
			if (errno == EINTR) {
				continue;
			}
			return LKI_ERR_FATAL;
		} 
		break;
	}

	




	return iRv;
}


static int LKI_IsBinCmd(const char* pCmd)
{
	int ii, iRv;
	char sTmp[ESCVP21_MAXLEN+1];

#ifdef ___DEBUG___
	fprintf(stderr, "----- LKI_IsBinCmd start ------\n");
#endif

	
	memset(sTmp, 0, ESCVP21_MAXLEN+1);

	
	if(pCmd == NULL){
		iRv = LKI_ERR_FATAL;
		goto ERR_RET_ISBIN;
	}

	
	for(ii=0; ii<ESCVP21_MAXLEN; ii++){

		if( (pCmd[ii] == '=') || (pCmd[ii] == ' ') ){
			memcpy(sTmp, pCmd, ii);
			strcat(sTmp, "?");
			break;
		}

	}
	if(sTmp[0] == '\0'){
		iRv = LKI_ERR_CMD;
		goto ERR_RET_ISBIN;
	}
#ifdef ___DEBUG___
	fprintf(stderr, "command=%s\n", sTmp);
#endif

	
	for(ii=0;;ii++){

		if(strcmp(gstCmdTbl[ii].sCmd, CMD_NULL) == 0){
#ifdef ___DEBUG___
			fprintf(stderr, "unmatch command\n");
#endif
			iRv = LKI_ERR_CMD;
			goto ERR_RET_ISBIN;
		}

		if(strcmp(gstCmdTbl[ii].sCmd, sTmp) == 0){
			break;
		}

	}

	
#ifdef ___DEBUG___
	fprintf(stderr, "----- LKI_IsBinCmd end ------\n");
#endif
	if(gstCmdTbl[ii].iClass & CMD_CLASS_BIN){
#ifdef ___DEBUG___
		fprintf(stderr, "return value : binary\n");
#endif
		return TRUE;
	}else{
#ifdef ___DEBUG___
		fprintf(stderr, "return value : text\n");
#endif
		return FALSE;
	}


ERR_RET_ISBIN:
#ifdef ___DEBUG___
	fprintf(stderr, "----- LKI_IsBinCmd err end ------\n");
#endif
	return iRv;
}

