/*
    Copyright (C) SEIKO EPSON CORPORATION 2002-2004. All Rights Reserved.

    3-3-5, Owa, Suwa-shi Nagano-ken, 392-8502 Japan
    Phone: +81-266-52-3131

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public License
    as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/







#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <signal.h>
#include <errno.h>


#ifdef WIN32
#include <winsock2.h>
#include <sys/Timeb.h>
#else
#include <unistd.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <sys/time.h>
#include <termios.h>
#include <netinet/in.h>
#include <err.h>
#include <netdb.h>
#include <arpa/inet.h>
#endif


#include "lki_private.h"
#include "libkeyinput.h"
#include "lki_define.h"


int giCmdFD = -1;				
int giEvFD = -1;				
int giRecErrCntPW;				

static void LKI_InitLib(void) __attribute__((constructor));



static void LKI_InitLib(void)
{
	giCmdFD = -1;
	giEvFD  = -1;
	giRecErrCntPW = 0;
}


int LKI_OpenCmdSocket(const char *pAddr)
{
	int iRv;
	struct sockaddr_in stDestaddr;
	struct	protoent *pTp = getprotobyname("tcp");
	unsigned long int ulAddr  = 0;

	DEBUGF("----- LKI_OpenCmdSocket start -----\n");
	DEBUGF("CmdFD = %d\n", giCmdFD);

	if (giCmdFD != -1) {
		return giCmdFD;
	}

	giRecErrCntPW = 0;

	if (pAddr == NULL) {
		
#ifdef WIN32
		ulAddr = (unsigned int)inet_addr(LOCALHOST_ADDR);
#else
		ulAddr = (unsigned int)inet_network(LOCALHOST_ADDR);
#endif
	} else {
#ifdef WIN32
		ulAddr = (unsigned int)inet_addr(pAddr);
#else
		ulAddr = (unsigned int)inet_network(pAddr);
#endif
	} 

	
	giCmdFD = socket(PF_INET, SOCK_STREAM, pTp->p_proto);
	if (giCmdFD < 0) {
		DEBUGF("LKI:socket open...error\n");
		LKI_CloseCmdSocket();
		return LKI_ERR_FATAL;
	}
	DEBUGF("LKI:socket open...ok  fd [%d]\n", giCmdFD);
 
	
	memset(&stDestaddr, 0x00, sizeof(stDestaddr));
	stDestaddr.sin_family = AF_INET;
	stDestaddr.sin_port   = htons(IM_TCP_PORT_NO);
#ifdef WIN32
	stDestaddr.sin_addr.s_addr = ulAddr;
#else
	stDestaddr.sin_addr.s_addr = htonl(ulAddr);
#endif

	iRv = connect(giCmdFD, (struct sockaddr*)&stDestaddr, sizeof(stDestaddr));
	if (iRv < 0) {
		DEBUGF("LKI:command socket connect...error\n");
		LKI_CloseCmdSocket();
		return LKI_ERR_FATAL;
	}

	DEBUGF("LKI:command socket connect...ok fd:[%d]\n", giCmdFD);
	DEBUGF("----- LKI_OpenCmdSocket end -----\n");

	return giCmdFD;
}


int LKI_OpenEventSocket(const char *pAddr)
{
	int iRv;
	struct	sockaddr_in stDestaddr;
	struct	protoent *pTp = getprotobyname("tcp");
	unsigned long int ulAddr  = 0;

	DEBUGF("----- LKI_OpenEventSocket start -----\n");
	DEBUGF("EvFD = %d\n", giEvFD);

	if (giEvFD != -1) {
		return giEvFD;
	}

	if (pAddr == NULL) {
		
#ifdef WIN32
		ulAddr = (unsigned int)inet_addr(LOCALHOST_ADDR);
#else
		ulAddr = (unsigned int)inet_network(LOCALHOST_ADDR);
#endif
	} else {
#ifdef WIN32
		ulAddr = (unsigned int)inet_addr(pAddr);
#else
		ulAddr = (unsigned int)inet_network(pAddr);
#endif
	} 


	
	giEvFD = socket(PF_INET, SOCK_STREAM, pTp->p_proto);
	if (giEvFD < 0) {
		DEBUGF("LKI:event socket open...error\n");
		LKI_CloseEventSocket();
		return LKI_ERR_FATAL;
	}
	DEBUGF("LKI:event socket open...ok fd:[%d]\n", giEvFD);

	
	memset(&stDestaddr, 0x00, sizeof(stDestaddr));
	stDestaddr.sin_family = AF_INET;
	stDestaddr.sin_port = htons(EVENT_PORT);
	stDestaddr.sin_addr.s_addr = htonl(ulAddr);

	iRv = connect(giEvFD, (struct sockaddr*)&stDestaddr, sizeof(stDestaddr));
	if (iRv < 0) {
		DEBUGF("LKI:event socket connect...error\n");

		LKI_CloseEventSocket();
		return LKI_ERR_FATAL;
	} 

	DEBUGF("LKI:event socket connect...ok\n");
	DEBUGF("LKI:----- LKI_OpenEventSocket end -----\n");

	return giEvFD;
}


int LKI_OpenSocket(const char* pAddr)
{
	int iRv;

	DEBUGF("----- LKI_OpenSocket start -----\n");

	
	if((giCmdFD != -1) && (giEvFD != -1)){
		return LKI_SUCCESS;
	}

	
	iRv = LKI_OpenCmdSocket(pAddr);
	if (iRv < 0) {
		DEBUGF("----- LKI_OpenCmdSocket err -----\n");

		return LKI_ERR_FATAL;
	}

	
	iRv = LKI_OpenEventSocket(pAddr);
	if (iRv < 0) {
		DEBUGF("----- LKI_OpenEventSocket err -----\n");

		return LKI_ERR_FATAL;
	}

	DEBUGF("----- LKI_OpenSocket end -----\n");

	return LKI_SUCCESS;

}


int LKI_CloseCmdSocket(void)
{
	int iRv;

	if (giCmdFD >= 0) {
#ifdef WIN32
		iRv = closesocket(giCmdFD);
#else
		iRv = close(giCmdFD);
#endif
		giCmdFD = -1;

		if (iRv != 0) {
			return LKI_ERR_FATAL;
		}
	}

	return LKI_SUCCESS;
}


int LKI_CloseEventSocket(void)
{
	int iRv;

	if (giEvFD >= 0) {

#ifdef WIN32
		iRv = closesocket(giEvFD);
#else
		iRv = close(giEvFD);
#endif
		giEvFD = -1;

		if (iRv != 0) {
			return LKI_ERR_FATAL;
		}
	}

	return LKI_SUCCESS;
}


int LKI_CloseSocket(void)
{
	int iRv;

	
	iRv = LKI_CloseCmdSocket();

	
	iRv = LKI_CloseEventSocket();

	return LKI_SUCCESS;
}

