/*
    Copyright (C) SEIKO EPSON CORPORATION 2002-2004. All Rights Reserved.

    3-3-5, Owa, Suwa-shi Nagano-ken, 392-8502 Japan
    Phone: +81-266-52-3131

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public License
    as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/







#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <signal.h>
#include <errno.h>
#include <assert.h>
#include <ctype.h>

#ifdef WIN32
#include <winsock2.h>
#include <sys/Timeb.h>
#else
#include <unistd.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <sys/time.h>
#include <termios.h>
#include <netinet/in.h>
#include <err.h>
#include <netdb.h>
#include <arpa/inet.h>
#endif


#include "lki_private.h"
#include "libkeyinput.h"
#include "lki_extern.h"
#include "lki_define.h"

#define LKI_UTY_LINE_DELIMITER		ESCVP21_DELIMITER	


int LKI_StrCmdLen(int iFlg, const char* pBuff)
{
	int ii;
	char cCmp;

	DEBUGF("----- LKI_StrCmdLen start -----\n");

	if(pBuff == NULL){
		return LKI_ERR_FATAL;
	}

	if(iFlg){
		cCmp = ESCVP21_STATUS_SUCCESS;
	}else{
		cCmp = ESCVP21_DELIMITER;
	}

	for(ii=0; ii<ESCVP21_MAXLEN; ii++){
		DEBUGF("[%d]=%02x%c\n", ii, pBuff[ii], pBuff[ii]);

		if(pBuff[ii] == cCmp){
			DEBUGF("----- LKI_StrCmdLen end -----\n");

			return ii+1;
		}

	}

	DEBUGF("----- LKI_StrCmdLen err end -----\n");

	return LKI_ERR_CMD_NON_DELIMITER;
}


int LKI_Ascii2Bin(const char* pHex, unsigned char** ppBin)
{
	int iBinLen, iCharLen, ii, jj, iRv;
	unsigned char byTmp;
	unsigned char *pBinData, *pTmp;

	DEBUGF("----- LKI_Ascii2Bin start -----\n");

	pBinData = NULL;

	
	for(ii=0,jj=0; ; ii++){

		if(pHex[ii] == ESCVP21_STATUS_SUCCESS){
			break;
		}
		if(pHex[ii] == ESCVP21_DELIMITER){
			continue;
		}else{
			DEBUGF("[%d]=%x(%c)\n", ii, pHex[ii], pHex[ii]);

			jj++;
		}
	}

	
	DEBUGF("Ascii num=%d\n", jj);

	if(jj & 0x00000001){
		DEBUGF("KISUU\n");
		iRv = LKI_ERR_CMD;
		goto ERRRET_ASCII2BIN;
	}

	
	iCharLen = ii;
	iBinLen  = jj / 2;
	pBinData = pTmp = (unsigned char*)calloc(iBinLen, 1);
	if(pBinData == NULL){
		DEBUGF("memory alocation error\n");
		iRv = LKI_ERR_FATAL;
		goto ERRRET_ASCII2BIN;
	}

	
	for(ii=0;ii<iCharLen;){

		for(byTmp=0,jj=1; jj>=0; ){

			if(ii>=iCharLen){
				goto SUCCESS_ASCII2BIN;
			}

			if(pHex[ii] == ESCVP21_STATUS_SUCCESS){

				
				if(jj == 0 && (pHex[ii-1] != LKI_UTY_LINE_DELIMITER)){
					DEBUGF("illigal status\n");
					iRv = LKI_ERR_CMD;
					goto ERRRET_ASCII2BIN;
				}
				break;

			}

			if(pHex[ii] != LKI_UTY_LINE_DELIMITER){	

				if(isxdigit(pHex[ii])){
					if(isdigit(pHex[ii])){
						byTmp |= (pHex[ii] - '0') << 4*jj;
					}else{
						byTmp |= (((char)toupper(pHex[ii])) - 'A' + 10) << 4*jj;
					}
					jj--;
				}else{
					DEBUGF("illigal character\n");

					iRv = LKI_ERR_CMD;
					goto ERRRET_ASCII2BIN;
				}

			}
			ii++;

		}

		*pTmp = byTmp;
		pTmp++;

	}

SUCCESS_ASCII2BIN:
	DEBUGF("----- LKI_Ascii2Bin end -----\n");
	*ppBin = pBinData;
	return iBinLen;


ERRRET_ASCII2BIN:
	DEBUGF("----- LKI_Ascii2Bin err end -----\n");
	if(pBinData != NULL){
		free(pBinData);
	}
	*ppBin = NULL;
	return iRv;
}


int LKI_Encode21(char* pBuff, COMID eID, const char* pParam)
{

	
	if(eID >= _END_){

		pBuff[0] = '\0';
		return LKI_ERR_CMD_NOT;

	}

	
	if(pBuff == NULL){
		return LKI_ERR_FATAL;
	}
	memset(pBuff, 0, ESCVP21_MAXLEN); 

	
	if(pParam != NULL && *pParam != '\0'){

		sprintf(pBuff, "%s %s" ESCVP21_DELIMITER_S, gstCmdTbl[(int)eID].sCmd, pParam);

	}else{

		sprintf(pBuff, "%s" ESCVP21_DELIMITER_S, gstCmdTbl[(int)eID].sCmd);

	}

	
	return(LKI_StrCmdLen(FALSE, pBuff));

}


CMDTBL* LKI_GetCmdInfo(const char* pCmd)
{
	int ii;

	if(pCmd == NULL){
		goto ERR_RET_GETCMDINFO;
	}

	for(ii=0;;ii++){

		if(strcmp(gstCmdTbl[ii].sCmd, CMD_NULL) == 0){
			goto ERR_RET_GETCMDINFO;
		}

		if(strcmp(gstCmdTbl[ii].sCmd, pCmd) == 0){
			return &(gstCmdTbl[ii]);
		}

	}

ERR_RET_GETCMDINFO:
	return NULL;

}
int
LKI_Send(COMID id, const char *data)
{
	 int result;
	 char *tmp = NULL;
	assert(data);

	result = LKI_ComSequence2(id, data, &tmp);
	if (result < 1 || tmp == NULL)
		goto bail;

	if (strcmp(tmp, ESCVP21_STATUS_ERR) == 0)
		goto bail;

	free(tmp);

	return LKI_SUCCESS;
bail:
	if (tmp)
		free(tmp);
	return LKI_ERR_FATAL;
}

size_t
LKI_Receive(COMID id, const char *parm, char *ptr, size_t size)
{
	 int result;
	char *tmp = NULL;
	assert(ptr);

	result = LKI_ComSequence2(id, parm, &tmp);
	if (result < 1 || tmp == NULL)	
		goto bail;

	if (strcmp(tmp, ESCVP21_STATUS_ERR) == 0)
		goto bail;

	if (strlen(tmp) >= size)
		goto bail;

	strcpy(ptr, tmp);

	free(tmp);

	return result;
bail:
	if (tmp)
		free(tmp);
	return 0;
}


int LKI_GetCmdID(const char* pCmd)
{
	int ii, jj;
	char sTmp[ESCVP21_MAXLEN+1];
#ifndef NDEBUG
	char sBuff[ESCVP21_MAXLEN+1];
#endif

	if(pCmd == NULL){
		DEBUGF("pCmd == NULL!\n");

		return LKI_ERR_FATAL;
	}

	memset(sTmp, 0, sizeof(sTmp));

	
	for(ii=0,jj=0; ii<ESCVP21_MAXLEN; ii++){
		if(pCmd[ii] == ' ' || 
		   pCmd[ii] == '=' ||
		   pCmd[ii] == ESCVP21_DELIMITER ||
		   pCmd[ii] == ESCVP21_STATUS_SUCCESS ||
		   pCmd[ii] == '\0'){

			break;

		}
		sTmp[jj] = pCmd[ii];
		jj++;

	}

#ifndef NDEBUG
	sprintf(sBuff, "command = %s\n", sTmp);
#endif

	
	for(ii=0;;ii++){

		if(strcmp(gstCmdTbl[ii].sCmd, CMD_NULL) == 0){
			return LKI_ERR_CMD;
		}

		if(strcmp(gstCmdTbl[ii].sCmd, sTmp) == 0){
			return ii;
		}

	}
}

