dnl Some useful autoconf macros
dnl
dnl This file is distributed in the hope that it will be useful,
dnl but WITHOUT ANY WARRANTY; without even the implied warranty of
dnl MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
dnl GNU Lesser General Public License for more details.
dnl
dnl You should have received a copy of the GNU Lesser General Public License
dnl along with this program; if not, write to the Free Software
dnl Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
dnl
dnl Copyright (C) 2002 Antoine Mine' and Nicholas Humfrey



dnl *****************************
dnl Check and set data type sizes
dnl *****************************

dnl MY_CHECK_TYPES
dnl --------------

AC_DEFUN(MY_CHECK_TYPES,[
	AC_CHECK_SIZEOF(char, 1)
	AC_CHECK_SIZEOF(short, 2)
	AC_CHECK_SIZEOF(int, 4)
	AC_CHECK_SIZEOF(long, 4)
	AC_CHECK_SIZEOF(long long, 8)
	
	
	if test "${ac_cv_sizeof_char}" = "1" ; then
		AC_CHECK_TYPE(__u8, unsigned char)
		AC_CHECK_TYPE(__s8, signed char)
	else
		# can't determine default value in case the type isn't present
		AC_MSG_ERROR(CHECK_TYPE__8 - please report to openquicktime-devel@lists.sourceforge.net)
	fi
	
	if test "${ac_cv_sizeof_short}" = "2" ; then
		AC_CHECK_TYPE(__u16, unsigned short)
		AC_CHECK_TYPE(__s16, signed short)
	else
		# can't determine default value in case the type isn't present
		AC_MSG_ERROR(CHECK_TYPE__16 - please report to openquicktime-devel@lists.sourceforge.net)
	fi
	
	if test "${ac_cv_sizeof_int}" = "4" ; then
		AC_CHECK_TYPE(__u32, unsigned int)
		AC_CHECK_TYPE(__s32, signed int)
	elif test "${ac_cv_sizeof_long}" = "4" ; then
		AC_CHECK_TYPE(__u32, unsigned long)
		AC_CHECK_TYPE(__s32, signed long)
	else
		# can't determine default value in case the type isn't present
		AC_MSG_ERROR(CHECK_TYPE__32 - please report to openquicktime-devel@lists.sourceforge.net)
	fi
	
	if test "${ac_cv_sizeof_long}" = "8" ; then
		AC_CHECK_TYPE(__u64, unsigned long)
		AC_CHECK_TYPE(__s64, signed long)
	elif test "${ac_cv_sizeof_long_long}" = "8" ; then
		AC_CHECK_TYPE(__u64, unsigned long long)
		AC_CHECK_TYPE(__s64, signed long long)
	else
		# can't determine default value in case the type isn't present
		AC_MSG_ERROR(CHECK_TYPE__64 - please report to openquicktime-devel@lists.sourceforge.net)
	fi

])



dnl ********
dnl Inlining (11/16/94 Jim Avera (jima@netcom.com))
dnl ********

dnl MY_C_INLINE - like AC_C_INLINE but works for compilers other than gcc:
dnl
dnl  Do nothing if gthe compiler (whatever it is) accepts the inline keyword.
dnl  Otherwise, define inline to __inline__ or  __inline if one of those work,
dnl  else define inline to be empty.   
dnl
AC_DEFUN(MY_C_INLINE,[dnl
  AC_MSG_CHECKING([for inline syntax])
  AC_CACHE_VAL([ac_cv_c_inline],[
    ac_cv_c_inline=NONE
    for confword in inline __inline__ __inline ; do
      AC_TRY_COMPILE(, [} $confword foo() {], [ac_cv_c_inline=$confword; break])
    done
  ])dnl
  AC_MSG_RESULT($ac_cv_c_inline)
  case "$ac_cv_c_inline" in
    inline) ;;
    NONE)   AC_DEFINE([inline],[]) ;;
    *)      AC_DEFINE_UNQUOTED([inline],$ac_cv_c_inline)
  esac
])



dnl *********************
dnl dlsym and Underscores
dnl *********************

dnl Check checks whether dlsym (if present) requires a leading underscore.
dnl Written by Dan Hagerty <hag@ai.mit.edu> for scsh-0.5.0.
AC_DEFUN(MY_DLSYM_USCORE, [
  AC_MSG_CHECKING(for underscore before symbols)
  AC_CACHE_VAL(my_cv_uscore,[
    echo "main(){int i=1;}
    fnord(){int i=23; int ltuae=42;}" > conftest.c
    ${CC} conftest.c > /dev/null
    if (nm a.out | grep _fnord) > /dev/null; then
      my_cv_uscore=yes
    else
      my_cv_uscore=no
    fi])
  AC_MSG_RESULT($my_cv_uscore)
  rm -f conftest.c a.out

  if test x$my_cv_uscore = xyes; then
    AC_DEFINE(USCORE)

    if test x$ac_cv_func_dlopen = xyes -o x$ac_cv_lib_dl_dlopen = xyes; then
	AC_MSG_CHECKING(whether dlsym always adds an underscore for us)
	AC_CACHE_VAL(my_cv_dlsym_adds_uscore,AC_TRY_RUN( [
#include <dlfcn.h>
#include <stdio.h>
fnord() { int i=42;}
main() { void *self, *ptr1, *ptr2; self=dlopen(NULL,RTLD_LAZY);
    if(self) { ptr1=dlsym(self,"fnord"); ptr2=dlsym(self,"_fnord");
    if(ptr1 && !ptr2) exit(0); } exit(1); } 
], [my_cv_dlsym_adds_uscore=yes
	AC_DEFINE(DLSYM_ADDS_USCORE) ], my_cv_dlsym_adds_uscore=no,
	my_cv_dlsym_adds_uscore=no))

        AC_MSG_RESULT($my_cv_dlsym_adds_uscore)
    fi
  fi
])



dnl ************************
dnl Assembly langage support
dnl ************************


dnl MY_TRY_AS(asm program,action if sucess,action if failure)
dnl ---------------------------------------------------------
dnl try to compile an assembly program
dnl you should have called MY_PROG_AS before

AC_DEFUN(MY_TRY_AS,[
	cat > conftest.s <<__EOF
$1
__EOF
	my_asm='$AS $ASFLAGS conftest.s -o conftest.${ac_objext} 1>&5'
	if { (eval echo configure:__oline__: \"$my_asm\") 1>&5; (eval $my_asm) 2>&5; };
	then
		rm -rf conftest*
		$2
	else
		echo "configure: failed program was:" >&5
		cat conftest.s >&5
		rm -rf conftest*
		$3
	fi
	rm -rf conftest*
])



dnl MY_RUN_AS(asm program,action if sucess,action if failure)
dnl ---------------------------------------------------------
dnl try to compile, link and execute an assembly program
dnl you should have called MY_PROG_AS before

AC_DEFUN(MY_RUN_AS,[
	cat > conftest.s <<__EOF
.globl main
main:
$1
__EOF
	my_asm='$AS $ASFLAGS conftest.s -o conftest.${ac_objext} 1>&5; ${CC-cc} $LDFLAGS conftest.${ac_objext} -o conftest${ac_exeext} 1>&5'
	if { (eval echo configure:__oline__: \"$my_asm\") 1>&5; (eval $my_asm) 2>&5; } && test -s conftest${ac_exeext} && (./conftest; exit) 2>/dev/null
	then
		rm -rf conftest*
		$2
	else
		echo "configure: failed program was:" >&5
		cat conftest.s >&5
		rm -rf conftest*
		$3
	fi
	rm -rf conftest*
])



dnl MY_PROG_AS
dnl ----------
dnl set my_cv_as_working to yes if a working assembler is found
dnl set AS to point to the assembler
dnl you can then use "$AS $ASFLAGS toto.s -o toto.o" in your Makefile

AC_DEFUN(MY_PROG_AS,[
	if test x$AS = x; then
		AS="${CC-cc} -c"
	fi
	AC_SUBST(AS)
	AC_SUBST(ASFLAGS)
	AC_CACHE_CHECK(whether the assembler ($AS $ASFLAGS) works,
		my_cv_as_working,
		[MY_TRY_AS(,my_cv_as_working=yes,my_cv_as_working=no)])
])



dnl ***********
dnl MMX support
dnl ***********


dnl MY_AS_MMX
dnl ---------
dnl set my_cv_as_has_mmx if the assembler supports MMX instructions
dnl set my_cv_mmx_working if the host can execute MMX instructions
dnl you should have called MY_PROG_AS before

dnl Now inspired from mmx.h "MultiMedia eXtensions GCC interface library"
dnl First see if mmx instructions are supported by the assembler.
dnl Then try to find out wheather the host supports cpuid and mmx.

AC_DEFUN(MY_AS_MMX,[
	case $target in
	i?86*)
		AC_CACHE_CHECK(whether the assembler ($AS) supports MMX,
		my_cv_as_has_mmx,
		[MY_TRY_AS([
			cpuid
			psubusb %mm0, %mm0
		],my_cv_as_has_mmx=yes,my_cv_as_has_mmx=no)])

		AC_CACHE_CHECK(whether MMX instructions are working on host,
		my_cv_mmx_working,
		[MY_RUN_AS([
			pushl %ebx	
			pushf
			popl %eax
			orl  $ 0x200000, %eax
			push %eax
			popf
			pushf
			popl %eax
			testl $ 0x200000, %eax
			jz   no_mmx

			movl $ 1, %eax
			cpuid
			testl $ 0x800000, %edx
			jnz mmx

			no_mmx:
			movl $ 1, %eax
			popl %ebx
			ret
			mmx:
			movl $ 0, %eax
			popl %ebx
			ret
		],my_cv_mmx_working=yes,my_cv_mmx_working=no)]);;
	*) my_cv_as_has_mmx=no
	   my_cv_mmx_working=no;;
	esac
])




dnl ******************
dnl Compilation switch
dnl ******************


dnl MY_C_SWITCH(switch)
dnl -------------------
dnl try to compile and link a simple C program with the switch compile switch
dnl "${CC-cc} $CFLAGS $1 conftest.c -o conftest"
dnl sets my_cc_switch to switch if it worked
dnl my_cc_switch is not modified elsewhere

AC_DEFUN(MY_C_SWITCH,[
#	AC_MSG_CHECKING(for $1 as C compilation switch)
	cat > conftest.c <<__EOF
int main() { return 0;}
__EOF
	my_c='${CC-cc} $CFLAGS $1 conftest.c -o conftest${ac_exeext}'
	(eval echo configure:__oline__: \"$my_c\") 1>&5
	(eval $my_c 1>/dev/null 2>conftest.log)
	cat conftest.log 1>&5
	if grep <conftest.log option  >/dev/null || 
	   grep <conftest.log ERROR >/dev/null
	then
		my_cc_switch="$my_cc_switch"
#		AC_MSG_RESULT(no)
	else	
		my_cc_switch="$1"
#		AC_MSG_RESULT(yes)
	fi
	rm -rf conftest*
])



dnl MY_C_COMMENT
dnl ------------
dnl set my_cv_comment to the C compiler flag needed to support C++-style 
dnl comments
dnl gcc half-recognises -xCC, so we check target and compiler name before...

AC_DEFUN(MY_C_COMMENT,[
	my_cv_comment=
	case ${target} in
	*solaris*)
		case ${CC-cc} in
		cc) 	my_cc_switch=
			MY_C_SWITCH(-xCC)
			my_cv_comment=$my_cc_switch;;
		esac;;
	esac
	echo "C++-style comment flag ................ $my_cv_comment"
])


dnl MY_C_OPTIM
dnl ----------
dnl set my_cv_optim to the C compiler optimizing switches

AC_DEFUN(MY_C_OPTIM,[
	AC_CACHE_VAL(my_cv_optim,[

	my_cc_switch=

	MY_C_SWITCH(-O)   # everybody likes this
	MY_C_SWITCH(-O2)  # seems common


	case ${CC-cc} in
	gcc*)
	# gcc
	MY_C_SWITCH(-O3 -ffast-math -funroll-loops -fstrength-reduce) # for gcc
	my_bak_switch=$my_cc_switch
	case $target in   # intel gcc
	i686*)  MY_C_SWITCH($my_bak_switch -m486)	
		MY_C_SWITCH($my_bak_switch -march=i486)
		MY_C_SWITCH($my_bak_switch -march=i586);;
# MY_C_SWITCH($my_bak_switch -march=i686);; 
	i586*) 	MY_C_SWITCH($my_bak_switch -m486)	
		MY_C_SWITCH($my_bak_switch -march=i486)
		MY_C_SWITCH($my_bak_switch -march=i586);; 
	i486*) 	MY_C_SWITCH($my_bak_switch -m486)	
		MY_C_SWITCH($my_bak_switch -march=i486);; 
	esac;;
	*)
	# other platforms
	case $target in
	*sparc*solaris*) MY_C_SWITCH(-xO4 -fsimple=2 -dalign -fsingle -xlibmil);; # cc
	i?86*solaris*) MY_C_SWITCH(-xO4 -nofstore -fsingle -xlibmil);; # cc
	*solaris*) MY_C_SWITCH(-xO4);; # cc
	*sunos*) MY_C_SWITCH(-fast -native -O4 -bsdmalloc)  # acc
		 MY_C_SWITCH(-O4 -dalign);; # cc
	*irix*5.*)  MY_C_SWITCH(-O2 -sopt,-r=3,-so=4,-o=5,-lo=s);; # cc
	*irix*6.*)  MY_C_SWITCH(-O2 -n32 -OPT:roundoff=3:IEEE_arithmetic=3);; # cc
	esac;;
	esac
	my_cv_optim=$my_cc_switch])
	echo "C optimization switchs ................ $my_cv_optim"
])



dnl ***********
dnl Link switch
dnl ***********


dnl MY_LD_SWITCH(switch)
dnl --------------------
dnl try to compile and link a simple C program with the switch link switch
dnl "${CC-cc} $LDFLAGS $1 conftest.o -o conftest"
dnl set my_ld_switch to switch if it worked
dnl my_ld_switch is not modified elsewhere

AC_DEFUN(MY_LD_SWITCH,[
#	AC_MSG_CHECKING(for $1 as link (${CC-cc} $LDSWITCH) switch)
	cat > conftest.c <<__EOF
int main() { return 0;}
__EOF
	my_c='${CC-cc} $CCFLAGS -c conftest.c -o conftest.${ac_objext}; ${CC-cc} $1 $LDFLAGS conftest.${ac_objext} -o conftest${ac_exeext}'

	(eval echo configure:__oline__: \"$my_c\") 1>&5
	(eval $my_c 1>/dev/null 2>conftest.log)
	cat conftest.log 1>&5
	if grep <conftest.log option  >/dev/null || 
	   grep <conftest.log ERROR >/dev/null
	then
		my_ld_switch="$my_ld_switch"
#		AC_MSG_RESULT(no)
	else
		my_ld_switch="$1"
#		AC_MSG_RESULT(yes)
	fi
	rm -rf conftest*
])





dnl *****************
dnl Check for pthread 
dnl *****************

dnl ACX_PTHREAD([ACTION-IF-FOUND[, ACTION-IF-NOT-FOUND]]) 
dnl -----------------------------------------------------
dnl
dnl Taken from GNO macro archvies:
dnl http://www.gnu.org/software/ac-archive/htmldoc/acx_pthread.html
dnl

AC_DEFUN([ACX_PTHREAD], [
AC_REQUIRE([AC_CANONICAL_HOST])
AC_LANG_SAVE
AC_LANG_C
acx_pthread_ok=no

# We used to check for pthread.h first, but this fails if pthread.h
# requires special compiler flags (e.g. on True64 or Sequent).
# It gets checked for in the link test anyway.

# First of all, check if the user has set any of the PTHREAD_LIBS,
# etcetera environment variables, and if threads linking works using
# them:
if test x"$PTHREAD_LIBS$PTHREAD_CFLAGS" != x; then
        save_CFLAGS="$CFLAGS"
        CFLAGS="$CFLAGS $PTHREAD_CFLAGS"
        save_LIBS="$LIBS"
        LIBS="$PTHREAD_LIBS $LIBS"
        AC_MSG_CHECKING([for pthread_join in LIBS=$PTHREAD_LIBS with CFLAGS=$PTHREAD_CFLAGS])
        AC_TRY_LINK_FUNC(pthread_join, acx_pthread_ok=yes)
        AC_MSG_RESULT($acx_pthread_ok)
        if test x"$acx_pthread_ok" = xno; then
                PTHREAD_LIBS=""
                PTHREAD_CFLAGS=""
        fi
        LIBS="$save_LIBS"
        CFLAGS="$save_CFLAGS"
fi

# We must check for the threads library under a number of different
# names; the ordering is very important because some systems
# (e.g. DEC) have both -lpthread and -lpthreads, where one of the
# libraries is broken (non-POSIX).

# Create a list of thread flags to try.  Items starting with a "-" are
# C compiler flags, and other items are library names, except for "none"
# which indicates that we try without any flags at all.

acx_pthread_flags="pthreads none -Kthread -kthread lthread -pthread -pthreads -mthreads pthread --thread-safe -mt"

# The ordering *is* (sometimes) important.  Some notes on the
# individual items follow:

# pthreads: AIX (must check this before -lpthread)
# none: in case threads are in libc; should be tried before -Kthread and
#       other compiler flags to prevent continual compiler warnings
# -Kthread: Sequent (threads in libc, but -Kthread needed for pthread.h)
# -kthread: FreeBSD kernel threads (preferred to -pthread since SMP-able)
# lthread: LinuxThreads port on FreeBSD (also preferred to -pthread)
# -pthread: Linux/gcc (kernel threads), BSD/gcc (userland threads)
# -pthreads: Solaris/gcc
# -mthreads: Mingw32/gcc, Lynx/gcc
# -mt: Sun Workshop C (may only link SunOS threads [-lthread], but it
#      doesn't hurt to check since this sometimes defines pthreads too;
#      also defines -D_REENTRANT)
# pthread: Linux, etcetera
# --thread-safe: KAI C++

case "${host_cpu}-${host_os}" in
        *solaris*)

        # On Solaris (at least, for some versions), libc contains stubbed
        # (non-functional) versions of the pthreads routines, so link-based
        # tests will erroneously succeed.  (We need to link with -pthread or
        # -lpthread.)  (The stubs are missing pthread_cleanup_push, or rather
        # a function called by this macro, so we could check for that, but
        # who knows whether they'll stub that too in a future libc.)  So,
        # we'll just look for -pthreads and -lpthread first:

        acx_pthread_flags="-pthread -pthreads pthread -mt $acx_pthread_flags"
        ;;
esac

if test x"$acx_pthread_ok" = xno; then
for flag in $acx_pthread_flags; do

        case $flag in
                none)
                AC_MSG_CHECKING([whether pthreads work without any flags])
                ;;

                -*)
                AC_MSG_CHECKING([whether pthreads work with $flag])
                PTHREAD_CFLAGS="$flag"
                ;;

                *)
                AC_MSG_CHECKING([for the pthreads library -l$flag])
                PTHREAD_LIBS="-l$flag"
                ;;
        esac

        save_LIBS="$LIBS"
        save_CFLAGS="$CFLAGS"
        LIBS="$PTHREAD_LIBS $LIBS"
        CFLAGS="$CFLAGS $PTHREAD_CFLAGS"

        # Check for various functions.  We must include pthread.h,
        # since some functions may be macros.  (On the Sequent, we
        # need a special flag -Kthread to make this header compile.)
        # We check for pthread_join because it is in -lpthread on IRIX
        # while pthread_create is in libc.  We check for pthread_attr_init
        # due to DEC craziness with -lpthreads.  We check for
        # pthread_cleanup_push because it is one of the few pthread
        # functions on Solaris that doesn't have a non-functional libc stub.
        # We try pthread_create on general principles.
        AC_TRY_LINK([#include <pthread.h>],
                    [pthread_t th; pthread_join(th, 0);
                     pthread_attr_init(0); pthread_cleanup_push(0, 0);
                     pthread_create(0,0,0,0); pthread_cleanup_pop(0); ],
                    [acx_pthread_ok=yes])

        LIBS="$save_LIBS"
        CFLAGS="$save_CFLAGS"

        AC_MSG_RESULT($acx_pthread_ok)
        if test "x$acx_pthread_ok" = xyes; then
                break;
        fi

        PTHREAD_LIBS=""
        PTHREAD_CFLAGS=""
done
fi

# Various other checks:
if test "x$acx_pthread_ok" = xyes; then
        save_LIBS="$LIBS"
        LIBS="$PTHREAD_LIBS $LIBS"
        save_CFLAGS="$CFLAGS"
        CFLAGS="$CFLAGS $PTHREAD_CFLAGS"

        # Detect AIX lossage: threads are created detached by default
        # and the JOINABLE attribute has a nonstandard name (UNDETACHED).
        AC_MSG_CHECKING([for joinable pthread attribute])
        AC_TRY_LINK([#include <pthread.h>],
                    [int attr=PTHREAD_CREATE_JOINABLE;],
                    ok=PTHREAD_CREATE_JOINABLE, ok=unknown)
        if test x"$ok" = xunknown; then
                AC_TRY_LINK([#include <pthread.h>],
                            [int attr=PTHREAD_CREATE_UNDETACHED;],
                            ok=PTHREAD_CREATE_UNDETACHED, ok=unknown)
        fi
        if test x"$ok" != xPTHREAD_CREATE_JOINABLE; then
                AC_DEFINE(PTHREAD_CREATE_JOINABLE, $ok,
                          [Define to the necessary symbol if this constant
                           uses a non-standard name on your system.])
        fi
        AC_MSG_RESULT(${ok})
        if test x"$ok" = xunknown; then
                AC_MSG_WARN([we do not know how to create joinable pthreads])
        fi

        AC_MSG_CHECKING([if more special flags are required for pthreads])
        flag=no
        case "${host_cpu}-${host_os}" in
                *-aix* | *-freebsd*)     flag="-D_THREAD_SAFE";;
                *solaris* | *-osf* | *-hpux*) flag="-D_REENTRANT";;
        esac
        AC_MSG_RESULT(${flag})
        if test "x$flag" != xno; then
                PTHREAD_CFLAGS="$flag $PTHREAD_CFLAGS"
        fi

        LIBS="$save_LIBS"
        CFLAGS="$save_CFLAGS"

        # More AIX lossage: must compile with cc_r
        AC_CHECK_PROG(PTHREAD_CC, cc_r, cc_r, ${CC})
else
        PTHREAD_CC="$CC"
fi

AC_SUBST(PTHREAD_LIBS)
AC_SUBST(PTHREAD_CFLAGS)
AC_SUBST(PTHREAD_CC)

# Finally, execute ACTION-IF-FOUND/ACTION-IF-NOT-FOUND:
if test x"$acx_pthread_ok" = xyes; then
        ifelse([$1],,AC_DEFINE(HAVE_PTHREAD,1,[Define if you have POSIX threads libraries and header files.]),[$1])
        :
else
        acx_pthread_ok=no
        $2
fi
AC_LANG_RESTORE
])dnl ACX_PTHREAD 





dnl ***
dnl SDL
dnl ***

dnl MY_SDL
dnl -------
dnl set my_cv_sdl_config to the name of the `sdl-config` program or NONE
dnl if SDL is not found

AC_DEFUN(MY_SDL,[
	AC_CACHE_VAL(my_cv_sdl_config,[
	my_cv_sdl_config=NONE

	AC_MSG_CHECKING([for sdl-config program])

	if eval sdl-config --version 2>/dev/null >/dev/null; then 
		my_cv_sdl_config="sdl-config"
		AC_MSG_RESULT([$my_cv_sdl_config])
	else
		AC_MSG_RESULT(FAILED)
		AC_MSG_WARN([sdl-config was not found])
	fi
	])
])


dnl ******
dnl libcurl
dnl ******

dnl MY_CURL
dnl -------
dnl set my_cv_curl_ver to the version of libcurl or NONE
dnl if libcurl is not found or is too old
sdl Thanks to Daniel Stenberg for help with Macro

AC_DEFUN(MY_CURL,[
	AC_CACHE_VAL(my_cv_curl_ver,[
		my_cv_curl_ver=NONE
		check="7.9.7"
		check_hex="070907"
	
		AC_MSG_CHECKING([for curl >= $check])
		
		if eval curl-config --version 2>/dev/null >/dev/null; then
			ver=`curl-config --version | sed -e "s/libcurl //g"`
			hex_ver=`curl-config --vernum | awk '{print substr(toupper(@S|@0),0,8);exit}'`
			
			if test "x$hex_ver" != "xUSAGE: C"; then
				ok=`echo "ibase=16; if($hex_ver>=$check_hex) $hex_ver else 0" | bc`
				
				if test x$ok != x0; then
					my_cv_curl_ver="$ver"
					AC_MSG_RESULT([$my_cv_curl_ver])
				else
					AC_MSG_RESULT(FAILED)
					AC_MSG_WARN([$ver is too old. Need version $check or higher.])
				fi
			else
				AC_MSG_RESULT(FAILED)
				AC_MSG_WARN([$ver is way too old. Need version $check or higher.])
			fi
		else
			AC_MSG_RESULT(FAILED)
			AC_MSG_WARN([curl-config was not found])
		fi
	])
])



