/*
 *  Copyright (c) by Shuu Yamaguchi <shuu@dotaster.com>
 *
 *  $Id: loader.c,v 1.1.1.1 2003/10/14 07:51:34 acyr Exp $
 *
 *  Can be freely distributed and used under the terms of the GNU GPL.
 */
#include	<unistd.h>
#include	<sys/types.h>
#include	<sys/stat.h>
#include	<fcntl.h>
#include	<limits.h>
#include	<string.h>
#include	<stdio.h>
#include	<sys/mman.h>
#include	<errno.h>
#include	<sys/utsname.h>
#include	<sys/wait.h>

#include	"murasaki.h"
#include	"env.h"

#define	CHAR_COMMENT	'#'

#define	FORK_OFF	0
#define	FORK_ON		1

#define	SCRIPT_MAX_ARG	3
#define CLASS_NAME_MAX	16

static void
executing(char *path,char *argv[],int process)
{
	pid_t pid;
	
		pid = fork();
		switch(pid) {
		case -1:
			syslog(LOG_LEVEL,"fork error err=%d",errno);
			break;
		case 0:	/* child */
			execvp(path,argv);
			beep(INVALID);
			syslog(LOG_LEVEL,"exec error \"%s\" err=%d",path,errno);
			break;
		default:	/* parent */
			wait(NULL);
			break;
		}
}

static void
exec_script(char *argv[],int action)
{
	int i;
	char *cmd[SCRIPT_MAX_ARG];

	if (action == ACTION_ADD){
		cmd[1] = "start";
	}
	else if (action == ACTION_REMOVE_READY){
		cmd[1] = "ready";
	}

	else { 
		cmd[1] = "stop";
	}
	cmd[2] = "sda";

	cmd[3] = NULL;
	for(i=0;argv[i] != NULL;i++) {
		cmd[0] = argv[i];
		syslog(LOG_LEVEL,"\"%s\" \"%s\" \"%s\"",cmd[0],cmd[1],cmd[2]);
		executing(cmd[0],cmd,FORK_ON);
	}
}

int
load_module(struct mu_op *opp,int action)
{
	int i,k;
	char *loader;
	char *default_path;
	int count,sum;
	MU_usb_config_t *config = opp->config;
	
	inform_config(opp->config);	/* print device configuration */

	/* create command */
	module_list[0] = "/sbin/modprobe" ;
	if (action == ACTION_ADD)
		module_list[1] = "-as";
	else 	/* ACTION_REMOVE */
		module_list[1] = "-asr";

	/* check device interface class */
	if ( config->interface.class != 0x8 ){
		if ( action == ACTION_REMOVE && config->interface.class == 0xff ){
			syslog(LOG_LEVEL, "USBDP mode!");
		}
		else {
			syslog(LOG_LEVEL,"This device is not USB Mass Storage Device...");
			return INVALID;
		}
	}
	else {
		syslog(LOG_LEVEL,"This device is USB Mass Storage Device!!");
	}
	module_list[2]="usb-storage";
	module_list[3]=NULL;

	/* get script names */
	script_list[0]="/usr/epson/etc/murasaki/usb.sh";
	script_list[1]=NULL;
	syslog(LOG_LEVEL,"get_script result:name=%s\n",script_list[0]);	

	if (count_of_list(script_list) != 0) {
		if (action == ACTION_REMOVE){
			usleep(500000);
			exec_script( script_list, ACTION_REMOVE_READY);
			syslog(LOG_LEVEL,"action ready done\n");
		}	
		executing(module_list[0],module_list,FORK_ON);
		usleep(500000);
		exec_script(script_list,action);	/* not come back */
	}
	/* not reached */
#ifdef	KERNEL_JOB	
	free(module_name);
	for(i = START_ARG+1;module_list[i] != NULL;i++)
		free(module_list[i]);
	for(i = 0;script_list[i] != NULL;i++)
		free(script_list[i]);
#endif	/* KERNEL_JOB */

	return GOOD;
}
	
#if 0
int
exec_class(char *class,int action)
{
	char class_name[CLASS_NAME_MAX+3];	/* 3: '['+']'+'\0' */
	char *list[2];

	if (strlen(class) > CLASS_NAME_MAX)
		return INVALID;
	class_name[0] = '[';
	strcpy(class_name+1,class);
	strcat(class_name+1,"]");
	list[0] = class_name;
	list[1] = NULL;
	if (get_script(CALL_PATH,call_list,list,0,LIST_MAX) == INVALID)
		return INVALID;
	exec_script(class_list,action);	/* not come back */
}
#endif
