/*
 *  Copyright (c) by Shuu Yamaguchi <shuu@dotaster.com>
 *
 *  $Id: loader_pci.c,v 1.3 2004/02/27 09:09:41 nozomi Exp $
 *
 *  Can be freely distributed and used under the terms of the GNU GPL.
 */
#include	<unistd.h>
#include	<sys/types.h>
#include	<sys/stat.h>
#include	<fcntl.h>
#include	<limits.h>
#include	<string.h>
#include	<stdio.h>
#include	<sys/mman.h>
#include	<errno.h>
#include	<sys/utsname.h>
#include	<sys/wait.h>

#include	"murasaki.h"
#include	"env.h"

#define	MODPROBLE_FILE "/proc/sys/kernel/modprobe"
#define	MODULE_BASEDIR	"/lib/modules/"
#define	CHAR_COMMENT	'#'

#define	FORK_OFF	0
#define	FORK_ON		1

#define	SCRIPT_MAX_ARG	3
#define CLASS_NAME_MAX	16

static void
executing(char *path,char *argv[],int process)
{
	pid_t pid;

		pid = fork();
		switch(pid) {
		case -1:
			syslog(LOG_LEVEL,"fork error err=%d",errno);
			break;
		case 0:	/* child */
			execvp(path,argv);
			syslog(LOG_LEVEL,"exec error \"%s\" err=%d",path,errno);
			break;
		default:	/* parent */
			wait(NULL);
			break;
		}
}

static int
get_module(char **modules,char *fname,void *config)
{
	struct stat st;
	int fd;
	char *mp;
	int count;

	DPRINTF(LOG_LEVEL, "map file name:%s",fname);
	if ((fd = open(fname,O_RDONLY)) == -1)
		return 0;
	if (fstat(fd,&st) == -1)
		return -1;
	if ((mp = mmap(0,st.st_size,PROT_READ,MAP_SHARED,fd,0)) == MAP_FAILED)
		return -1;
	close(fd);

	if ((count = match_config(modules,mp,mp+st.st_size,config)) == 0) {
		if (debug)
			syslog(LOG_LEVEL,"Not found in %s", fname);
	}
	munmap(mp,st.st_size);

	return count;
}

int
load_module(struct mu_op *opp,int action)
{
	int i;
	char *loader;
	char *default_path;
	int count,sum;

	inform_config(opp->config);	/* print device configuration */

	module_list[0] = "/sbin/modprobe" ;
	/* create command */
	if (action == ACTION_ADD) {
		module_list[1] = "-as";
	}
	else { 	/* ACTION_REMOVE */
		module_list[1] = "-asr";
	}
	/*
	 *  search murasaki.pcimap
	 */
	if ((count = get_module(module_list,opp->mappath,opp->config)) == -1){
		return INVALID;
	}
	sum = count;
	if (sum == 0) {	/* not found */
		syslog(LOG_LEVEL,"The device match nothing in mapfile");
		syslog(LOG_LEVEL,"Please change MODULE in following line to appropriate module name, add it to %s",opp->mappath);
		print_config(opp->config);
		return INVALID;
	}
	/*
	 * adding depended modules & expanding alias modules 
	 */
	if (get_depend(module_list) == INVALID) {
		DPRINTF(LOG_LEVEL,"get_depend return INVALID");
		return INVALID;
	}

	syslog(LOG_LEVEL, "executing %s %s %s", module_list[0], module_list[1], module_list[2]);
	/* execute modprobe */
	executing(module_list[0],module_list,FORK_OFF);
	return GOOD;
}
