/*
 *  Copyright (c) by Shuu Yamaguchi <shuu@dotaster.com>
 *
 *  $Id: loader_usb.c,v 1.2 2003/12/18 02:45:41 nozomi Exp $
 *
 *  Can be freely distributed and used under the terms of the GNU GPL.
 */
#include	<unistd.h>
#include	<sys/types.h>
#include	<sys/stat.h>
#include	<fcntl.h>
#include	<limits.h>
#include	<string.h>
#include	<stdio.h>
#include	<sys/mman.h>
#include	<errno.h>
#include	<sys/utsname.h>
#include	<sys/wait.h>

#include	"murasaki.h"
#include	"env.h"

#define	CHAR_COMMENT	'#'

#define	FORK_OFF	0
#define	FORK_ON		1

#define	SCRIPT_MAX_ARG	3
#define CLASS_NAME_MAX	16

static void
executing(char *path,char *argv[],int process)
{
	pid_t pid;
	
		pid = fork();
		switch(pid) {
		case -1:
			syslog(LOG_LEVEL,"fork error err=%d",errno);
			break;
		case 0:	/* child */
			execvp(path,argv);
			beep(INVALID);
			syslog(LOG_LEVEL,"exec error \"%s\" err=%d",path,errno);
			break;
		default:	/* parent */
			wait(NULL);
			break;
		}
}

static void
exec_script(char *argv[],int action)
{
	int i;
	char *cmd[SCRIPT_MAX_ARG];

	cmd[0] = argv[0];
	if (action == ACTION_ADD){
		cmd[1] = "start";
	}
	else if (action == ACTION_REMOVE_READY){
		cmd[1] = "ready";
	}

	else { 
		cmd[1] = "stop";
	}
	cmd[2] = "sda";

	cmd[3] = NULL;
	executing(cmd[0],cmd,FORK_ON);
}

int
load_module(struct mu_op *opp,int action)
{
	int i,k;
	char *loader;
	char *default_path;
	int count,sum;
	MU_usb_config_t *config = opp->config;
	
	inform_config(opp->config);	/* print device configuration */

	/* create command */
	module_list[0] = "/sbin/modprobe" ;
	if (action == ACTION_ADD)
		module_list[1] = "-as";
	else 	/* ACTION_REMOVE */
		module_list[1] = "-asr";

	/* check device interface class */
	if ( config->interface.class != 0x8 ){
		if ( action == ACTION_REMOVE && config->interface.class == 0xff ){
			syslog(LOG_LEVEL, "USBDP mode!");
		}
		else {
			syslog(LOG_LEVEL,"This device is not USB Mass Storage Device...");
			return INVALID;
		}
	}
	else {
		syslog(LOG_LEVEL,"This device is USB Mass Storage Device!!");
	}
	module_list[2]="usb-storage";
	module_list[3]=NULL;

	/* get script names */
	script_list[0]="/usr/epson/etc/murasaki/usb.sh";
	script_list[1]=NULL;

	if (count_of_list(script_list) != 0) {
		if (action == ACTION_REMOVE){
			usleep(500000);
			exec_script( script_list, ACTION_REMOVE_READY);
			syslog(LOG_LEVEL,"action ready done\n");
		}	
		executing(module_list[0],module_list,FORK_ON);
		usleep(500000);
		exec_script(script_list,action);	/* not come back */
	}
	return GOOD;
}
