/*
    Copyright (C) SEIKO EPSON CORPORATION 2002-2004. All Rights Reserved.

    3-3-5, Owa, Suwa-shi Nagano-ken, 392-8502 Japan
    Phone: +81-266-52-3131

    This file is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This file is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this file; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include <limits.h>
#include "SDL_util.h"
#include "config.h"
#include "UIRsrcMgr.h"
#include "VncNavGuide.h"
#include "InfoBar.h"
#include "Item.h"
#include "DialogBtn.h"
#include "vncviewer.h"
#include "SDL_stretch.h"
#include "StatusBar.h"
#include "StatusItem.h"
#include "Dialog.h"

const int BTN_WIDTH     = 188;
const int BTN_HEIGHT    = 34;
const int BTN_SPACING   = 50;
const int BTN_THIN      = 3;
const int BTN_TOP_SPACE = 13;

const int NAVBAR_HEIGHT    = 42;
const int STATUSBAR_HEIGHT = 184;


enum NetworkMode {
    NETWORKMODE_ESSID,          
    NETWORKMODE_ACCESSPOINT,    
    NETWORKMODE_WIRED           
};

Dialog::
Dialog		(SDL_Surface		*screen,
			 const ::SDL_Rect&	rect)
    : BaseDialog(screen,rect)
{
	
	m_AlphaValue	= UCHAR_MAX;
	m_NavGuide		= NULL;
	m_InfoBar		= NULL;
	m_msgbar		= NULL;
}

Dialog::
~Dialog()
{
	delete m_NavGuide;
	delete m_InfoBar;
    delete m_msgbar;
	m_restore = false;
}


int Dialog::
Init	()
{    
    int result;
    SDL_Rect rect;

	
    rect = INFOBAR_DEFAULT_RECT;
    rect.w = m_screen->w;
	rect.y = drawable_area.y - rect.h;
	m_InfoBar = new InfoBar(m_screen, rect); 
	if (!m_InfoBar) {
		return -1;
	}	
	result = m_InfoBar->Init();
	if (result != 0) {
		return result;
	}	

    
    rect.x = 0; 
    rect.y = drawable_area.y;
    rect.w = m_screen->w;
    rect.h = STATUSBAR_HEIGHT;
    m_msgbar = new StatusBar(m_screen, rect);
    m_msgbar->Init();

    
    CreateStatusBar();

    
    rect.w = BTN_WIDTH + BTN_THIN * 2;
    rect.h = BTN_HEIGHT + BTN_THIN * 2;
    rect.x = (drawable_area.w/2) - (BTN_SPACING/2) - rect.w;
    rect.y = drawable_area.y + BTN_TOP_SPACE;

    DialogBtn *p;
    p = new DialogBtn(m_screen, rect, DlgBtn::BtnExit);
    AddGrain(p);
    result = p->Init();
    if (result != 0) {
        return result;
    }

    
    rect.x = drawable_area.w / 2 + BTN_SPACING / 2;
    p = new DialogBtn(m_screen, rect, DlgBtn::BtnContinue);
    AddGrain(p);
    result = p->Init();
    if (result != 0) {
        return result;
    }

    
	p->Select();

	
	m_NavGuide = new VncNavGuide(m_screen);
	if (!m_NavGuide) {
		return -1;
	}
	result = m_NavGuide->Init();
	if (result != 0) {
		return result;
	}
	result = SetNavGuide();
	if (result != 0) {
		return result;
	}

    UpdateResources();

	return 0;
}

int Dialog::
SetNavGuide	()
{
	assert(m_NavGuide);

    int result;
    NavGuide *p = m_NavGuide;
	result = p->SetIconDescr(NavGuide::Icon1, 
                             NavItem::IconEnter, 
                             NavItem::DescrDecide);
	if (result != 0) { 
		return result;
	}

	result = p->SetIconDescr(NavGuide::Icon2,
                             NavItem::IconHoriz,
                             NavItem::DescrSelect);
	if (result != 0) { 
		return result;
	}

	result = p->SetIconDescr(NavGuide::Icon3,
                             NavItem::IconESC,
                             NavItem::DescrBack);
	if (result != 0) { 
		return result;
	}

	result = p->SetIconDescr(NavGuide::Icon4,
                             NavItem::IconHidden,
                             NavItem::DescrDecide);
	if (result != 0) { 
		return result;
	}

	result = p->SetIconDescr(NavGuide::Icon5,
                             NavItem::IconHidden,
                             NavItem::DescrDecide);
	if (result != 0) { 
		return result;
	}

	return 0;
}


int Dialog::
Draw	()
{
	assert(m_NavGuide);
	assert(m_InfoBar);
	assert(m_msgbar);

    int result;

	result = BaseDialog::Draw();
	if (result != 0) {
		return result;
	}

	DebugPrint("drawing infobar...\n");
	result = m_InfoBar->Draw();
	if (result != 0) {
		return result;
	}

	DebugPrint("drawing message bar...\n");
	result = m_msgbar->Draw();
	if (result != 0) {
		return result;
	}

	
    Grain *p = m_child;
	while (p) {
		DebugPrint("drawing child 0x%8.8x...\n", (int)p);
		result = p->Draw();
		if (result != 0) {
			return result;
		}
		p = p->GetNext();
	}	

	DebugPrint("drawing navigation guide...\n");
	result = m_NavGuide->Draw();
	if (result != 0) {
		return result;
	}

	return 0;
}


void Dialog::
Update		()
{
	SDL_Rect update = { 0, 472, 1024, 296 };
	UpdateRect(m_screen, update);
}

ProcStatus Dialog::
Event	(const ::SDL_Event& event)
{
    Grain *p = m_child;
	while (p) {
		assert(dynamic_cast<DialogBtn *>(p));
		if (static_cast<DialogBtn*>(p)->IsSelected()) {
            DebugPrint("%s: Passing event to child widget\n", __FUNCTION__);
			return p->Event(event);
		}
		p = p->GetNext();
	}
	return ProcKeepGoing;
}


void Dialog::
DoCallback  (const DlgBtn::BtnType type)
{
    DebugPrint("Entering %s\n", __FUNCTION__);
    if ( type == DlgBtn::BtnExit ) {
        DebugPrint("Exit button selected.  Closing VNC\n");
        ::Cleanup(CLIENT_QUIT);
    } else if ( type == DlgBtn::BtnContinue ) {
        DebugPrint("Continue button selected.  Closing dialog window\n");
        ::CloseDialog();
    }
}

void
Dialog::UpdateResources()
{
    SDL_Surface* infoImage = 
        UIRsrcMgr::Instance()->GetResource(UIRsrcMgr::msgBarBack1);

    if ( infoImage ) {
        m_InfoBar->SetBackground(infoImage);
    }
    infoImage = UIRsrcMgr::Instance()->GetResource(UIRsrcMgr::labGuide3);
    if ( infoImage ) {
        SDL_Rect rect;
        rect.w = infoImage->w; rect.h = infoImage->h;        
        rect.x = drawable_area.w/2 - rect.w/2; rect.y = 33 - rect.h/2;
        m_InfoBar->SetForeground(infoImage, rect);
    }
}


void
Dialog::SelectButton(DlgBtn::BtnType type)
{
	Grain* p = m_child;
	while (p) {
		assert(dynamic_cast<DialogBtn*>(m_child));
		DialogBtn* btn = static_cast<DialogBtn*>(p);

		if ( btn->GetButtonType() == type ) {
			btn->Select();
		} else {
			btn->Unselect();
		}

		p = p->GetNext();
	}
}


void
Dialog::CreateStatusBar()
{
    SDL_Surface* image;

    m_msgbar->RemoveAllStatusItems();

    
    image = SDL_LoadImage(RESOURCE_DIR "dlgBack2.png", 
                          NULL, SDL_ROTATE_0, 
                          SDL_STRETCH_BILINEAR, SDL_FALSE);
    if ( image == 0 ) {
        DebugPrint("Unable to load %s\n", "dlgBack2.png");
        return;
    }
    m_msgbar->SetBackground(image);


    
    m_msgbar->AddStatusItem(UIRsrcMgr::erdTagProjectorName, 
                            appData.confHostname);

    switch ( appData.confNetworkMode ) {
    case NETWORKMODE_ACCESSPOINT:
        
        m_msgbar->AddStatusItem(UIRsrcMgr::erdTagIPAddress, 
                                appData.confAddress);
        break;

    case NETWORKMODE_ESSID:
        
        m_msgbar->AddStatusItem(UIRsrcMgr::erdTagESSID, 
                                appData.confEssid);
        break;

    case NETWORKMODE_WIRED:
        
        m_msgbar->AddStatusItem(UIRsrcMgr::erdTagIPAddress,
                                appData.confAddress);
        break;
    default:
        break;
    }

    if ( strlen(appData.confKeyword) > 0 ) {
        m_msgbar->AddStatusItem(UIRsrcMgr::erdTagKeyword,
                                appData.confKeyword);
    }
}
