/*
    Copyright (C) SEIKO EPSON CORPORATION 2002-2004. All Rights Reserved.

    3-3-5, Owa, Suwa-shi Nagano-ken, 392-8502 Japan
    Phone: +81-266-52-3131

    This file is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This file is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this file; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include <new>
#include "debug.h"
#include "StatusItem.h"
#include "StatusBar.h"


StatusBar::StatusBar(SDL_Surface *screen, const SDL_Rect& rect)
    : InfoBar(screen,rect), 
      m_SeparatorImage(UIRsrcMgr::erdTagSeparator)
{
}




StatusBar::~StatusBar()
{
    RemoveAllStatusItems();
}


int
StatusBar::Init()
{
    return InfoBar::Init();
}



int
StatusBar::Draw()
{
    int status;

    
    status = InfoBar::Draw();
    if ( status != 0 ) {
        return status;
    }

    
    DrawStaticItems();

    
    StatusItemList::iterator iter;
    for (iter = m_items.begin(); iter != m_items.end(); iter++) {
        status = (*iter)->Draw();
        if ( status != 0 ) {
            return status;
        }
    }

    return 0;
}



void
StatusBar::DrawStaticItems() const
{
    struct {
        SDL_Rect area;
        Uint32   colour;
    } rects[] = 
        {
            {{145, 716, 732, 1}, SDL_MapRGB(m_screen->format, 51, 51, 51)},
            {{876, 609, 1, 108}, SDL_MapRGB(m_screen->format, 51, 51, 51)},
            {{146, 715, 730, 1}, SDL_MapRGB(m_screen->format, 66, 66, 66)},
            {{875, 610, 1, 105}, SDL_MapRGB(m_screen->format, 66, 66, 66)},
            {{145, 608, 732, 1}, SDL_MapRGB(m_screen->format, 20, 20, 20)},
            {{145, 608, 1, 107}, SDL_MapRGB(m_screen->format, 20, 20, 20)},
            {{146, 609, 730, 1}, SDL_MapRGB(m_screen->format, 0, 0, 0)},
            {{146, 609, 1, 106}, SDL_MapRGB(m_screen->format, 0, 0, 0)}
        };
    
    for (Uint32 i = 0; i < (sizeof(rects)/sizeof(rects[0])); ++i) {
        
        
        
        SDL_Rect tmpRect = rects[i].area;
        tmpRect.x += (m_screen->w - 1024) / 2;
        tmpRect.y += drawable_area.y - 537;

        
        SDL_FillRect(m_screen, &tmpRect, rects[i].colour);
    }
}



int 
StatusBar::SetSeparatorImage(UIRsrcMgr::Rsrc separator)
{
    m_SeparatorImage = separator;

    
    StatusItemList::iterator iter;
    for (iter = m_items.begin(); iter != m_items.end(); iter++) {
        if (*iter) {
            int status = (*iter)->SetSeparatorImage(m_SeparatorImage, 0);
            if ( status != 0 ) {
                return status;
            }
        }
    }

    return 0;
}



int
StatusBar::AddStatusItem(UIRsrcMgr::Rsrc label, const char* text)
{
    StatusItem* newItem;

    newItem = new(std::nothrow) StatusItem(m_screen);
    if ( newItem == 0 ) {
        return -1;
    }
    newItem->Init();

    if ( newItem->SetLabelImage(label) != 0) {
        delete newItem;
        return -2;
    }

    if (newItem->SetSeparatorImage(m_SeparatorImage) != 0) {
        delete newItem;
        return -3;
    }

    if ( newItem->SetTextData(text) != 0 ) {
        delete newItem;
        return -4;
    }

    int anchorX = m_UpdateArea.x + m_UpdateArea.w / 2;
    int anchorY = m_UpdateArea.y + 77 +
        GetStatusItemCount() * VerticalItemSpacing;

    DebugPrint("Anchor: (%d,%d)\n", anchorX, anchorY);
    newItem->SetAnchorPoint(anchorX, anchorY);

    m_items.push_back(newItem);

    return 0;
}



int
StatusBar::RemoveStatusItem(unsigned int index)
{
    if ( index > m_items.size() ) {
        return -1;
    }

    
    StatusItemList::iterator iter = m_items.begin();
    for ( unsigned int i = 0; i < index; i++ ) {
        iter++;
    }

    
    StatusItem* tmp = (*iter);
    m_items.erase(iter);

    
    delete tmp;

    return 0;
}



StatusItem&
StatusBar::GetStatusItem(unsigned int index) const
{
    assert( index < m_items.size() );

    return *(m_items[index]);
}



int
StatusBar::GetStatusItemCount() const
{
    return m_items.size();
}



void
StatusBar::RemoveAllStatusItems()
{
    StatusItemList::iterator iter = m_items.begin();
    for (; iter != m_items.end(); iter++ ) {
        delete (*iter);
    }
    m_items.clear();
}
