/*
    Copyright (C) SEIKO EPSON CORPORATION 2002-2004. All Rights Reserved.

    3-3-5, Owa, Suwa-shi Nagano-ken, 392-8502 Japan
    Phone: +81-266-52-3131

    This file is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This file is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this file; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/



#include <assert.h>
#include "SDL_text.h"
#include "ImageArray.h"
#include "StatusItem.h"


#ifdef mips
static const char* FONTPATH = "/usr/epson/rsrc/fonts/epson1224.pcf";
#else
static const char* FONTPATH = "../rsrc/fonts/epson1224.pcf";
#endif


StatusItem::StatusItem(SDL_Surface *screen, const SDL_Rect& rect)
    : Grain(screen,rect), m_images(0)
{
    m_UpdateArea = drawable_area;
}



StatusItem::~StatusItem()
{
    delete m_images;
}



int
StatusItem::Init()
{
    assert( m_images == 0 );

    
    m_images = new(std::nothrow) ImageArray(m_screen, m_UpdateArea);
    if ( m_images == 0 ) {
        return -1;
    }

    return m_images->Init(TotalCells);
}



int
StatusItem::Draw()
{
    assert( m_images != 0);

    m_images->SetPosition(m_UpdateArea.x, m_UpdateArea.y);
    return m_images->Draw();
}



void
StatusItem::SetAnchorPoint(int x, int y)
{
    assert( m_images != 0);

    m_UpdateArea.x = x - m_images->GetCellWidth(LabelIndex);
    m_UpdateArea.y = y;
}



int
StatusItem::SetLabelImage(UIRsrcMgr::Rsrc label, int width)
{
    assert( m_images != 0);
    assert(width >= 0);

    SDL_Surface* labelSurface;
    labelSurface = UIRsrcMgr::Instance()->GetResource(label);
    if ( labelSurface == 0 ) {
        return -1;
    }

    if ( width < labelSurface->w ) {
        width = labelSurface->w;
    }

    
    
    m_images->SetCell(labelSurface, width, LabelIndex);

    return 0;
}



int
StatusItem::SetSeparatorImage(UIRsrcMgr::Rsrc separator, int width)
{
    assert( m_images != 0);
    assert(width >= 0);

    SDL_Surface* sepSurface;
    sepSurface = UIRsrcMgr::Instance()->GetResource(separator);
    if ( sepSurface == 0 ) {
        return -1;
    }

    if ( width < sepSurface->w ) {
        width = sepSurface->w;
    }

    
    
    m_images->SetCell(sepSurface, width, SeparatorIndex);

    return 0;
}



int
StatusItem::SetTextData(const char* text, int width)
{
    assert( m_images != 0);
    assert(text != 0);
    assert(width >= 0);

    SDL_Surface* textImage;

    
    textImage = SDL_CreateText(text, FONTPATH);
    if ( textImage == 0 ) {
        return -1;
    }

    
    if ( width <= textImage->w ) {
        width = textImage->w;
    }

    SDL_Surface* oldText;
    oldText = m_images->SetCell(textImage, width, TextIndex);
    if ( oldText ) {
        SDL_FreeSurface(oldText);
    }

    return 0;
}



int
StatusItem::GetHeight()
{
    return m_images->GetHeight();
}



int
StatusItem::GetWidth()
{
    return m_images->GetWidth();
}
