/*
 *  Copyright (C) 1999 AT&T Laboratories Cambridge.  All Rights Reserved.
 *
 *  This is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This software is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this software; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
 *  USA.
 */

/*
 * rre.c - handle RRE encoding.
 *
 * This file shouldn't be compiled directly.  It is included multiple times by
 * rfbproto.c, each time with a different definition of the macro BPP.  For
 * each value of BPP, this file defines a function which handles an RRE
 * encoded rectangle with BPP bits per pixel.
 */

#define HandleRREBPP CONCAT2E(HandleRRE,BPP)
#define UintBPP CONCAT2E(Uint,BPP)

static SDL_bool
HandleRREBPP (int rx, int ry, int rw, int rh)
{
    SDL_Rect rect;
    rfbRREHeader hdr;
    Uint32 fgColour;
    int i;
    UintBPP pix;
    rfbRectangle subrect;

    if (!ReadFromRFBServer((char *)&hdr, sz_rfbRREHeader))
        return SDL_FALSE;

    hdr.nSubrects = Swap32IfLE(hdr.nSubrects);

    if (!ReadFromRFBServer((char *)&pix, sizeof(pix)))
        return SDL_FALSE;

#if (BPP == 8)
    fgColour = (appData.useBGR233 ? BGR233ToPixel[pix] : pix);
#else
    fgColour = pix;
#endif

    rect.x = rx;
    rect.y = ry;
    rect.w = rw;
    rect.h = rh;
    SDL_FillRect(ovl, &rect, fgColour);
    
    for (i = 0; i < hdr.nSubrects; i++) {
        if (!ReadFromRFBServer((char *)&pix, sizeof(pix)))
            return SDL_FALSE;

        if (!ReadFromRFBServer((char *)&subrect, sz_rfbRectangle))
            return SDL_FALSE;

        subrect.x = Swap16IfLE(subrect.x);
        subrect.y = Swap16IfLE(subrect.y);
        subrect.w = Swap16IfLE(subrect.w);
        subrect.h = Swap16IfLE(subrect.h);

#if (BPP == 8)
        fgColour = (appData.useBGR233 ? BGR233ToPixel[pix] : pix);
#else
        fgColour = pix;
#endif

        rect.x = rx + subrect.x;
        rect.y = ry + subrect.y;
        rect.w = subrect.w;
        rect.h = subrect.h;
        SDL_FillRect(ovl, &rect, fgColour);
    }

    return SDL_TRUE;
}
