/*
 *  Copyright (C) 1999 AT&T Laboratories Cambridge.  All Rights Reserved.
 *
 *  This is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This software is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this software; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
 *  USA.
 */

/*
 * listen.c - listen for incoming connections
 */

#include <unistd.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/wait.h>
#include <sys/time.h>
#include <sys/utsname.h>
#include <vncviewer.h>
#include "vnes.h"

/* Timeout after 10 seconds have elapsed */
#define LISTEN_TIMEOUT      10


/*
 * listenForIncomingConnections() - listen for incoming connections from
 * servers, and fork a new process to deal with each connection.  We must do
 * all this before invoking any Xt functions - this is because Xt doesn't
 * cope with forking very well.
 */
void
listenForIncomingConnections(int *argc, char **argv)
{
    int listenSocket;
    int recvBufSize = 262144; 
#if 0
    int recvBufSizeLen = 0; 
#endif
    fd_set fds;

    listenSocket = ListenAtTcpPort(LISTEN_PORT_OFFSET);
    DebugPrint("%s -listen: Listening on TCP port %d\n",
               programName, LISTEN_PORT_OFFSET);

    if (listenSocket < 0) Cleanup(ERROR_INIT);

    while (SDL_TRUE) {
        struct timeval tm;
        int ret;
        SDL_Event event;
        int status, pid;
        vnes_msg_t vnesmsg;

        DebugPrint("VNC: Waiting for connection...\n");

        /* reap any zombies */
        while ((pid= wait3(&status, WNOHANG, (struct rusage *)0))>0);

        /* Send the ready message to the controlling daemon */
        memset(&vnesmsg, 0, sizeof(vnesmsg));
        vnesmsg.type = VNES_MSG_VNC_INITIALIZED;
        if (SendDomainMessage(&vnesmsg) == 0) {
            // SendDomainMessage failed so exit with error code 1
            Cleanup(ERROR_INIT);
        }

        tm.tv_sec = LISTEN_TIMEOUT;
        tm.tv_usec = 0;
        FD_ZERO(&fds);
        FD_SET(listenSocket, &fds);
#ifdef NOUSE_LISTEN_TIMEOUT
        ret = select(FD_SETSIZE, &fds, NULL, NULL, NULL);
#else
        ret = select(FD_SETSIZE, &fds, NULL, NULL, &tm);
#endif
        if ( ret <= 0 ) {
            close(listenSocket);

            if ( ret == -1 ) {
                perror("VNC listen");

            }
            Cleanup(ERROR_INIT);
        }

        // initialize the socket descriptors
        rfbsockTCP = -1;

        /* discard any events */
        while (SDL_PollEvent(&event)) {};

        if (FD_ISSET(listenSocket, &fds)) {
            rfbsockTCP = AcceptTcpConnection(listenSocket);
            if (rfbsockTCP < 0) {
                Cleanup(ERROR_INIT);
            }

            if (!SetNonBlocking(rfbsockTCP)) {
                Cleanup(ERROR_INIT);
            }

            setsockopt(rfbsockTCP, SOL_SOCKET, SO_RCVBUF,
                 &recvBufSize, sizeof(recvBufSize));

#if 0
            recvBufSize = 0;
            recvBufSizeLen = sizeof(recvBufSize);
            getsockopt(rfbsockTCP, SOL_SOCKET, SO_RCVBUF,
                 &recvBufSize, &recvBufSizeLen);

            printf("setting receive buffer to %d\n", recvBufSize);
#endif

            // Close the TCP listening socket
            close(listenSocket);
            return;
        }
    }
}
