/*
 *  Copyright (C) 1999 AT&T Laboratories Cambridge.  All Rights Reserved.
 *
 *  This is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This software is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this software; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
 *  USA.
 */

/*
 * vncviewer.c - the Xt-based VNC viewer.
 */

#include <unistd.h>
#include <stdlib.h>
#include <signal.h>
#include "imconfig.h"
#include "SDL_e21.h"
#include "vnes.h"
#include "vncviewer.h"

char *programName;

int
main(int argc, char **argv)
{
    int flags;
    sigset_t unblock, saved;
#ifdef UI_TEST
    Uint32 colour;
    SDL_Event event;
#endif

    sigemptyset(&unblock);
    sigaddset(&unblock, SIGQUIT);
    sigaddset(&unblock, SIGTERM);
    sigaddset(&unblock, SIGHUP);
    sigprocmask(SIG_UNBLOCK, &unblock, &saved);

    dpy = NULL;
	ovl = NULL;

    programName = argv[0];

	setenv("SDL_VIDEO_YUV_DIRECT",  "1", 1);
	setenv("SDL_VIDEO_YUV_HWACCEL", "1", 1);
	setenv("SDL_ACL_RGB_SURFACE",  "g0", 1);
	setenv("SDL_ACL_YUV_OVERLAY",  "v0", 1);

    /* Start the domain socket */
    if (ConnectDomainSocket()) {
        Cleanup(ERROR_INIT);
    }

    /* Initialize the SDL video and event subsystems */
    sigprocmask(SIG_SETMASK, &saved, NULL);
    flags = SDL_INIT_VIDEO;
    SDL_Init(flags);
    sigprocmask(SIG_UNBLOCK, &unblock, NULL);


#ifdef USE_ESCVP21
    // initialize libSDLescvp.so 
    if(SDL_E21Init() < 0){
        DebugPrint( "Could not initialize ESCVP21\n");
        Cleanup(ERROR_INIT);
    }
#endif //USE_ESCVP21

    /* Wait for an incoming connection.  This is the only mode that is
       supported by the projector */
    listenForIncomingConnections(&argc, argv);


	// Set the default parameters
	SetAppDataDefaults();

	// Poll the domain socket once to get the config parameters
	PollDomainSocket();

    /* Initialise the VNC connection, including reading the
       password */
    if (!InitialiseRFBConnection()) {
        Cleanup(ERROR_INIT);
    }

    /* Create the "desktop" widget, and perform initialisation which
       needs doing before the widgets are realized */

    ToplevelInitBeforeRealization();

    DesktopInitBeforeRealization();

    /* Find the best pixel format and X visual/colormap to use */
    SetVisualAndCmap();
	DebugPrint("SetVisualAndCmap finished\n");

    /* Perform initialisation that needs doing after realization, now
       that the X windows exist */
    ToplevelInitAfterRealization();
	DebugPrint("ToplevelInitAfterRealization finished\n");

    DesktopInitAfterRealization();
	DebugPrint("DesktopInitAfterRealization finished\n");

    /* Tell the VNC server which pixel format and encodings we want to
       use */
    SetFormatAndEncodings();
	DebugPrint("SetFormatAndEncodings finished\n");

    /* Now enter the main loop, processing VNC messages.  X events
       will automatically be processed whenever the VNC connection is
       idle. */
	DebugPrint("starting VNC message loop...\n");
    while ( ! appData.isFinished ) {
        if (!HandleRFBServerMessage()) {
			DebugPrint("codec error... exiting\n");
			Cleanup(ERROR_CODEC);
		}
    }

    Cleanup(EXIT_SUCCESS);
    return 0;
}
