/*
 *  Copyright (C) 2000, 2001 Const Kaplinsky.  All Rights Reserved.
 *  Copyright (C) 2000 Tridia Corporation.  All Rights Reserved.
 *  Copyright (C) 1999 AT&T Laboratories Cambridge.  All Rights Reserved.
 *
 *  This is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This software is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this software; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
 *  USA.
 */

/*
 * vncviewer.h
 */

#ifndef VNCVIEWER_H
#define VNCVIEWER_H

#ifdef __cplusplus
extern "C" {
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/time.h>
#include <unistd.h>
#include <SDL.h>
#include "rfbproto.h"
#include "exitcode.h"
#include "debug.h"      // for DebugPrint()


extern int endianTest;

#define Swap16IfLE(s) \
    (*(char *)&endianTest ? ((((s) & 0xff) << 8) | (((s) >> 8) & 0xff)) : (s))

#define Swap32IfLE(l) \
    (*(char *)&endianTest ? ((((l) & 0xff000000) >> 24) | \
			     (((l) & 0x00ff0000) >> 8)  | \
			     (((l) & 0x0000ff00) << 8)  | \
			     (((l) & 0x000000ff) << 24))  : (l))

#define MAX_ENCODINGS 20

#define FLASH_PORT_OFFSET 5400
#define LISTEN_PORT_OFFSET 3621     //!< IANA Network Screen Port
#define TUNNEL_PORT_OFFSET 5500
#define SERVER_PORT_OFFSET 5900

// To enable client->server communication define CAN_SEND = 1
#define CAN_SEND 0

typedef char* String;

/* argsresources.c */
typedef struct {
    SDL_bool shareDesktop;
    SDL_bool viewOnly;
    SDL_bool fullScreen;
    SDL_bool raiseOnBeep;

    String encodingsString;

    SDL_bool useBGR233;
    int nColours;
    SDL_bool useSharedColours;
    SDL_bool forceOwnCmap;
    SDL_bool forceTrueColour;
    int requestedDepth;

    SDL_bool useShm;

    int wmDecorationWidth;
    int wmDecorationHeight;

    char *passwordFile;
    SDL_bool passwordDialog;

    int rawDelay;
    int copyRectDelay;

    SDL_bool debug;

    int bumpScrollTime;
    int bumpScrollPixels;

    int compressLevel;
    int qualityLevel;
    SDL_bool useRemoteCursor;

	int useEncryptionType;
	int encBlockSize;
    String encryptString;
    char serverIP[4];

    // Projector specific parameters
    char confHostname[32];
    char confAddress[32];
    char confEssid[32];
    char confChannel[8];
    char confKeyword[8];
    int  confNetworkMode;

    SDL_Rect zoomRect;
    SDL_bool useZoomRect;

    SDL_bool isFinished;

} AppData;

extern AppData appData;

extern char *fallback_resources[];
extern char vncServerHost[];
extern int vncServerPort;

/* argsresources.c */
extern void SetAppDataDefaults();


/* colour.c */

extern Uint32 BGR233ToPixel[];
extern SDL_Palette cmap;

extern void SetVisualAndCmap();

/* cursor.c */

extern SDL_bool HandleXCursor(int xhot, int yhot, int width, int height);
extern SDL_bool HandleRichCursor(int xhot, int yhot, int width, int height);
extern void SoftCursorLockArea(int x, int y, int w, int h);
extern void SoftCursorUnlockScreen(void);
extern void SoftCursorMove(int x, int y);

/* desktop.c */

extern SDL_Surface* dpy;
extern SDL_Surface* ovl;

extern void DesktopInitBeforeRealization();
extern void DesktopInitAfterRealization();
extern void SendRFBEvent(SDL_Event* event, String *params, int *num_params);
extern void HandleBasicDesktopEvent(SDL_Event* event);
extern void CopyDataToScreen(char *buf, int x, int y, int width, int height);
extern void UpdateOverlayRect(SDL_Surface* ovl, Sint16 x, Sint16 y, Uint16 w, Uint16 h);
extern void ZoomOverlay(SDL_Rect* zoomrect);
extern void CleanupVideoSurfaces();

/* fullscreen.c */

extern void ToggleFullScreen();
extern void SetFullScreenState();
extern SDL_bool BumpScroll(SDL_Event* ev);
extern void FullScreenOn();
extern void FullScreenOff();

/* listen.c */

extern void listenForIncomingConnections(int *argc, char **argv);

/* misc.c */

extern void ToplevelInitBeforeRealization();
extern void ToplevelInitAfterRealization();
extern void Cleanup(int code);


/* ui.cpp */
extern void InitUI();
extern void DestroyUI();
extern void PollDialog(const SDL_Event* event);
extern void CloseDialog();
extern void SetVideoArea(SDL_Rect* area);


/* rfbproto.c */

extern int rfbsockTCP;

extern SDL_bool canUseCoRRE;
extern SDL_bool canUseHextile;
extern char *desktopName;
extern rfbPixelFormat myFormat;
extern rfbServerInitMsg si;
extern char *serverCutText;
extern SDL_bool newServerCutText;

extern SDL_bool ConnectToRFBServer(const char *hostname, int port);
extern SDL_bool InitialiseRFBConnection();
extern SDL_bool SetFormatAndEncodings();
extern SDL_bool SendIncrementalFramebufferUpdateRequest();
extern SDL_bool SendFramebufferUpdateRequest(int x, int y, int w, int h,
					 SDL_bool incremental);
extern SDL_bool SendPointerEvent(int x, int y, int buttonMask);
extern SDL_bool SendKeyEvent(Uint32 key, SDL_bool down);
extern SDL_bool SendClientCutText(char *str, int len);
extern SDL_bool HandleRFBServerMessage();

extern void PrintPixelFormat(rfbPixelFormat *format);


/* sockets.c */

extern SDL_bool errorMessageOnReadFailure;

#ifdef BENCHMARK
extern unsigned int decryptTime;
extern unsigned int decryptBytes;
extern unsigned int encryptTime;
extern unsigned int encryptBytes;
#endif

extern SDL_bool ReadFromRFBServer(char *out, unsigned int n);
extern SDL_bool WriteToRFBServer( char *buf, unsigned int n);
extern int FindFreeTcpPort(void);

extern int ListenAtTcpPort(int port);

extern int ConnectToTcpAddr(unsigned int host, int port);
extern int AcceptTcpConnection(int listenSock);
extern SDL_bool SetNonBlocking(int sock);

extern SDL_bool StringToIPAddr(const char *str, unsigned int *addr);
extern SDL_bool SameMachine(int sock);
extern int ReadVnesHeader();

/* vncviewer.c */

extern char *programName;

#ifdef __cplusplus
}
#endif

#endif // VNCVIEWER_H
